/*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
#+
#+     Internet Radio Automation & Encoding Toolkit
#+
#+     Copyright (C) 2018 by Kevin C. O'Kane
#+
#+     Kevin C. O'Kane
#+     kc.okane@gmail.com
#+     okane@cs.uni.edu
#+     https://www.cs.uni.edu/~okane
#+     http://threadsafebooks.com/
#+
#+ This program is free software; you can redistribute it and/or modify
#+ it under the terms of the GNU General Public License as published by
#+ the Free Software Foundation; either version 2 of the License, or
#+ (at your option) any later version.
#+
#+ This program is distributed in the hope that it will be useful,
#+ but WITHOUT ANY WARRANTY; without even the implied warranty of
#+ MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#+ GNU General Public License for more details.
#+
#+ You should have received a copy of the GNU General Public License
#+ along with this program; if not, write to the Free Software
#+ Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#+
#++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/

// February 20, 2019

#include <stdlib.h>
#include <sys/types.h>
#include <signal.h>
#include <unistd.h>
#include <string.h>
#include <gtk/gtk.h>
#include <sys/mman.h>
#include "settings.h"
#include <math.h>
#include <ctype.h>
#include <time.h>

#define PULSE 1
#define JACK 0

//***********************************************
// function headers
//***********************************************

void	css_set(GtkCssProvider * cssProvider, GtkWidget *g_widget);
void	strip(char* str);
void	clearButton(GtkButton *);
void	setButton(GtkButton *);
void	stream(int strm, GtkButton *);
void	Message(char *);
int	ClearFullButtons();
void	FullPanelCommon();
void	normalSlides();
void	fullSlides();
void	hidePics();
void	panel_view();
void	fullView();
void	backImage(int i);
void	killSlides();

int LOOPBACK=1; // default audio method - see automation.config

//------------------------------------------
// global GTK widgets for buttons and things
//------------------------------------------

GtkWidget *g_btn_play;
GtkWidget *g_btn_play2;
GtkWidget *g_btn_play4;
GtkWidget *Player;
GtkWidget *Deploy;
GtkWidget *Deploy2;
GtkWidget *TopLabel;
GtkWidget *BotLabel;
GtkWidget *HorLine;
GtkWidget *Banner;
GtkWidget *Record;
GtkWidget *clrPgm;
GtkWidget *Camera0;
GtkWidget *Camera1;
GtkWidget *SlidePause;
GtkWidget *g_lbl_hello1;
GtkWidget *g_radiobutton1;
GtkWidget *g_radiobutton2;
GtkWidget *g_radiobutton3;
GtkWidget *g_radiobutton4;
GtkWidget *g_radiobutton5;
GtkWidget *g_radiobutton6;
GtkWidget *g_radiobutton7;
GtkWidget *g_radiobutton8;
GtkWidget *g_radiobutton9;
GtkWidget *g_radiobutton10;
GtkWidget *g_radiobutton11;
GtkWidget *g_radiobutton12;
GtkWidget *g_radiobutton13;
GtkWidget *g_radiobutton14;
GtkWidget *g_radiobutton15;
GtkWidget *g_radiobutton16;
GtkWidget *g_radiobutton17;
GtkWidget *g_radiobutton18;
GtkWidget *g_radiobutton19;
GtkWidget *g_radiobutton20;
GtkWidget *g_radiobutton21;
GtkWidget *g_radiobutton22;
GtkWidget *g_radiobutton23;
GtkWidget *g_radiobutton24;
GtkWidget *g_scale1;
GtkWidget *g_label1;
GtkWidget *g_button27;
GtkWidget *g_button29;
GtkWidget *g_button38;
GtkWidget *g_button40;
GtkWidget *g_button52;
GtkWidget *g_button60;
GtkWidget *g_image1;
GtkWidget *g_image8;
GtkWidget *g_image9;
GtkWidget *g_image10;
GtkWidget *g_image200;
GtkWidget *g_cont;
GtkEntry *g_lbl_entry7;
GtkEntry *g_lbl_entry8;
GtkWidget *ScrolledWindow1;
GtkWidget *image;
GdkColor color;
GtkBuilder *builder; 
GtkWidget  *window;
GtkWidget  *fixed1;

gint x,y;

static int image2=0;
static int AudioLevel=0x10000;
static int ScaleLevel=0;

// flags to timer telling it to clear the associated buttons

int b2=0;
int b46=0;
int b31=0;
int b33=0;
int b56=0;
int b18=0;
int b35=0;
int b58=0;
int b59=0;
int b60=0;
int b37=0;
int b57=0;
int b36=0;
int b25=0;
int b43=0;
int b28=0;
int b49=0;
int b16=0;
int b48=0;
int b47=0;
int b50=0;
int b53=0;
int b24=0;
int b26=0;
int b39=0;
int b41=0;
int b44=0;
int b45=0;
int b51=0;
int b42=0;

int PLAYER_HOR = 12;
int PLAYER_VER = 40;

#define SSHOR 1350
#define SSVER 200
#define SSWID 400
#define SSHGH 300

//***********************************************
// window size request - this is changed later
// depending on the layout
//***********************************************

int MAIN_WIND_WIDTH=0;
int MAIN_WIND_HEIGHT=0;
int MAIN_WIND_HOR=0;
int MAIN_WIND_VER=0;

//	default casting window same as main window

int STREAM_WIDE = 0;
int STREAM_HIGH = 0;
int STREAM_HOR = 0;
int STREAM_VER = 0;

//*******************************************
// default visual placement
//*******************************************

#define WID 640
#define HGT 360
#define HOR 930
#define VER 160

//***********************************************
// cameras
//***********************************************

int CAM0_DELAY=3; // this can be set in automation.config
int CAM1_DELAY=3; // this can be set in automation.config

//***********************************************
// communication with timer
// some code runs as child proceses and their
// data is not otherwise visible to the timer
// function.
// this is a structure of pointers to data that
// is in shared memory that can be seen by the
// timer and the child processes.
// pointer values to be filled in below.
//***********************************************

typedef struct {
	GtkWidget *w_lbl_stream;
	} app_widgets;

	app_widgets *widgets;

//***********************************************
// definition of timer function. it is invoked
// once per secon by GTK
//***********************************************

gboolean timer_handler(app_widgets *widgets);

static char *stream_output;
static int  *audio;
static int  *decorate;
static int  *skip;

// static char layout[128];

int  scount = 0;

int  *Pause;

int  panelView = 1;	// are we in panel casting mode?
int  fullScreen = 0;	// are we in full screen casting mode?

int  streamOn = 0;
int  vlcDir = 0;
int  slideShow = 0;
int  audsys = PULSE;

char MUSIC_HOME_DIR[1024];
char CODE_HOME[1024];		

char directories[5][1024] = {0};
int  lastDir = -1;

//-------------------------------------------------------------------------------------------
//	css codes. note back slash characters.
//-------------------------------------------------------------------------------------------

char css0[2048];	// read from file

//-----------------------------------------------------------------------------------------

// used to set button text color to red

char css1[1024] = "GtkButton, button, label {	\
      color: red;	\
      border-color: shade (mix (rgb (34, 34, 120), #fff, 0.5), 0.1);	\
      }";

//-------------------------------------------------------------
// chyrons
//-------------------------------------------------------------

#define CHYRONS 1

struct CHY {

	int active;

	char file[512];
	int hor;
	int ver;

	} chyron[CHYRONS];

#define BACKGROUNDS 3000

int backCount=0;
int SLIDE_TIME=10;

struct backgrounds {

//	char file[512];
	char *file;
	char name[16];
//	GtkWidget *image;
	int hor;
	int ver;
	int valid;
	} backs[BACKGROUNDS];

// ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// main main main main main main main main main main main main main
// ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

int main(int argc, char *argv[]) {

	char cmd[4096],tmp[4096];
	FILE *s1,*s2;
	int i,j,k,m,n;

	srand(time(0));

//----------------------
//	css definitions
//----------------------

	strcpy (css0, "GtkModelButton, GtkButton {    \
      		color: blue; \
      		padding-left: 2px; \
      		padding-right: 2px; \
      		border-color: shade (mix (rgb (34, 34, 120), #fff, 0.5), 0.1); \
      		}  \
		button {     \
      		color: blue; \
      		padding-left: 2px; \
      		padding-right: 2px; \
      		border-color: shade (mix (rgb (34, 34, 120), #fff, 0.5), 0.1); \
      		} ");

//---------------------------------------------------------------------
//	establish home directories
//---------------------------------------------------------------------

	strcpy(MUSIC_HOME_DIR, getenv("HOME"));
	strcat(MUSIC_HOME_DIR, "/Desktop/Broadcast");
	printf("**** Music Home Directory: %s\n", MUSIC_HOME_DIR);

        readlink("/proc/self/exe", CODE_HOME, 1024);

        for (i = strlen(CODE_HOME); i > 0; i--)
                if (CODE_HOME[i] == '/' ) {
                        CODE_HOME[i] = 0;
                        break;
                        }
        if (i == 0) {
                printf("Directory error finding CODE_HOME\n");
                return EXIT_FAILURE;
                }

	printf("**** Code Home Directory: %s\n", CODE_HOME);

//---------------------------------------------------------------------
//	allocate shared memory used to communicate with the timer function
//	from detached child processes.
//---------------------------------------------------------------------

	stream_output = mmap(NULL, 512, PROT_READ | PROT_WRITE, 
		MAP_SHARED | MAP_ANONYMOUS, -1, 0); // shred memory allocation

	audio = mmap(NULL, 16, PROT_READ | PROT_WRITE, 
		MAP_SHARED | MAP_ANONYMOUS, -1, 0); // shred memory allocation

	skip = mmap(NULL, 16, PROT_READ | PROT_WRITE, 
		MAP_SHARED | MAP_ANONYMOUS, -1, 0); // shred memory allocation

	decorate = mmap(NULL, 16, PROT_READ | PROT_WRITE, 
		MAP_SHARED | MAP_ANONYMOUS, -1, 0); // shred memory allocation

	Pause = mmap(NULL, 16, PROT_READ | PROT_WRITE, 
		MAP_SHARED | MAP_ANONYMOUS, -1, 0); // shred memory allocation

	*Pause=0;
	vlcDir=0;
	*audio=0; // flag to say if player is runing an audio only
	*skip=0;  // skip next 
	*decorate=1;

	for (int i=0; i<CHYRONS; i++) {

		chyron[i].file[0]=0;
		chyron[i].active=0;
		chyron[i].hor=0;
		chyron[i].hor=0;
		}

	for (int i=0; i<BACKGROUNDS; i++) {

		backs[i].file = NULL;
		backs[i].hor=0;
		backs[i].ver=0;
		backs[i].valid=0;
		backs[i].name[0]=0;
		sprintf(backs[i].name,"%3d", i);
		}

	strcpy(backs[0].name, "blnk");



// -------------------------------------------------------------------------------
// cleanup any running conflicts
// -------------------------------------------------------------------------------

	strcpy(stream_output,"Stream inactive");	// initialization

        system("killall --signal SIGTERM vlc 2> /dev/null");
	system("wmctrl -c animation");
        system("killall --signal SIGTERM mpv 2> /dev/null");
        system("killall --signal SIGTERM phototonic 2> /dev/null");
	system("killall --signal SIGTERM gst-launch-1.0 2> /dev/null");
        system("killall --signal SIGTERM pqiv 2> /dev/null");
        system("killall --signal SIGHUP ffmpeg 2>/dev/null 1>/dev/null ");

	char word[64],sval[128];;
	int val;

// -------------------------------------------------------------------------------
// configuration file input
// -------------------------------------------------------------------------------

	s1=fopen("automation.config","r");

	if (s1!=NULL) {
		while (fgets(cmd,1024,s1)!=NULL) { // read line of automation.config
			cmd[strlen(cmd)-1]=0;      // remove newline
			if (cmd[0]=='#') continue; // comment - ignore

			if (strlen(cmd)==0) continue; // empty line

			sscanf(cmd,"%s %d",word, &val);

			printf("Config -> %s\n",cmd);

			if (strcmp(word,"panelView")==0) {
				if (val==1) {
					panel_view(); 
					panelView=val;
					setButton(GTK_BUTTON(g_button38));
					}
				else { 
					fullScreen=1;
					panelView=0;
					fullView();
					setButton(GTK_BUTTON(g_button29));
					}
				continue;
				}
			else if (strcmp(word,"CAM0_DELAY")==0) CAM0_DELAY=val;
			else if (strcmp(word,"CAM1_DELAY")==0) CAM1_DELAY=val;
			else if (strcmp(word,"LOOPBACK")==0) LOOPBACK=val;
			else if (strcmp(word,"SLIDE_TIME")==0) SLIDE_TIME=val;
			else if (strcmp(word,"PLAYER_HOR")==0) PLAYER_HOR=val;
			else if (strcmp(word,"PLAYER_VER")==0) PLAYER_VER=val;
			else if (strcmp(word,"BACKGROUND")==0) {

				if (val<0 || val >= BACKGROUNDS) {
					printf("*** background number out of range\n");
					return EXIT_FAILURE;
					}

				char dummy[32];
				char file[4096];

				sscanf(cmd,"%s %s %s %s %d %d", dummy, dummy, backs[val].name , file, 
					&backs[val].hor, &backs[val].ver ); // SCENE X word

				printf("*** Background %d name=%s file=%s hor=%d ver=%d\n",
					val, backs[val].name, file, backs[val].hor, backs[val].ver);

				backs[val].file = 
					malloc (strlen(file)+strlen(CODE_HOME)+1); // allocate file name

				if (file[0] != '*' )  {
					strcpy(backs[val].file, CODE_HOME); // must be AutoPlay directory
					strcat(backs[val].file, file);
					}

				else {
					if (strcmp( backs[val].name, "disco") == 0 ) {
						strcat (file, " -speed .2 -no-wander");
						}

					strcpy(backs[val].file, file);
					strcat(backs[val].file, " &");
					}

				backs[val].valid = 1;

				}

			}
		}

	backCount = 25;
	s1=fopen("slides.config","r");

	if (s1!=NULL) { // read slide show file references
		while (fgets(cmd,2048,s1)!=NULL) { // read line of automation.config
			if (cmd[0] == '#' ) continue; //comment
			cmd[strlen(cmd)-1]=0;      // remove newline
			backs[backCount].file = malloc(strlen(cmd)+1);
			strcpy(backs[backCount].file, cmd);
			backs[backCount].hor = 610;
			backs[backCount].ver = 0;
			backs[backCount].valid = 1;
			backCount++;
			if (backCount >= BACKGROUNDS) break;
			}
		}

	MAIN_WIND_HOR = 0;
	MAIN_WIND_VER = 0;

	widgets = g_slice_new(app_widgets);

        for (i = 0; i< argc; i++) {
                if (strcmp(argv[i], "PULSE") == 0 ) audsys = PULSE;
                if (strcmp(argv[i], "JACK") == 0 ) audsys = JACK;
                }

	gtk_init(&argc, &argv);

//---------------------------------------------------------------------
// establish contact with CSS code used to adjust widget settings
//---------------------------------------------------------------------
 
	builder = gtk_builder_new_from_file ("./Automation.glade");
 
	window = GTK_WIDGET(gtk_builder_get_object(builder, "window_main"));

//	gtk_window_set_default_size (GTK_WINDOW(window), 700, MAIN_WIND_HEIGHT);

//---------------------------------------------------------------------
//	volume control code - not currently active
//---------------------------------------------------------------------

	gtk_builder_connect_signals(builder, NULL);

	g_signal_connect(window, "destroy", G_CALLBACK(gtk_main_quit), NULL);

//---------------------------------------------------------------------
// get the addresses of the major components
//---------------------------------------------------------------------

	fixed1		= GTK_WIDGET(gtk_builder_get_object(builder, "fixed1"));
	g_label1	= GTK_WIDGET(gtk_builder_get_object(builder, "label1"));
	g_image1	= GTK_WIDGET(gtk_builder_get_object(builder, "image1"));
	g_image8	= GTK_WIDGET(gtk_builder_get_object(builder, "image8"));
	g_image9	= GTK_WIDGET(gtk_builder_get_object(builder, "image9"));
	g_image10	= GTK_WIDGET(gtk_builder_get_object(builder, "image10"));
	g_image200	= GTK_WIDGET(gtk_builder_get_object(builder, "image200"));
	Player		= GTK_WIDGET(gtk_builder_get_object(builder, "Player"));
	Deploy		= GTK_WIDGET(gtk_builder_get_object(builder, "Deploy"));
	Deploy2		= GTK_WIDGET(gtk_builder_get_object(builder, "Deploy2"));
	BotLabel	= GTK_WIDGET(gtk_builder_get_object(builder, "BotLabel"));
	HorLine		= GTK_WIDGET(gtk_builder_get_object(builder, "HorLine"));
	TopLabel	= GTK_WIDGET(gtk_builder_get_object(builder, "TopLabel"));
	Banner		= GTK_WIDGET(gtk_builder_get_object(builder, "Banner"));
	Record		= GTK_WIDGET(gtk_builder_get_object(builder, "Record"));
	clrPgm		= GTK_WIDGET(gtk_builder_get_object(builder, "clrPgm"));
	SlidePause	= GTK_WIDGET(gtk_builder_get_object(builder, "SlidePause"));
	Camera0		= GTK_WIDGET(gtk_builder_get_object(builder, "Camera0"));
	Camera1		= GTK_WIDGET(gtk_builder_get_object(builder, "Camera1"));
        g_lbl_hello1    = GTK_WIDGET(gtk_builder_get_object(builder, "lbl_hello1"));
	g_radiobutton1	= GTK_WIDGET(gtk_builder_get_object(builder, "radiobutton1"));
	g_radiobutton2	= GTK_WIDGET(gtk_builder_get_object(builder, "radiobutton2"));
	g_radiobutton3	= GTK_WIDGET(gtk_builder_get_object(builder, "radiobutton3"));
	g_radiobutton4	= GTK_WIDGET(gtk_builder_get_object(builder, "radiobutton4"));
	g_radiobutton5	= GTK_WIDGET(gtk_builder_get_object(builder, "radiobutton5"));
	g_radiobutton6	= GTK_WIDGET(gtk_builder_get_object(builder, "radiobutton6"));
	g_radiobutton7	= GTK_WIDGET(gtk_builder_get_object(builder, "radiobutton7"));
	g_radiobutton8	= GTK_WIDGET(gtk_builder_get_object(builder, "radiobutton8"));
	g_radiobutton9	= GTK_WIDGET(gtk_builder_get_object(builder, "radiobutton9"));
	g_radiobutton10	= GTK_WIDGET(gtk_builder_get_object(builder, "radiobutton10"));
	g_radiobutton11	= GTK_WIDGET(gtk_builder_get_object(builder, "radiobutton11"));
	g_radiobutton12	= GTK_WIDGET(gtk_builder_get_object(builder, "radiobutton12"));
	g_radiobutton13	= GTK_WIDGET(gtk_builder_get_object(builder, "radiobutton13"));
	g_radiobutton14	= GTK_WIDGET(gtk_builder_get_object(builder, "radiobutton14"));
	g_radiobutton15	= GTK_WIDGET(gtk_builder_get_object(builder, "radiobutton15"));
	g_radiobutton16	= GTK_WIDGET(gtk_builder_get_object(builder, "radiobutton16"));
	g_radiobutton17	= GTK_WIDGET(gtk_builder_get_object(builder, "radiobutton17"));
	g_radiobutton18	= GTK_WIDGET(gtk_builder_get_object(builder, "radiobutton18"));
	g_radiobutton19	= GTK_WIDGET(gtk_builder_get_object(builder, "radiobutton19"));
	g_radiobutton20	= GTK_WIDGET(gtk_builder_get_object(builder, "radiobutton20"));
	g_radiobutton21	= GTK_WIDGET(gtk_builder_get_object(builder, "radiobutton21"));
	g_radiobutton22	= GTK_WIDGET(gtk_builder_get_object(builder, "radiobutton22"));
	g_radiobutton23	= GTK_WIDGET(gtk_builder_get_object(builder, "radiobutton23"));
	g_radiobutton24	= GTK_WIDGET(gtk_builder_get_object(builder, "radiobutton24"));
	g_button27	= GTK_WIDGET(gtk_builder_get_object(builder, "button27"));
	g_button29	= GTK_WIDGET(gtk_builder_get_object(builder, "button29"));
	g_button38	= GTK_WIDGET(gtk_builder_get_object(builder, "button38"));
	g_button40	= GTK_WIDGET(gtk_builder_get_object(builder, "button40"));
	g_button52	= GTK_WIDGET(gtk_builder_get_object(builder, "button52"));
	g_button60	= GTK_WIDGET(gtk_builder_get_object(builder, "button60"));

	g_lbl_entry7	= GTK_ENTRY(gtk_builder_get_object(builder, "entry7"));
	g_lbl_entry8	= GTK_ENTRY(gtk_builder_get_object(builder, "entry8"));

	g_btn_play	= GTK_WIDGET(gtk_builder_get_object(builder, "button1"));

        widgets -> w_lbl_stream = g_lbl_hello1;

	// make these button pointers available to the timer function


//---------------------------------------------------------------------
//	set css styles onto objects
//	this applies the CSS file loaded above to the components named.
//	there is probably a better way to do this.
//---------------------------------------------------------------------

	GtkCssProvider * cssProvider;
	cssProvider  = gtk_css_provider_new();
	gtk_css_provider_load_from_data(cssProvider, css0, -1, NULL);

	css_set(cssProvider, g_button27);
	css_set(cssProvider, g_button29);
	css_set(cssProvider, g_button38);
	css_set(cssProvider, g_button40);
	css_set(cssProvider, g_button52);
	css_set(cssProvider, g_button60);
	css_set(cssProvider, Player);
	css_set(cssProvider, Deploy);
	css_set(cssProvider, Deploy2);
	css_set(cssProvider, Banner);
	css_set(cssProvider, Record);
	css_set(cssProvider, clrPgm);
	css_set(cssProvider, SlidePause);
	css_set(cssProvider, Camera0);
	css_set(cssProvider, Camera1);

	ScrolledWindow1 = GTK_WIDGET(gtk_builder_get_object(builder, "scrolledwindow1"));

	g_object_unref(builder);

	g_timeout_add_seconds(1, (GSourceFunc) timer_handler, widgets);

//	g_timeout_add(100, (GSourceFunc) timer_handler, widgets);

	gtk_widget_show(window);                

//	gtk_window_move (GTK_WINDOW(window), 0, 0);

// launch automation

	usleep(500000);
	strcpy(cmd, CODE_HOME);
	if ( audsys == PULSE ) strcat(cmd, "/player.script");
	else strcat(cmd, "/player.script JACK");

	system(cmd);

	sleep(1);

	if (audsys == PULSE) {
		sprintf(cmd,"wmctrl -r \"Spark Gap Radio Player\" -e 0,%d,%d,-1,-1", 
			PLAYER_HOR, PLAYER_VER);
		system(cmd);
		}
	else {
		sprintf(cmd,"wmctrl -r \"Spark Gap Radio Player\" -e 0,%d,%d,-1,-1", 
			4, 20);
		system(cmd);
		}

	system("wmctrl -r \"Spark Gap Radio Player\" -b add,above");
	sleep(1);

	strcpy(cmd, 
	"xprop -name \"Spark Gap Radio Player\" "
	"-f _MOTIF_WM_HINTS 32c -set _MOTIF_WM_HINTS \"0x2, 0x0, 0x0, 0x0, 0x0\"");
	system(cmd);

	system("killall --signal SIGTERM gst-launch-1.0 2> /dev/null");

	gtk_widget_hide(GTK_WIDGET(g_image8)); // 'on-air' image
	gtk_widget_hide(GTK_WIDGET(g_image10)); // 'record' image
	gtk_widget_show(GTK_WIDGET(g_image9)); // 'standby' image
	hidePics();
	gtk_widget_hide(GTK_WIDGET(g_image200)); // 'on-air' image

	if (panelView) {
		panel_view(); 
		panelView=1;
		fullScreen=0;
		setButton(GTK_BUTTON(g_button38));
		}
	else {
		fullScreen=1;
		panelView=0;
		fullView();
		setButton(GTK_BUTTON(g_button29));
		}

	system("killall --signal SIGTERM vlc 2> /dev/null");
        system("killall --signal SIGTERM mpv 2> /dev/null");

//---------------------------------------------------------------------
//	From here on, it's all callbacks
//---------------------------------------------------------------------

	GdkColor color;
        color.red = 0x3333;
        color.green = 0x3333;
        color.blue = 0x6666;
	gtk_widget_modify_bg(window, GTK_STATE_NORMAL, &color);

	GdkColor rcolor;
	gdk_color_parse ("#FFFF55", &rcolor);

	gtk_widget_modify_fg (g_radiobutton1, GTK_STATE_NORMAL, &rcolor);
	gtk_widget_modify_fg (g_radiobutton2, GTK_STATE_NORMAL, &rcolor);
	gtk_widget_modify_fg (g_radiobutton3, GTK_STATE_NORMAL, &rcolor);
	gtk_widget_modify_fg (g_radiobutton4, GTK_STATE_NORMAL, &rcolor);
	gtk_widget_modify_fg (g_radiobutton5, GTK_STATE_NORMAL, &rcolor);
	gtk_widget_modify_fg (g_radiobutton6, GTK_STATE_NORMAL, &rcolor);
	gtk_widget_modify_fg (g_radiobutton7, GTK_STATE_NORMAL, &rcolor);
	gtk_widget_modify_fg (g_radiobutton8, GTK_STATE_NORMAL, &rcolor);
	gtk_widget_modify_fg (g_radiobutton9, GTK_STATE_NORMAL, &rcolor);
	gtk_widget_modify_fg (g_radiobutton10, GTK_STATE_NORMAL, &rcolor);
	gtk_widget_modify_fg (g_radiobutton11, GTK_STATE_NORMAL, &rcolor);
	gtk_widget_modify_fg (g_radiobutton12, GTK_STATE_NORMAL, &rcolor);
	gtk_widget_modify_fg (g_radiobutton13, GTK_STATE_NORMAL, &rcolor);
	gtk_widget_modify_fg (g_radiobutton14, GTK_STATE_NORMAL, &rcolor);
	gtk_widget_modify_fg (g_radiobutton15, GTK_STATE_NORMAL, &rcolor);
	gtk_widget_modify_fg (g_radiobutton16, GTK_STATE_NORMAL, &rcolor);
	gtk_widget_modify_fg (g_radiobutton17, GTK_STATE_NORMAL, &rcolor);
	gtk_widget_modify_fg (g_radiobutton18, GTK_STATE_NORMAL, &rcolor);
	gtk_widget_modify_fg (g_radiobutton19, GTK_STATE_NORMAL, &rcolor);
	gtk_widget_modify_fg (g_radiobutton20, GTK_STATE_NORMAL, &rcolor);
	gtk_widget_modify_fg (g_radiobutton21, GTK_STATE_NORMAL, &rcolor);
	gtk_widget_modify_fg (g_radiobutton22, GTK_STATE_NORMAL, &rcolor);
	gtk_widget_modify_fg (g_radiobutton23, GTK_STATE_NORMAL, &rcolor);
	gtk_widget_modify_fg (g_radiobutton24, GTK_STATE_NORMAL, &rcolor);
	
// 	The following generate a lot of:
//	"GTK gtk_box_gadget_distribute: assertion 'size >= 0' failed..."
//	error messages which are meaningless.

	gtk_button_set_label (GTK_BUTTON(g_radiobutton6), backs[1].name);
	gtk_button_set_label (GTK_BUTTON(g_radiobutton2), backs[2].name);
	gtk_button_set_label (GTK_BUTTON(g_radiobutton7), backs[3].name);
	gtk_button_set_label (GTK_BUTTON(g_radiobutton5), backs[4].name);
	gtk_button_set_label (GTK_BUTTON(g_radiobutton4), backs[5].name);
	gtk_button_set_label (GTK_BUTTON(g_radiobutton3), backs[6].name);
	gtk_button_set_label (GTK_BUTTON(g_radiobutton8), backs[7].name);
	gtk_button_set_label (GTK_BUTTON(g_radiobutton9), backs[8].name);
	gtk_button_set_label (GTK_BUTTON(g_radiobutton11), backs[9].name);
	gtk_button_set_label (GTK_BUTTON(g_radiobutton12), backs[10].name);
	gtk_button_set_label (GTK_BUTTON(g_radiobutton10), backs[11].name);
	gtk_button_set_label (GTK_BUTTON(g_radiobutton13), backs[12].name);
	gtk_button_set_label (GTK_BUTTON(g_radiobutton14), backs[13].name);
	gtk_button_set_label (GTK_BUTTON(g_radiobutton15), backs[14].name);
	gtk_button_set_label (GTK_BUTTON(g_radiobutton16), backs[15].name);
	gtk_button_set_label (GTK_BUTTON(g_radiobutton17), backs[16].name);
	gtk_button_set_label (GTK_BUTTON(g_radiobutton18), backs[17].name);
	gtk_button_set_label (GTK_BUTTON(g_radiobutton19), backs[18].name);
	gtk_button_set_label (GTK_BUTTON(g_radiobutton20), backs[19].name);
	gtk_button_set_label (GTK_BUTTON(g_radiobutton21), backs[20].name);
	gtk_button_set_label (GTK_BUTTON(g_radiobutton22), backs[21].name);
	gtk_button_set_label (GTK_BUTTON(g_radiobutton23), backs[22].name);
	gtk_button_set_label (GTK_BUTTON(g_radiobutton24), backs[23].name);

	// make encoder full screen with no 'decorations' (title bar, etc)
	// comment out these lines if you don't want full screen

//	strcpy(cmd, 
//	"xprop -name \"Spark Gap Radio Encoder\" "
//	"-f _MOTIF_WM_HINTS 32c -set _MOTIF_WM_HINTS \"0x2, 0x0, 0x0, 0x0, 0x0\"");
//	system(cmd);

	gtk_widget_hide(g_image1);

	if (audsys == PULSE) {
		system("pavumeter --record &");
		sleep(1);
		system("wmctrl -r \"PulseAudio\" -e 0,1190,870,-1,-1");
		system("wmctrl -r \"PulseAudio\" -b add,above");
		}

	gtk_main();

	return EXIT_SUCCESS;
	}


// ----------------------------------------------------------------
// draw window signal arrives here
// ----------------------------------------------------------------

gboolean on_window_main_draw (GtkWidget *widget, GdkDragContext *context,
                gpointer user_data) {
        return FALSE;
        }


// ----------------------------------------------------------------
// Handler for the 1 second timer tick
// ----------------------------------------------------------------

gboolean timer_handler(app_widgets *widgets) {

	gint x,y;
	char cmd[1024];
	char cmd1[1024];
	int i,j;
	double lev;
	char tmp[1024];
	static int NET=0;
	static int back = 25;
	static int countdown = 0;

	if (countdown) countdown --;

	if (slideShow) {

		if ( countdown == 0 ) {

			srand(time(0));

			while (1) {
				back = rand() % backCount;
				if (back > 24) break; // lower limit
				}

			backImage(back);
			countdown = SLIDE_TIME;
			}
		}

	sprintf(cmd,"wmctrl -r \"Spark Gap Radio Encoder\" -e 0,%d,%d,-1,-1 ", 
		0, 0);
	system(cmd);

	if (!NET) {
		FILE *s1;
		strcpy(cmd,CODE_HOME);
		strcat(cmd,"/NetTmp2");
		s1=fopen(cmd,"r");
		if (s1 != NULL) {
			fgets(cmd,256,s1); // read and discard first 2 lines
			fgets(cmd,256,s1);
			strcpy(cmd,""); // needs to be empty
			fgets(cmd,256,s1);
			cmd[strlen(cmd)-1]=0; // remove new-line
			fclose(s1);
			strcpy(cmd1,CODE_HOME);
			strcat(cmd1,"/LoadTmp");
			s1=fopen(cmd1,"r");
			cmd1[0]=0;
			if (s1 != NULL) {
				fgets(cmd1,256,s1);	
				fclose(s1);
				for (int i=0; i< 20; i++) if (cmd1[i]==' ') {cmd1[i]=0; break; }
				}
			if (strlen(cmd)) { // get anything?
				strcpy(tmp,"In: ");
				strip(cmd);
				for (i=0; cmd[i] != ' ' && cmd[i] !=0; i++); // first field
				if (cmd[i]) { // not zero?
					cmd[i]=0;
					strcat(tmp,cmd);
					strcat(tmp,"; Out: ");
					strcat(tmp,&cmd[i+1]); // second field
					strcat(tmp,"; Ld: ");
					strcat(tmp,cmd1);
					gtk_label_set_text(GTK_LABEL(widgets->w_lbl_stream), tmp);    // update label
					}
				}
			}
		}
	NET++;
	if (NET>=3) NET = 0; // how frequently to update network times


//	gtk_window_get_position(GTK_WINDOW(window), 
//		&MAIN_WIND_HOR, &MAIN_WIND_VER);


	if (b60) { usleep(500000); clearButton(GTK_BUTTON(g_button60)); b60=0; }
	return TRUE;
	}
 
// ----------------------------------------------------------------
// Window is closed
// Cleanup dependent windows
// ----------------------------------------------------------------

void on_window_main_destroy() { 

	char cmd[256];

	system("killall --signal SIGTERM vlc 2> /dev/null");
	system("wmctrl -c animation");
        system("killall --signal SIGTERM phototonic 2> /dev/null");
        system("killall --signal SIGTERM mpv 2> /dev/null");
	system("killall --signal SIGTERM Player-bin 2> /dev/null");
	system("killall --signal SIGTERM gst-launch-1.0 2> /dev/null");
	system("killall --signal SIGTERM pqiv 2> /dev/null");
        system("killall --signal SIGHUP ffmpeg 2>/dev/null 1>/dev/null ");
	

// ----------------------------------------------------------------
//	this will kill ALL chromium windows - you may not want to do it
//	system("killall chrome");
// ----------------------------------------------------------------

	gtk_main_quit(); 
	}
 
// ----------------------------------------------------------------
// stream V  stream control
// ----------------------------------------------------------------

void on_button27_clicked(GtkButton *button) {
	stream(1,button);
        }

// ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// full screen
// ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

void on_button29_clicked(GtkButton *button) {

	if (streamOn) {
		Message("Stream not running");
		return;
		}

	gtk_widget_hide(GTK_WIDGET(HorLine)); // 'on-air' image

	setButton(GTK_BUTTON(button));
	clearButton(GTK_BUTTON(g_button38));
	panelView=0;
	fullScreen=1;
	fullView();
	gtk_widget_show(GTK_WIDGET(g_image200)); // 'on-air' image
	gtk_window_set_default_size (GTK_WINDOW(window), 1920, 1080);
	gtk_window_resize (GTK_WINDOW(window), 1920, 1080);
	return;
	}

// ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// panel view
// ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

void on_button38_clicked (GtkButton *button) {

	if (streamOn) {
		Message("Stream not running");
		return;
		}

	MAIN_WIND_VER = 0;
	MAIN_WIND_HOR = 0;

	gtk_widget_show(GTK_WIDGET(HorLine)); // 'on-air' image

	panelView = 1;
	fullScreen=0;

	setButton(GTK_BUTTON(button));
	clearButton(GTK_BUTTON(g_button29));

	panel_view();

	gtk_widget_hide(GTK_WIDGET(g_image200)); 
	gtk_window_set_default_size (GTK_WINDOW(window), 500, 670);
	gtk_window_resize (GTK_WINDOW(window), 600, 670);

	return;
	}

// ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// stream I
// ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

void	on_button40_clicked (GtkButton *button, GtkEntry *user_data) {
	stream(2,button);
	}

// ----------------------------------------------------------------
// radio button 1 
// ----------------------------------------------------------------

void on_radiobutton1_toggled (GtkToggleButton *togglebutton) {
	if (gtk_toggle_button_get_active (togglebutton) == FALSE) 
		return; // being unset
	killSlides();
	system("wmctrl -c animation");
	hidePics();
	gtk_window_set_default_size (GTK_WINDOW(window), 500, 670);
	gtk_window_resize (GTK_WINDOW(window), 600, 670);
	}

// ----------------------------------------------------------------
// radio button 2 
// ----------------------------------------------------------------

void on_radiobutton2_toggled (GtkToggleButton *togglebutton) {
	if (gtk_toggle_button_get_active (togglebutton) == FALSE) 
		return; // being unset
	killSlides();
	int i=2;
	backImage(i);
	}

// ----------------------------------------------------------------
// radio button 3 
// ----------------------------------------------------------------

void on_radiobutton3_toggled (GtkToggleButton *togglebutton) {
	if (gtk_toggle_button_get_active (togglebutton) == FALSE) 
		return; // being unset
	killSlides();
	int i=6;
	backImage(i);
	}

// ----------------------------------------------------------------
// radio button 4 
// ----------------------------------------------------------------

void on_radiobutton4_toggled (GtkToggleButton *togglebutton) {
	if (gtk_toggle_button_get_active (togglebutton) == FALSE) 
		return; // being unset
	killSlides();
	int i=5;
	backImage(i);
	}

// ----------------------------------------------------------------
// radio button 5 
// ----------------------------------------------------------------

void on_radiobutton5_toggled (GtkToggleButton *togglebutton) {
	if (gtk_toggle_button_get_active (togglebutton) == FALSE) 
		return; // being unset
	killSlides();
	int i=4;
	backImage(i);
	}

// ----------------------------------------------------------------
// radio button 6 
// ----------------------------------------------------------------

void on_radiobutton6_toggled (GtkToggleButton *togglebutton) {
	if (gtk_toggle_button_get_active (togglebutton) == FALSE) 
		return; // being unset
	killSlides();
	int i=1;
	backImage(i);
	}

// ----------------------------------------------------------------
// radio button 7 
// ----------------------------------------------------------------

void on_radiobutton7_toggled (GtkToggleButton *togglebutton) {
	if (gtk_toggle_button_get_active (togglebutton) == FALSE) 
		return; // being unset
	killSlides();
	int i=3;
	backImage(i);
	}

// ----------------------------------------------------------------
// radio button 8 
// ----------------------------------------------------------------

void on_radiobutton8_toggled (GtkToggleButton *togglebutton) {
	if (gtk_toggle_button_get_active (togglebutton) == FALSE) 
		return; // being unset
	killSlides();
	int i=7;
	backImage(i);
	}

// ----------------------------------------------------------------
// radio button 9 
// ----------------------------------------------------------------

void on_radiobutton9_toggled (GtkToggleButton *togglebutton) {
	if (gtk_toggle_button_get_active (togglebutton) == FALSE) 
		return; // being unset
	killSlides();
	int i=8;
	backImage(i);
	}

// ----------------------------------------------------------------
// radio button 10 
// ----------------------------------------------------------------

void on_radiobutton10_toggled (GtkToggleButton *togglebutton) {
	if (gtk_toggle_button_get_active (togglebutton) == FALSE) 
		return; // being unset
	killSlides();
	int i=11;
	backImage(i);
	}

// ----------------------------------------------------------------
// radio button 11 - uni 3
// ----------------------------------------------------------------

void on_radiobutton11_toggled (GtkToggleButton *togglebutton) {
	if (gtk_toggle_button_get_active (togglebutton) == FALSE) 
		return; // being unset
	killSlides();
	int i=9;
	backImage(i);
	}

// ----------------------------------------------------------------
// radio button 12 
// ----------------------------------------------------------------

void on_radiobutton12_toggled (GtkToggleButton *togglebutton) {
	if (gtk_toggle_button_get_active (togglebutton) == FALSE) 
		return; // being unset
	killSlides();
	int i=10;
	backImage(i);
	}

// ----------------------------------------------------------------
// radio button 13 
// ----------------------------------------------------------------

void on_radiobutton13_toggled (GtkToggleButton *togglebutton) {
	if (gtk_toggle_button_get_active (togglebutton) == FALSE) 
		return; // being unset
	killSlides();
	int i=12;
	backImage(i);
	}

// ----------------------------------------------------------------
// radio button 14 
// ----------------------------------------------------------------

void on_radiobutton14_toggled (GtkToggleButton *togglebutton) {
	killSlides();
	if (gtk_toggle_button_get_active (togglebutton) == FALSE) 
		return; // being unset
	int i=13;
	backImage(i);
	}

// ----------------------------------------------------------------
// radio button 15 
// ----------------------------------------------------------------

void on_radiobutton15_toggled (GtkToggleButton *togglebutton) {
	if (gtk_toggle_button_get_active (togglebutton) == FALSE) 
		return; // being unset
	killSlides();
	int i=14;
	backImage(i);
	}

// ----------------------------------------------------------------
// radio button 16 
// ----------------------------------------------------------------

void on_radiobutton16_toggled (GtkToggleButton *togglebutton) {
	if (gtk_toggle_button_get_active (togglebutton) == FALSE) 
		return; // being unset
	killSlides();
	int i=15;
	backImage(i);
	}

// ----------------------------------------------------------------
// radio button 17 
// ----------------------------------------------------------------

void on_radiobutton17_toggled (GtkToggleButton *togglebutton) {
	if (gtk_toggle_button_get_active (togglebutton) == FALSE) 
		return; // being unset
	killSlides();
	int i=16;
	backImage(i);
	}

// ----------------------------------------------------------------
// radio button 18 
// ----------------------------------------------------------------

void on_radiobutton18_toggled (GtkToggleButton *togglebutton) {
	if (gtk_toggle_button_get_active (togglebutton) == FALSE) 
		return; // being unset
	killSlides();
	int i=17;
	backImage(i);
	}

// ----------------------------------------------------------------
// radio button 19 
// ----------------------------------------------------------------

void on_radiobutton19_toggled (GtkToggleButton *togglebutton) {
	if (gtk_toggle_button_get_active (togglebutton) == FALSE) 
		return; // being unset
	killSlides();
	int i=18;
	backImage(i);
	}

// ----------------------------------------------------------------
// radio button 20 
// ----------------------------------------------------------------

void on_radiobutton20_toggled (GtkToggleButton *togglebutton) {
	if (gtk_toggle_button_get_active (togglebutton) == FALSE) 
		return; // being unset
	killSlides();
	int i=19;
	backImage(i);
	}

// ----------------------------------------------------------------
// radio button 21 
// ----------------------------------------------------------------

void on_radiobutton21_toggled (GtkToggleButton *togglebutton) {
	if (gtk_toggle_button_get_active (togglebutton) == FALSE) 
		return; // being unset
	killSlides();
	int i=20;
	backImage(i);
	}

// ----------------------------------------------------------------
// radio button 22 
// ----------------------------------------------------------------

void on_radiobutton22_toggled (GtkToggleButton *togglebutton) {
	if (gtk_toggle_button_get_active (togglebutton) == FALSE) 
		return; // being unset
	killSlides();
	int i=21;
	backImage(i);
	}

// ----------------------------------------------------------------
// radio button 23 
// ----------------------------------------------------------------

void on_radiobutton23_toggled (GtkToggleButton *togglebutton) {
	if (gtk_toggle_button_get_active (togglebutton) == FALSE) 
		return; // being unset
	killSlides();
	int i=22;
	backImage(i);
	}

// ----------------------------------------------------------------
// radio button 24 
// ----------------------------------------------------------------

void on_radiobutton24_toggled (GtkToggleButton *togglebutton) {
	if (gtk_toggle_button_get_active (togglebutton) == FALSE) 
		return; // being unset
	killSlides();
	int i=23;
	backImage(i);
	}

// ----------------------------------------------------------------
// button 52 - Exit
// ----------------------------------------------------------------

void on_button52_clicked (GtkToggleButton *b) {

        int i;
        on_window_main_destroy(); 
	return; // ubuntu does not allow the dialiog window to move

        GtkDialogFlags flags = GTK_DIALOG_DESTROY_WITH_PARENT |
                GTK_MESSAGE_ERROR | GTK_DIALOG_MODAL | 
		GTK_DIALOG_USE_HEADER_BAR ;

        GtkWidget *dialog = gtk_dialog_new_with_buttons  ("Exit Encoder?", 
		GTK_WINDOW(window),
                flags,
                "Yes", 100, "No", 200, NULL);

//      expect 200 for NO and 100 for YES

	gtk_window_move(GTK_WINDOW(dialog), 300,730);
        gint r = gtk_dialog_run (GTK_DIALOG (dialog));

        gtk_widget_destroy (dialog);

        if ( r == 100) on_window_main_destroy(); 

	}

//--------------------------------------
// the following is not currently active
// it is used to change audio levels
//--------------------------------------

void on_scalebutton1_value_changed(GtkScaleButton *button, gdouble value, gpointer *user_data) {

	int i;
	char cmd[512];

	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (user_data), TRUE);
	ScaleLevel=value;
	AudioLevel=0x10000+(ScaleLevel-50)*500;

	i = round( ((double)AudioLevel/0x10000) * 100.0);
	sprintf(cmd,"Player: %d%%",i);
	sprintf(cmd,"pacmd set-source-volume %d 0x%x",AUDIO_PROG_IN,AudioLevel);
	system(cmd);

	}

// ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// strip blanks
// ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

void	strip(char* str) {
	int i,x;
	for(i=x=0; str[i]; ++i)
		if(!isspace(str[i]) || (i>0 && !isspace(str[i-1])))
			str[x++] = str[i];
	str[x] = '\0';
	}

// ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//	make a button blue
// ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

void	clearButton(GtkButton *button) {
	GtkCssProvider * cssProvider;
	cssProvider  = gtk_css_provider_new();
	gtk_css_provider_load_from_data(cssProvider, css0, -1, NULL);
	css_set(cssProvider, GTK_WIDGET(button));
	}

// ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//	make a button red
// ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

void	setButton(GtkButton *button) {
	GtkCssProvider * cssProvider;
	cssProvider  = gtk_css_provider_new();
	gtk_css_provider_load_from_data(cssProvider, css1, -1, NULL);
	css_set(cssProvider, GTK_WIDGET(button));
	}

// ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// paint css on a widget
// ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

void	css_set(GtkCssProvider * cssProvider, GtkWidget *g_widget) {

	GtkStyleContext *context;

	context = gtk_widget_get_style_context(g_widget);    

	gtk_style_context_add_provider (context,
		GTK_STYLE_PROVIDER(cssProvider),
		500);

	gtk_style_context_save (context);
	}

//------------------------------------------------
// terminate and close dependent windows and tasks
//------------------------------------------------

void	cleanCast() {	// terminate system
	char cmd[256];

	system("killall --signal SIGTERM vlc 2> /dev/null");
        system("killall --signal SIGTERM mpv 2> /dev/null");
	system("wmctrl -c animation");
	system(cmd);
	system("killall --signal SIGTERM phototonic 2> /dev/null");
	system("killall --signal SIGTERM pavumeter 2> /dev/null");
	system("killall --signal SIGTERM pavucontrol 2> /dev/null");
	system("killall --signal SIGTERM gst-launch-1.0 2> /dev/null");
        system("killall --signal SIGHUP ffmpeg 2>/dev/null 1>/dev/null ");
	}

//----------------------------------------------
//	hide pictures
//----------------------------------------------

void	hidePics() {
	gtk_widget_hide(g_image1);
	}

//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//	PANEL
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

//----------------------------------------------
//	panel_view
//----------------------------------------------

void	panel_view() {

	STREAM_WIDE = 1306;
	STREAM_HIGH = 750;
	STREAM_HOR = MAIN_WIND_HOR+614;
	STREAM_VER = MAIN_WIND_VER+28;
	FullPanelCommon();
	}

//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//	COMMON FULL and PANEL
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

void	FullPanelCommon() {

	gtk_window_move (GTK_WINDOW(window), 0, 0);
	MAIN_WIND_HOR = 0;
	MAIN_WIND_VER = 0;
	MAIN_WIND_WIDTH = 1324;
	MAIN_WIND_HEIGHT = 750;
	return;
	}

//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//	FULL
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

void	fullView() {
	STREAM_WIDE = 1920;
	STREAM_HIGH = 1080;
	STREAM_HOR = 0;
	STREAM_VER = 0;
	FullPanelCommon();
	}

// ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// stream control
// ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

void	stream(int strm, GtkButton *button) {
        char cmd[4096];
        gint x,y;
	static int running=0;
	static int tog=0;

	GtkCssProvider * cssProvider  = gtk_css_provider_new();

        system("killall ffmpeg 2>/dev/null 1>/dev/null ");

        if (running) { 
		gtk_widget_hide(GTK_WIDGET(g_image8)); // turns off 'on air' image
		gtk_widget_hide(GTK_WIDGET(g_image10)); // turns off 'record' image
		gtk_widget_show(GTK_WIDGET(g_image9)); // turns on 'standby' image

		running = 0; 
		strcpy(stream_output,"Stream inactive");
		gtk_css_provider_load_from_data(cssProvider, css0, -1, NULL);
		css_set(cssProvider,GTK_WIDGET(button));
		streamOn = 0;
		return; 
		}

	if (audsys == PULSE )
		sprintf(cmd,"%s %d %d %dx%d %d %d &", 
			"$HOME/Desktop/AutoPlay/Encoders/FullFeatureEncoder/stream.script", 
			STREAM_VER, STREAM_HOR, STREAM_WIDE, STREAM_HIGH, strm, LOOPBACK );

	else // JACK Audio - loopback 3
		sprintf(cmd,"%s %d %d %dx%d %d %d &", 
			"$HOME/Desktop/AutoPlay/Encoders/FullFeatureEncoder/stream.script", 
			STREAM_VER, STREAM_HOR, STREAM_WIDE, STREAM_HIGH, strm, 3 );

	running = 1;
	streamOn = 1;

	if (strm == 3 )	gtk_widget_show(GTK_WIDGET(g_image10)); // turns on 'record' image
		else	gtk_widget_show(GTK_WIDGET(g_image8)); // turns on 'on air' image

	gtk_widget_hide(GTK_WIDGET(g_image9)); // turns off 'standby' image

	setButton(GTK_BUTTON(button));

        system(cmd); // start stream
        sleep(1);

        }

// ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//	pop up message window
// ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

void	Message(char * msg) {

	GdkColor color1;
	color1.red = 0x5555;
	color1.green = 0x5555;
	color1.blue = 0x5555;

	GdkColor color2;
	color2.red = 0xFFFF;
	color2.green = 0xFFFF;
	color2.blue = 0xFFFF;

	GtkDialogFlags flags = GTK_DIALOG_DESTROY_WITH_PARENT ;
	GtkWidget *dialog = gtk_message_dialog_new (GTK_WINDOW(window),
		flags,
		GTK_MESSAGE_WARNING,
		GTK_BUTTONS_CLOSE,
		msg);

	gtk_widget_modify_bg(dialog, GTK_STATE_NORMAL, &color1);
	gtk_widget_modify_fg(dialog, GTK_STATE_NORMAL, &color2);

	gtk_window_move (GTK_WINDOW(dialog), 215, 582);
	gtk_dialog_run (GTK_DIALOG (dialog));
	gtk_widget_destroy (dialog);
	}

//------------------------------------------------
//	SLIDE SHOW & Background Image
//	Paint background image & slide show
//	SLIDE SHOW
//------------------------------------------------

void	backImage(int i) {
	const gchar *p;
	char cmd[1024];
	int j,k,h,h1,v,v1,tmpFlg=0;
	FILE *f1;
	char fileName[4096];

	if (! backs[i].valid) return;

	system("wmctrl -c animation");

	if (backs[i].file[0] == '*') {

		system(&backs[i].file[1]);
		sprintf(cmd,
			"wmctrl -r XScreensaver -e 0,%d,%d,%d,%d",
			SSHOR,SSVER,SSWID,SSHGH);
		sleep(1);
		system (cmd);
                system("wmctrl -r XScreensaver -b add,above");

		system( "xprop -name \"Spark Gap Radio Player\" "
		"-f _MOTIF_WM_HINTS 32c -set _MOTIF_WM_HINTS \"0x0, 0x0, 0x0, 0x0, 0x0\"");
                system("wmctrl -r Player -b add,above");
		system( "xprop -name \"Spark Gap Radio Player\" "
		"-f _MOTIF_WM_HINTS 32c -set _MOTIF_WM_HINTS \"0x2, 0x0, 0x0, 0x0, 0x0\"");
                system("wmctrl -r XScreensaver -T animation");
		return;
		}

	f1 = fopen(backs[i].file,"r");
	if (f1 == NULL) {
		printf("*** Bad slide file name: %s\n", backs[i].file);
		return;
		}

	fclose(f1);

	strcpy(fileName, backs[i].file);

	// get dimensions of image

	sprintf(cmd, "identify -format %%wx%%h \"%s\"\n", fileName);

	f1 = popen(cmd, "r");

	strcpy(cmd,"");
	fgets(cmd, 512, f1);
	fclose (f1);

	h = v = 1;

	if (strlen(cmd)) {
		for (j=0; j<strlen(cmd)-1; j++) if (cmd[j] == 'x') break;
		if (cmd[j] == 'x') { // x is a delimiter between horizontal and vertical
			cmd[j] = 0;
			sscanf(cmd,"%d",&h);
			sscanf(&cmd[j+1], "%d", &v);
			}
		}

	if (h < 100 || v < 100 ) {
		printf("**** questionable image: %s\n",fileName);
		return;	// probably a bad image
		}

	gtk_container_remove (GTK_CONTAINER (fixed1), g_image1); // remove old slide

// CENTERED slides

	if (i<24) {	// buttoned images

		if (h > 1302 || v > 748) { // large image

			if (h > 1302 ) {
				v = v * (1302.0/h);
				sprintf(cmd, "convert \"%s\" -resize 1302x%d tmp.jpg", fileName, v);
				system(cmd);
				tmpFlg = 1;
				strcpy(fileName,"tmp.jpg");
				backs[i].hor = 610;
				backs[i].ver = (748  - v )/2;
				h=1302; //new width
				}
			if (v > 748) {
				h = h * (748.0/v);
				sprintf(cmd, "convert \"%s\" -resize %dx748 tmp.jpg", fileName, h);
				system(cmd);
				tmpFlg = 1;
				strcpy(fileName,"tmp.jpg");
				backs[i].hor = 610 + (1302 -h)/2;
				backs[i].ver = 0;
				v = 748;
				}
			}

		else { // normal image
			backs[i].hor = 610 + (1302 - h)/2;
			backs[i].ver = (748 - v)/2;
			}
		}

	else {	// slides

//		RESIZE
//		convert image to desired size

		int width = 700; // also: 1000, 870
		int height = 730; 

		sprintf(cmd, "convert \"%s\" -resize %dx%d tmp.jpg", fileName, width, height);
		system(cmd);

		tmpFlg = 1;
		strcpy(fileName,"tmp.jpg");

		sprintf(cmd, "identify -format %%wx%%h \"%s\"\n", fileName);
		f1 = popen(cmd, "r");
		strcpy(cmd,"");
		fgets(cmd, 512, f1);
		fclose (f1);
		h = v = 1;

//		Get image dimensions  - actual dimensions will be in the
//		range of the conversion. Conversion will maintain ratio.

		if (strlen(cmd)) {
			for (j=0; j<strlen(cmd)-1; j++) if (cmd[j] == 'x') break;
			if (cmd[j] == 'x') {
				cmd[j] = 0;
				sscanf(cmd,"%d",&h);
				sscanf(&cmd[j+1], "%d", &v);
				}
			}

		printf("slide=%d %s hor=%d ver=%d\n", i, backs[i].file, h, v);

//		Rearrange these as desired - last one counts
//		Small adjustments can set the slides off center
//		Several variants shown.

//		vertical placement - v is actual height

		backs[i].ver = 2;			// place at top
		backs[i].ver = (height-v)/2;		// place at vert center
		backs[i].ver = 2;			// place at top
		backs[i].ver = 10 + (height-v)/2;	// place at vert center

//		horizontal placement - h is actual width

		backs[i].hor = 630;   			// left justified
		backs[i].hor = 1044;   			// left justified relative to 1044
		backs[i].hor = 1920-h-10;               // right justified
		backs[i].hor = 630 + (width - h)/2; 	// centered relative to 630
		backs[i].hor = 1220 + (width - h)/2;	// centered relative to 1220
							// width + 1210 -> 1920
		}

	g_image1 = gtk_image_new_from_file (fileName);
	gtk_widget_show(g_image1);

	gtk_container_add (GTK_CONTAINER (fixed1), g_image1);

	gtk_fixed_move (GTK_FIXED(fixed1), g_image1, backs[i].hor, backs[i].ver);

	gtk_window_set_default_size (GTK_WINDOW(window), 1918, 670);

	if (tmpFlg) system("rm tmp.jpg");

	p=gtk_label_get_text(GTK_LABEL(widgets->w_lbl_stream));	// these lines trigger a re-paint
	gtk_label_set_text(GTK_LABEL(widgets->w_lbl_stream), p);   

	}

void	on_Camera0_clicked (GtkButton *b) {
		char cmd[1024];
//                sprintf(cmd,"guvcview -a none -g none -d \"/dev/video0\" &");
		system ("wmctrl -r gst0 -e 0,680,880,240,160 ");
                sprintf(cmd,
			"gst-launch-1.0 -v v4l2src device=/dev/video0 ! "
			"video/x-raw,width=640,height=480,framerate=20/1 ! xvimagesink &");
                system(cmd); // start camera
                sleep(1);
                sprintf(cmd, "wmctrl -r \"gst-launch-1.0\" -T gst0");
                system (cmd);
                sprintf(cmd, "wmctrl -r gst0 -b add,above");
                system (cmd);
		system ("wmctrl -r gst0 -e 0,680,880,240,160 ");
	}

gboolean	on_Camera0_button_press_event(GtkEntry *entry, GdkEventButton *event) {
        	if (event->button == 1 ) return FALSE;	// left click
		system ("wmctrl -r gst0 -e 0,1250,70,640,460 ");
                system ("wmctrl -r gst0 -b add,above");
		system ("wmctrl -r gst1 -e 0,936,880,240,160 ");
		return TRUE;
		}

void	on_Camera1_clicked (GtkButton *b) { // set for my logitech 720p - adjust as needed
		char cmd[1024];
		system ("wmctrl -r gst1 -e 0,936,880,240,160 ");
                sprintf(cmd,
			"gst-launch-1.0 -v v4l2src device=/dev/video1 ! "
			"video/x-raw,width=640,height=480 ! xvimagesink &");
                system(cmd); // start camera
                sleep(1);
                sprintf(cmd, "wmctrl -r \"gst-launch-1.0\" -T gst1");
                system (cmd);
                sprintf(cmd, "wmctrl -r gst1 -b add,above");
                system (cmd);
		system ("wmctrl -r gst1 -e 0,936,880,240,160 ");
		}

gboolean	on_Camera1_button_press_event(GtkEntry *entry, GdkEventButton *event) {
        	if (event->button == 1 ) return FALSE;	// left click
		system ("wmctrl -r gst1 -e 0,1250,50,640,460 ");
                system ("wmctrl -r gst1 -b add,above");
		system ("wmctrl -r gst0 -e 0,680,880,240,160 ");
		return TRUE;
		}

void	on_Player_clicked (GtkButton *b) {
	char cmd[1024];
	sprintf(cmd,"wmctrl -r \"Spark Gap Radio Player\" -e 0,%d,%d,-1,-1 ", 
		PLAYER_HOR+4, PLAYER_VER+28); // compensate for missing decorations
	system(cmd);
	system("wmctrl -r \"Spark Gap Radio Player\" -b add,above");
	system(cmd);
	sprintf(cmd,
		"wmctrl -r animation -e 0,%d,%d,%d,%d",
		SSHOR,SSVER,SSWID,SSHGH);
	system (cmd); // restore
	}

gboolean    on_Player_button_press_event(GtkEntry *entry, GdkEventButton *event) {
        if (event->button == 1 ) return FALSE;	// left click
	char cmd[1024];
	sprintf(cmd,"wmctrl -r \"Spark Gap Radio Player\" -e 0,%d,%d,-1,-1 ", 
		5, 28); // place at upper left
	system(cmd);
	system("wmctrl -r \"Spark Gap Radio Player\" -b add,above");
	system(cmd);
	sprintf(cmd,
		"wmctrl -r animation -e 0,740,0,1300,750");
	system(cmd);
	return TRUE;
        }

void	on_SlidePause_clicked ( GtkButton *b) {
	GtkCssProvider * cssProvider;

	if (slideShow) { // blue
		cssProvider  = gtk_css_provider_new();
		gtk_css_provider_load_from_data(cssProvider, css0, -1, NULL);
		css_set(cssProvider, GTK_WIDGET(b));
		slideShow = 0;
		}
	else { // red
		// second slide must be full width
		cssProvider  = gtk_css_provider_new();
		gtk_css_provider_load_from_data(cssProvider, css1, -1, NULL);
		css_set(cssProvider, GTK_WIDGET(b));
		slideShow = 1;
		}
	}

void	killSlides() {
	GtkCssProvider * cssProvider;
	cssProvider  = gtk_css_provider_new();
	gtk_css_provider_load_from_data(cssProvider, css0, -1, NULL);
	css_set(cssProvider, GTK_WIDGET(SlidePause));
	slideShow = 0;
	}

void	on_Record_clicked(GtkButton *b) {

	char tmp[1024] = "";

//        FILE *s1=popen("ps -ale | grep jackd" , "r");

//        if (s1 != NULL) {
//                tmp[0]=0;
//                fgets(tmp,1024,s1);
//                fclose(s1);
//                }
//
//	if (strlen(tmp) == 0 || s1 == NULL ) return;

	stream(3,b);
	}

//	insert entry

void	on_Deploy_clicked (GtkButton *b) {
	char cmd[2048];
	strncpy(cmd, gtk_entry_get_text (GTK_ENTRY(Banner)),2024);
	for (int i=0; i< strlen(cmd); i++ ) {
		if (cmd[i] == '\\' && cmd[i+1] == 'n') {
			cmd[i]=' ';
			cmd[i+1]='\n';
			}
		}
	gtk_label_set_text(GTK_LABEL(TopLabel),cmd);
	gtk_label_set_xalign (GTK_LABEL(TopLabel), 0.0 );
	}

//	insert entry

void	on_Deploy2_clicked (GtkButton *b) {
	char cmd[2048];
	strncpy(cmd, gtk_entry_get_text (GTK_ENTRY(Banner)),2024);
	for (int i=0; i< strlen(cmd); i++ ) {
		if (cmd[i] == '\\' && cmd[i+1] == 'n') {
			cmd[i]=' ';
			cmd[i+1]='\n';
			}
		}
	gtk_label_set_text(GTK_LABEL(BotLabel),cmd);
	gtk_label_set_xalign (GTK_LABEL(BotLabel), 0.0 );
	}

//	erase entry

gboolean    on_Deploy_button_press_event(GtkEntry *entry, GdkEventButton *event) {
        if (event->button == 1 ) return FALSE;	// left click
	gtk_label_set_text (GTK_LABEL(TopLabel), "");
	return TRUE;
        }

//	erase entry

gboolean    on_Deploy2_button_press_event(GtkEntry *entry, GdkEventButton *event) {
        if (event->button == 1 ) return FALSE;	// left click
	gtk_label_set_text (GTK_LABEL(BotLabel), "");
	return TRUE;
        }


