/*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
#+
#+     Internet Radio Automation & Encoding Toolkit
#+
#+     Copyright (C) 2018 by Kevin C. O'Kane
#+
#+     Kevin C. O'Kane
#+     kc.okane@gmail.com
#+     https://www.cs.uni.edu/~okane
#+     http://threadsafebooks.com/
#+
#+ This program is free software; you can redistribute it and/or modify
#+ it under the terms of the GNU General Public License as published by
#+ the Free Software Foundation; either version 2 of the License, or
#+ (at your option) any later version.
#+
#+ This program is distributed in the hope that it will be useful,
#+ but WITHOUT ANY WARRANTY; without even the implied warranty of
#+ MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#+ GNU General Public License for more details.
#+
#+ You should have received a copy of the GNU General Public License
#+ along with this program; if not, write to the Free Software
#+ Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#+
#++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/

// February 20, 2019

#include <stdlib.h>
#include <sys/types.h>
#include <signal.h>
#include <unistd.h>
#include <string.h>
#include <gtk/gtk.h>
#include <libgen.h>
#include <sys/mman.h>
#include "settings.h"
#include <math.h>
#include <ctype.h>
#include <time.h>

#define STR_MAX 512

// maximum length of a program list entry

#define PGM_MAX 512

#define VLC_PLAYER 1
#define MPV_PLAYER 0

#define PULSE 1
#define JACK 0

//***********************************************
// function headers
//***********************************************

int	hexcleaner(char *t);
void	sounders(void);
void	css_set(GtkCssProvider * cssProvider, GtkWidget *g_widget);
int	lastWord(char * tmp, int off);
void	entry_insert(GtkEditable  *entry, gchar *edit);
void	entry_insert1(GtkEntry  *entry, gchar *edit);
void	entry_changed (GtkEntry *entry, gchar *preedit, 
		gpointer  user_data, int Row);
void	entry_changed1 (GtkEntry *entry, gchar *preedit, 
		gpointer  user_data, int Row);
void	strip(char* str);
void	sounderButton(GtkWidget *, int);
void	AdvanceSounderPage(); 
void	clearButton(GtkButton *);
void	setButton(GtkButton *);
int	filetype(char *);
int	smallFilename(GtkEntry *, char *) ;
void	RandomPlay (GtkButton *, GtkEntry *, GtkWidget *, int );
void	PlayClip (GtkButton *, GtkEntry *, int);
void	DirChooser (GtkButton *, GtkEntry *, int , GtkWidget *);
void	FileChooser (GtkButton *, GtkEntry *, int , GtkWidget *);
char* 	shuffle(int , char *);
int	smallDirname(GtkEntry *, char *);
void	Message(char *);
int	VideoRunning();
void	on_button30_clicked (GtkButton *); // raise camera 0
void	on_button32_clicked (GtkButton *button);
char*	sounderName(char *, int nbr);
int	fileExists(const char *fname);
char*	sndr[24];
void	paste(int *Row); 
void    paste1(int *Row);
void	delRow(int Row);
void	sndr_clicked (GtkButton *button, int bnbr);
void	moveToTop (int row);
void	moveToBottom (int row);
void	orgPgm();
int	delDialog();
void    PangoAttributes();
void	setCssAttributes( GtkCssProvider * cssProvider1);
gboolean	scroll_event(GtkButton *b, GdkEventScroll *e);
void	AudioSubSys(char * cmd);
int	WhatAudio();
void	on_button34_clicked (GtkButton *button); 
void	on_oneTime_clicked (GtkButton *b);
char *	fileName(char * sndr, char * cmd);
void	TimeOfNext();
void	ClearTimeOfNext();
void	WindowsCallBacks();
void	LoadProgram(char *);
void	GetTime (int Row, char *file);
void	PauseOff ();
void    ClearPlayers();
int     MediaProcess(int *, char *);
void	on_button31_clicked (GtkButton *button);

//--------------------------------------------
// return codes for file fytpes from filetype()
//--------------------------------------------

#define MP4 1
#define MP3 2
#define MPG 3
#define DVD 4
#define OGG 5
#define WAV 6
#define FLAC 7
#define PLAY 8
#define WEB 9
#define PNG 10
#define JPG 11
#define WMV 12
#define AVI 12
#define MPV 13

//------------------------------------------
// global GTK widgets for buttons and things
//------------------------------------------

GtkEntry	*g_dbox1Entry;
GtkWidget	*scroll;
GtkWidget	*button16;
GtkWidget	*dxb1;
GtkWidget	*dxb2;
GtkWidget	*g_dbox1;
GtkWidget	*gPlayer;
GtkWidget	*orgPlay;
GtkWidget	*up1;
GtkWidget	*up2;
GtkWidget	*up3;
GtkWidget	*up4;
GtkWidget	*up5;
GtkWidget	*up6;
GtkWidget	*up7;
GtkWidget	*up8;
GtkWidget	*up9;
GtkWidget	*up10;
GtkWidget	*up11;
GtkWidget	*up12;
GtkWidget	*Playerx;
GtkWidget	*Audsys;
GtkWidget	*findVlc;
GtkWidget	*oneTime;
GtkWidget	*onetimeImage;
GtkWidget	*clrPgm;
GtkWidget	*check1;
GtkWidget	*check2;
GtkWidget	*check3;
GtkWidget	*check4;
GtkWidget	*check5;
GtkWidget	*check6;
GtkWidget	*check7;
GtkWidget	*check8;
GtkWidget	*check9;
GtkWidget	*check10;
GtkWidget	*check11;
GtkWidget	*check12;
GtkWidget	*showCurrent;
GtkWidget	*del1;
GtkWidget	*del2;
GtkWidget	*del3;
GtkWidget	*del4;
GtkWidget	*del7;
GtkWidget	*del8;
GtkWidget	*del9;
GtkWidget	*del10;
GtkWidget	*del11;
GtkWidget	*del12;
GtkWidget	*del13;
GtkWidget	*del14;
GtkWidget	*save1;
GtkWidget	*g_btn_hello;
GtkWidget	*g_btn_hello1;
GtkWidget	*g_btn_play;
GtkWidget	*g_btn_play2;
GtkWidget	*sndr1;
GtkWidget	*sndr2;
GtkWidget	*sndr3;
GtkWidget	*sndr4;
GtkWidget	*sndr5;
GtkWidget	*sndr6;
GtkWidget	*sndr7;
GtkWidget	*sndr8;
GtkWidget	*sndr9;
GtkWidget	*sndr10;
GtkWidget	*sndr11;
GtkWidget	*sndr12;
GtkWidget	*sndr13;
GtkWidget	*sndr14;
GtkWidget	*sndr15;
GtkWidget	*sndr16;
GtkWidget	*sndr17;
GtkWidget	*sndr18;
GtkWidget	*sndr19;
GtkWidget	*sndr20;
GtkWidget	*aud1;
GtkWidget	*aud2;
GtkWidget	*g_scale1;
GtkWidget	*g_label1;
GtkWidget	*timeNext;
GtkWidget	*TimeOfDay;
GtkWidget	*g_label2;
GtkWidget	*g_label4;
GtkWidget	*g_button1;
GtkWidget	*g_button2;
GtkWidget	*g_button3;
GtkWidget	*g_button4;
GtkWidget	*g_button5;
GtkWidget	*g_button6;
GtkWidget	*g_button7;
GtkWidget	*g_button8;
GtkWidget	*g_button9;
GtkWidget	*g_button10;
GtkWidget	*g_button11;
GtkWidget	*g_button12;
GtkWidget	*g_button13;
GtkWidget	*g_button14;
GtkWidget	*g_button15;
GtkWidget	*g_button18;
GtkWidget	*g_button30;
GtkWidget	*g_button31;
GtkWidget	*g_button32;
GtkWidget	*g_button33;
GtkWidget	*g_button34;
GtkWidget	*g_button35;
GtkWidget	*g_button37;
GtkWidget	*g_button52;
GtkWidget	*g_button57;
GtkWidget	*g_button58;
GtkWidget	*g_rnd_btn1;
GtkWidget	*g_rnd_btn2;
GtkWidget	*first;
GtkWidget	*g_image1;
GtkWidget	*g_image9;
GtkWidget	*g_cont;
GtkSearchEntry	*searchEntry1;
GtkSearchEntry	*searchEntry2;
GtkEntry	*g_lbl_entry1;
GtkEntry	*g_lbl_entry2;
GtkEntry	*g_lbl_entry3;
GtkEntry	*g_lbl_entry4;
GtkEntry	*g_lbl_entry5;
GtkEntry	*g_lbl_entry6;
GtkEntry	*g_lbl_entry7;
GtkEntry	*g_lbl_entry8;
GtkEntry	*g_lbl_entry9;
GtkEntry	*g_lbl_entry10;
GtkEntry	*g_lbl_entry11;
GtkEntry	*g_lbl_entry12;
GtkEntry	*g_lbl_entry13;
GtkEntry	*g_lbl_entry14;
GtkEntry	*g_lbl_entry15;
GtkWidget	*image;
GdkColor	color;
GtkBuilder	*builder; 
GtkWidget	*window;
GtkWidget	*fixed1;
GtkCssProvider	*cssProvider1;
GtkCssProvider	*cssProvider2;
GtkCssProvider	*cssProvider3;
GtkCssProvider	*cssProvider4;

gint x,y;

static int AudioLevel=0x10000;
static int ScaleLevel=0;
static int PlaySelected=-1;

static int *CurrentPlay;
static int *NowPlaying;		// sounder button id of currently playing
static int *NowPlaying2;	// indicator that seq order has been changed
static int *Clock;
static char *TimeNext;

int   copyClipRow;
int   e7Row,e8Row,e9Row,e10Row,e11Row,e12Row,e13Row,e14Row,e15Row,e16Row,e17Row,e18Row;
char  cutRow[512], top1[512], top2[512], top3[512]; // row file name holders
char  cutTime[32];
int   cutPlay;
int   cutPlayed;
int   *vlc_hor,*vlc_ver,*vlc_hgt,*vlc_wid,*player,*audsys;
int   *playonce;
int   *OrgPlay;
int   doSounders;
int   ubuntu;
int   *AudioSystem;
int   SaveFlg;
int   flg18, flg18a;
int   flg35, flg35a;

// playlist check boxes

int c1state = 0;
int c2state = 0;
int c3state = 0;
int c4state = 0;
int c5state = 0;
int c6state = 0;
int c7state = 0;
int c8state = 0;
int c9state = 0;
int c10state = 0;
int c11state = 0;
int c12state = 0;

//***********************************************
// playlist button definintions
//***********************************************

// max number of characters of title in button
#define SOUNDER_SIZE 64

// max number of pointers to titles
int SOUNDER_MAX=4096;

// max number of characters per button line
#define SOUNDER_ROW 32

// sounder buttons per page
#define SOUNDER_BUTTON_COUNT 12

//***********************************************
// mpv player startup macros
//***********************************************

#define VLC_PLAY sprintf(cmd,"vlc --video-on-top --no-qt-video-autoresize \
 --no-qt-name-in-title --qt-continue 0 --qt-minimal-view -q --play-and-exit ");

#define VLC_PLAY_AUDIO sprintf(cmd,"vlc --video-on-top --no-qt-video-autoresize \
 --no-qt-name-in-title --qt-continue 0 --qt-minimal-view -q --play-and-exit ");

#define MPV_AUDIO_DELAY 0.0

// some 'video' files have no video so force a windo to appear

#define MPV_PLAY sprintf(cmd,"mpv --audio-delay=%f --ontop \
 --force-window=yes --osd-back-color=1.0/0.0/0.0 --title=\"mpv player\" --autofit=%d ", \
 MPV_AUDIO_DELAY, *vlc_wid);

// --osd-back-color=1.0/0.0/0.0 --title=\"mpv player\" --geometry=%dx%d --autofit=%d ", 
// MPV_AUDIO_DELAY, *vlc_wid, *vlc_hgt, *vlc_hor, *vlc_ver);

#define MPV_PLAY_AUDIO sprintf(cmd,"mpv --audio-delay=%f --force-window=yes \
 --osd-level=3 --osd-back-color=0.9/0.5/0.9/0.6  --osd-scale 3 --osd-font-size=88 \
 --osd-align-y=center --osd-align-x=center --ontop \
 --title=\"mpv player\" --geometry=%dx%d+%d+%d ", \
 MPV_AUDIO_DELAY, 300, 100, *vlc_hor, *vlc_ver);

//***********************************************
// communication with timer
// some code runs as child proceses and their
// data is not otherwise visible to the timer
// function.
// this is a structure of pointers to data that
// is in shared memory that can be seen by the
// timer and the child processes.
// pointer values to be filled in below.
//***********************************************

typedef struct {
	GtkWidget *w_lbl_nowplay;
	GtkWidget *w_lbl_timeNext;
	GtkWidget *w_lbl_TimeOfDay;
	GtkWidget *w_lbl_button3;
	GtkWidget *w_lbl_button8;
	GtkWidget *w_lbl_button9;
	GtkWidget *w_lbl_button10;
	GtkWidget *w_lbl_button11;
	GtkWidget *w_lbl_button12;
	GtkWidget *w_lbl_button13;
	GtkWidget *w_lbl_button14;
	GtkWidget *w_lbl_button2;
	GtkWidget *w_lbl_button5;
	GtkWidget *w_lbl_button6;
	GtkWidget *w_lbl_button15;
	GtkWidget *w_lbl_button18;
	GtkWidget *w_lbl_button31;
	GtkWidget *w_label4; // next random selection
	GtkWidget *w_label2; // next random selection
	} app_widgets;

	app_widgets *widgets;

//***********************************************
// definition of timer function. it is invoked
// once per secon by GTK
//***********************************************

gboolean timer_handler(app_widgets *widgets);

static char *next_random;
static char *currentPlay;;
static int  *audio;
static int  *videoPlaying;
static int  *skip;

int soundersLoading,inFlg;
static char **sounder;
static int  *sounderPlayed;

struct ST { 
	int play;
	char Time[32]; 
	} **sounderTime;

volatile static int  *scount;
volatile static int  *begin;

volatile int  *Pause;
volatile int  *PauseKill;

int  vlcDir = 0;

char directories[5][1024] = {0};
int  lastDir = -1;
char Music_Home_Directory[1024];
char Code_Home[1024];

//-----------------------------------------------------
//	css codes. note back slash characters.
//-----------------------------------------------------

char css0[2048]	= "GtkModelButton, GtkButton { \
      color: blue;\
      padding-left: 2px; \
      padding-right: 2px; \
      border-color: shade (mix (rgb (34, 34, 120), #fff, 0.5), 0.1); \
      }  \
\
button { \
      color: blue; \
      padding-left: 2px; \
      padding-right: 2px; \
      border-color: shade (mix (rgb (34, 34, 120), #fff, 0.5), 0.1); \
      };" ;

//---------------------------------------------------

// used to set button text color to red

char css1[1024] = "GtkButton, button, label {	\
      color: red;	\
      border-color: shade (mix (rgb (34, 34, 120), #fff, 0.5), 0.1);	\
      }";

	char mpvFile[512];;

// ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// main main main main main main main main main main main main main
// ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

int main(int argc, char *argv[]) {

	char cmd[1024],tmp[1024];
	FILE *s1,*s2;
	int i,j,k,m,n;

	srand(time(0));

	readlink("/proc/self/exe", Code_Home, 1024); // where are we?

	for (i = strlen(Code_Home); i > 0; i--)
		if (Code_Home[i] == '/' ) {
			Code_Home[i] = 0;
			break;
			}
	if (i == 0) {
		printf("Directory error finding Code_Home\n");
		return EXIT_FAILURE;
		}

	strcpy(mpvFile,"rm ");
	strcat(mpvFile,Code_Home);
	strcat(mpvFile,"/mpvTmp 2>/dev/null");

	s1 = fopen("Player.css","r");
	if (s1 == NULL ) {
		printf("*** CSS file not found\n");
		return 1;
		}

	strcpy (css0,"");

	while (fgets(cmd,1024,s1)) {
		cmd[strlen(cmd)-1] = 0; // new line chop
		strcat(css0,cmd);
		strcat(css0," ");
		}

	printf("css0 %s\n",css0);

	fclose (s1);

//---------------------------------------------------------------------
//	allocate shared memory used to communicate with the timer function
//	from detached child processes.
//---------------------------------------------------------------------

	audsys = mmap(NULL, 1024, PROT_READ | PROT_WRITE, 
		MAP_SHARED | MAP_ANONYMOUS, -1, 0); // shared memory

	next_random = mmap(NULL, 1024, PROT_READ | PROT_WRITE, 
		MAP_SHARED | MAP_ANONYMOUS, -1, 0); // shared memory

	currentPlay = mmap(NULL, 1024, PROT_READ | PROT_WRITE, 
		MAP_SHARED | MAP_ANONYMOUS, -1, 0); // shared memory

	NowPlaying = mmap(NULL, 16, PROT_READ | PROT_WRITE, 
		MAP_SHARED | MAP_ANONYMOUS, -1, 0); // shared memory

	NowPlaying2 = mmap(NULL, 16, PROT_READ | PROT_WRITE, 
		MAP_SHARED | MAP_ANONYMOUS, -1, 0); // shared memory

	CurrentPlay = mmap(NULL, 16, PROT_READ | PROT_WRITE, 
		MAP_SHARED | MAP_ANONYMOUS, -1, 0); // shared memory

	Clock = mmap(NULL, 16, PROT_READ | PROT_WRITE, 
		MAP_SHARED | MAP_ANONYMOUS, -1, 0); // shared memory

	begin = mmap(NULL, 16, PROT_READ | PROT_WRITE, 
		MAP_SHARED | MAP_ANONYMOUS, -1, 0); // shared memory

	scount = mmap(NULL, 16, PROT_READ | PROT_WRITE, 
		MAP_SHARED | MAP_ANONYMOUS, -1, 0); // shared memory

	audio = mmap(NULL, 16, PROT_READ | PROT_WRITE, 
		MAP_SHARED | MAP_ANONYMOUS, -1, 0); // shared memory

	videoPlaying = mmap(NULL, 16, PROT_READ | PROT_WRITE, 
		MAP_SHARED | MAP_ANONYMOUS, -1, 0); // shared memory

	skip = mmap(NULL, 16, PROT_READ | PROT_WRITE, 
		MAP_SHARED | MAP_ANONYMOUS, -1, 0); // shared memory

	Pause = mmap(NULL, 16, PROT_READ | PROT_WRITE, 
		MAP_SHARED | MAP_ANONYMOUS, -1, 0); // shared memory

	PauseKill = mmap(NULL, 16, PROT_READ | PROT_WRITE, 
		MAP_SHARED | MAP_ANONYMOUS, -1, 0); // shared memory

	player = mmap(NULL, 16, PROT_READ | PROT_WRITE, 
		MAP_SHARED | MAP_ANONYMOUS, -1, 0); // shared memory

	vlc_hor = mmap(NULL, 16, PROT_READ | PROT_WRITE, 
		MAP_SHARED | MAP_ANONYMOUS, -1, 0); // shared memory

	vlc_ver = mmap(NULL, 16, PROT_READ | PROT_WRITE, 
		MAP_SHARED | MAP_ANONYMOUS, -1, 0); // shared memory

	vlc_wid = mmap(NULL, 16, PROT_READ | PROT_WRITE, 
		MAP_SHARED | MAP_ANONYMOUS, -1, 0); // shared memory

	vlc_hgt = mmap(NULL, 16, PROT_READ | PROT_WRITE, 
		MAP_SHARED | MAP_ANONYMOUS, -1, 0); // shared memory

	OrgPlay = mmap(NULL, 16, PROT_READ | PROT_WRITE, 
		MAP_SHARED | MAP_ANONYMOUS, -1, 0); // shared memory

	TimeNext = mmap(NULL, 128, PROT_READ | PROT_WRITE, 
		MAP_SHARED | MAP_ANONYMOUS, -1, 0); // shared memory

	playonce = mmap(NULL, 128, PROT_READ | PROT_WRITE, 
		MAP_SHARED | MAP_ANONYMOUS, -1, 0); // shared memory

//----------------------------------------------------------------
// program list row data arrays
//----------------------------------------------------------------

	sounder = mmap(NULL, SOUNDER_MAX * 8, PROT_READ | PROT_WRITE, 
		MAP_SHARED | MAP_ANONYMOUS, -1, 0); // shared memory

	sounderTime =  mmap(NULL, SOUNDER_MAX * sizeof(struct ST), PROT_READ | PROT_WRITE, 
		MAP_SHARED | MAP_ANONYMOUS, -1, 0); // shared memory

	sounderPlayed = mmap(NULL, SOUNDER_MAX * sizeof(int), PROT_READ | PROT_WRITE, 
		MAP_SHARED | MAP_ANONYMOUS, -1, 0); // shared memory

	strcpy(TimeNext,"");
	*audsys = PULSE;
	*begin = 0;
	*scount = 0;
	*Pause=0;
	*PauseKill=0;
	*Clock=0;
	*CurrentPlay = -1;
	*NowPlaying = -1;
	*NowPlaying2 = -1;
	vlcDir=0;
	*audio=0; // flag to say if player is runing an audio only
	*videoPlaying=0;
	*skip=0;  // skip next 
	strcpy(next_random,"Next Program Selection");
	strcpy(currentPlay,"Curr Program Selection");
	*vlc_hor=-1;
	*vlc_ver=200;
	*vlc_hgt=100;
	*vlc_wid=200;
	*player = MPV_PLAYER; // 1 = vlc; 0 = mpv
	*playonce = 1;
	*OrgPlay = 0;
	doSounders = 0;
	ubuntu = 0;
	flg18 = 0;	// button18 not pushed
	flg35 = 0;	// button35 not pushed

	char word[64];
	int val;

// -------------------------------------------------------------------------------
// configuration file input
// -------------------------------------------------------------------------------

	s1=fopen("Player.config","r");

	if (s1!=NULL) {
		while (fgets(cmd,1024,s1)!=NULL) { // read line of Player.config
			cmd[strlen(cmd)-1]=0;      // remove newline
			if (cmd[0]=='#') continue; // comment - ignore

			if (strlen(cmd)==0) continue; // empty line

			sscanf(cmd,"%s",word);

			printf("Config -> %s\n",cmd);

			if (strcmp(word,"SOUNDER_MAX")==0) {
				sscanf(cmd,"%s %d",word, &val);
				SOUNDER_MAX=val;
				}

			if (strcmp(word,"PLAYER")==0) {
				sscanf(cmd,"%s %d",word, &val);
				if (val != 0 && val != 1 ) {
					printf("PLAYER specification error. VLC used.\n");
					val = 0; // default
					}
				*player=val; // 0:mpv; 1:vlc
				}

			if (strcmp(word,"PULSE")==0) {
				sscanf(cmd,"%s %d",word, &val);
				if (val != PULSE && val != JACK ) {
					printf("PULSE specification error. PULSE used.\n");
					val = PULSE; // default
					}
				*audsys=val; // 0:JACK; 1:PULSE
				}

			if (strcmp(word,"AUDIO")==0) {
				sscanf(cmd,"%s %d",word, &val);
				if (val != PULSE && val != JACK ) {
					printf("AUDIO specification error. VLC used.\n");
					val = 0; // default
					}
				*audsys=val; // 0:JACK; 1:PULSE
				}

			if (strcmp(word,"PLAYER_HOR")==0) {
				sscanf(cmd,"%s %d",word, &val);
				*vlc_hor=val;
				}

			if (strcmp(word,"PLAYER_WIDE")==0) {
				sscanf(cmd,"%s %d",word, &val);
				*vlc_wid=val;
				}

			if (strcmp(word,"PLAYER_HEIGHT")==0) {
				sscanf(cmd,"%s %d",word, &val);
				*vlc_hgt=val;
				}

			if (strcmp(word,"PLAYER_VER")==0) {
				sscanf(cmd,"%s %d",word, &val);
				*vlc_ver=val;
				}

			if (strcmp(word,"MUSIC_HOME_DIR")==0) {
				sscanf(&cmd[14],"%s",Music_Home_Directory);
				printf("**** Music Home Directory: %s\n", Music_Home_Directory);
				}
			}
		}

//-------------------------------
// read sounder button clip names
//-------------------------------

#define SNDR_MAX 1024

	for (int i=0; i<24; i++) { sndr[i] = (char *) malloc(SNDR_MAX); sndr[i][0]=0; }

	s1 = fopen("sounders.config","r");

	if (s1 == NULL) {
		printf("No sounders config file\n");
		}

	for (int i=0; i<20; i++) {
		if (fgets (cmd,2048,s1) == NULL ) break;
		if (strlen(cmd)==0) break;
		strcpy(sndr[i],Code_Home);
		if (strlen(cmd) + strlen(sndr[i]) > SNDR_MAX) {
			printf("Sender %s name too long\n",cmd);
			strcpy(sndr[i], "");
			}
		else strcat(sndr[i],cmd);
		}

	fclose (s1);

//---------------------------------------------------------------------
//	load sounder file names and allocate memory
//---------------------------------------------------------------------

	copyClipRow=-1;
	strcpy(cutRow,"");
	strcpy(top1,"");
	strcpy(top2,"");
	strcpy(top3,"");
	cutPlayed=0;
	cutPlay=0;
	strcpy(cutTime,"");
	e7Row=0;
	e8Row=0;
	e9Row=0;
	e10Row=0;
	e11Row=0;
	e12Row=0;
	e13Row=0;
	e14Row=0;
	e15Row=0;
	e16Row=0;
	e17Row=0;
	e18Row=0;
	inFlg=1;
	soundersLoading=0;

//----------------------------------------------------------------
//	allocate memory for program entries
//----------------------------------------------------------------

	for (i=0; i < SOUNDER_MAX; i++ ) {

		// allocate shared memory for ST

		sounderTime[i] = mmap(NULL, sizeof(struct ST), PROT_READ | PROT_WRITE, 
			MAP_SHARED | MAP_ANONYMOUS, -1, 0); // shared memory

		strcpy(sounderTime[i]->Time,"");
		sounderTime[i]->play = 0; // play it?
		sounderPlayed[i] = 0;
		}

//	PGM_MAX is the maximum length of a line in the program list

	while(*scount < SOUNDER_MAX) { // blank out entire list

		sounder[*scount] = mmap(NULL, PGM_MAX, PROT_READ | PROT_WRITE, 
			MAP_SHARED | MAP_ANONYMOUS, -1, 0); // shared memory

		strcpy(sounder[*scount],"");
		sounderTime[*scount]->play = 1;
		sounderPlayed[*scount] = 0;
		strcpy(sounderTime[*scount]->Time, "");	// record time stamp
		(*scount)++;
		}

	LoadProgram("program.lst");

//---------------------------------------------------------------------
//	volume control code - not currently active
//---------------------------------------------------------------------
//	sprintf(cmd,"pacmd set-source-volume %d 0x0",AUDIO_MIC_IN); // full
//	system(cmd);
//	sprintf(cmd,"pacmd set-source-volume %d 0x10000",AUDIO_PROG_IN);
//	system(cmd);
//---------------------------------------------------------------------

	widgets = g_slice_new(app_widgets);

	for (i = 0; i< argc; i++) {
		if (strcmp(argv[i], "PULSE") == 0 ) *audsys = PULSE;
		if (strcmp(argv[i], "JACK") == 0 ) *audsys = JACK;
		}
			
	gtk_init(&argc, &argv);

//---------------------------------------------------------------------
// establish contact with xml code used to adjust widget settings
//---------------------------------------------------------------------
 
	builder = gtk_builder_new_from_file ("./Player.glade");
 
	window = GTK_WIDGET(gtk_builder_get_object(builder, "window_main"));

#ifndef WINDOWS
	gtk_builder_connect_signals(builder, NULL);
	#endif

	g_signal_connect(window, "destroy", G_CALLBACK(gtk_main_quit), NULL);

//---------------------------------------------------------------------
// get the definitions of the major components
//---------------------------------------------------------------------

	scroll		= GTK_WIDGET(gtk_builder_get_object(builder, "scroll"));
	fixed1		= GTK_WIDGET(gtk_builder_get_object(builder, "fixed1"));
	dxb1		= GTK_WIDGET(gtk_builder_get_object(builder, "dxb1"));
	dxb2		= GTK_WIDGET(gtk_builder_get_object(builder, "dxb2"));
	g_label1	= GTK_WIDGET(gtk_builder_get_object(builder, "label1"));
	timeNext	= GTK_WIDGET(gtk_builder_get_object(builder, "timeNext"));
	TimeOfDay	= GTK_WIDGET(gtk_builder_get_object(builder, "TimeOfDay"));
	g_label2	= GTK_WIDGET(gtk_builder_get_object(builder, "label2"));
	g_label4	= GTK_WIDGET(gtk_builder_get_object(builder, "label4"));
	showCurrent	= GTK_WIDGET(gtk_builder_get_object(builder, "showCurrent"));
	oneTime		= GTK_WIDGET(gtk_builder_get_object(builder, "oneTime"));
	onetimeImage	= GTK_WIDGET(gtk_builder_get_object(builder, "onetimeImage"));
	clrPgm		= GTK_WIDGET(gtk_builder_get_object(builder, "clrPgm"));
	findVlc		= GTK_WIDGET(gtk_builder_get_object(builder, "findVlc"));
	g_dbox1		= GTK_WIDGET(gtk_builder_get_object(builder, "dbox1"));
	g_dbox1Entry	= GTK_ENTRY(gtk_builder_get_object(builder,  "dbox1Entry"));
	gPlayer		= GTK_WIDGET(gtk_builder_get_object(builder, "player"));
	Audsys		= GTK_WIDGET(gtk_builder_get_object(builder, "Audsys"));
	orgPlay		= GTK_WIDGET(gtk_builder_get_object(builder, "orgPlay"));
	up1		= GTK_WIDGET(gtk_builder_get_object(builder, "up1"));
	up2		= GTK_WIDGET(gtk_builder_get_object(builder, "up2"));
	up3		= GTK_WIDGET(gtk_builder_get_object(builder, "up3"));
	up4		= GTK_WIDGET(gtk_builder_get_object(builder, "up4"));
	up5		= GTK_WIDGET(gtk_builder_get_object(builder, "up5"));
	up6		= GTK_WIDGET(gtk_builder_get_object(builder, "up6"));
	up7		= GTK_WIDGET(gtk_builder_get_object(builder, "up7"));
	up8		= GTK_WIDGET(gtk_builder_get_object(builder, "up8"));
	up9		= GTK_WIDGET(gtk_builder_get_object(builder, "up9"));
	up10		= GTK_WIDGET(gtk_builder_get_object(builder, "up10"));
	up11		= GTK_WIDGET(gtk_builder_get_object(builder, "up11"));
	up12		= GTK_WIDGET(gtk_builder_get_object(builder, "up12"));
	button16	= GTK_WIDGET(gtk_builder_get_object(builder, "button16"));
	Playerx		= GTK_WIDGET(gtk_builder_get_object(builder, "Playerx"));
	check1		= GTK_WIDGET(gtk_builder_get_object(builder, "check1"));
	check2		= GTK_WIDGET(gtk_builder_get_object(builder, "check2"));
	check3		= GTK_WIDGET(gtk_builder_get_object(builder, "check3"));
	check4		= GTK_WIDGET(gtk_builder_get_object(builder, "check4"));
	check5		= GTK_WIDGET(gtk_builder_get_object(builder, "check5"));
	check6		= GTK_WIDGET(gtk_builder_get_object(builder, "check6"));
	check7		= GTK_WIDGET(gtk_builder_get_object(builder, "check7"));
	check8		= GTK_WIDGET(gtk_builder_get_object(builder, "check8"));
	check9		= GTK_WIDGET(gtk_builder_get_object(builder, "check9"));
	check10		= GTK_WIDGET(gtk_builder_get_object(builder, "check10"));
	check11		= GTK_WIDGET(gtk_builder_get_object(builder, "check11"));
	check12		= GTK_WIDGET(gtk_builder_get_object(builder, "check12"));
	del1		= GTK_WIDGET(gtk_builder_get_object(builder, "del1"));
	del2		= GTK_WIDGET(gtk_builder_get_object(builder, "del2"));
	del3		= GTK_WIDGET(gtk_builder_get_object(builder, "del3"));
	del4		= GTK_WIDGET(gtk_builder_get_object(builder, "del4"));
	del7		= GTK_WIDGET(gtk_builder_get_object(builder, "del7"));
	del8		= GTK_WIDGET(gtk_builder_get_object(builder, "del8"));
	del9		= GTK_WIDGET(gtk_builder_get_object(builder, "del9"));
	del10		= GTK_WIDGET(gtk_builder_get_object(builder, "del10"));
	del11		= GTK_WIDGET(gtk_builder_get_object(builder, "del11"));
	del12		= GTK_WIDGET(gtk_builder_get_object(builder, "del12"));
	del13		= GTK_WIDGET(gtk_builder_get_object(builder, "del13"));
	del14		= GTK_WIDGET(gtk_builder_get_object(builder, "del14"));
	save1		= GTK_WIDGET(gtk_builder_get_object(builder, "save1"));
	sndr1		= GTK_WIDGET(gtk_builder_get_object(builder, "sndr1"));
	sndr2		= GTK_WIDGET(gtk_builder_get_object(builder, "sndr2"));
	sndr3		= GTK_WIDGET(gtk_builder_get_object(builder, "sndr3"));
	sndr4		= GTK_WIDGET(gtk_builder_get_object(builder, "sndr4"));
	sndr5		= GTK_WIDGET(gtk_builder_get_object(builder, "sndr5"));
	sndr6		= GTK_WIDGET(gtk_builder_get_object(builder, "sndr6"));
	sndr7		= GTK_WIDGET(gtk_builder_get_object(builder, "sndr7"));
	sndr8		= GTK_WIDGET(gtk_builder_get_object(builder, "sndr8"));
	sndr9		= GTK_WIDGET(gtk_builder_get_object(builder, "sndr9"));
	sndr10		= GTK_WIDGET(gtk_builder_get_object(builder, "sndr10"));
	sndr11		= GTK_WIDGET(gtk_builder_get_object(builder, "sndr11"));
	sndr12		= GTK_WIDGET(gtk_builder_get_object(builder, "sndr12"));
	sndr13		= GTK_WIDGET(gtk_builder_get_object(builder, "sndr13"));
	sndr14		= GTK_WIDGET(gtk_builder_get_object(builder, "sndr14"));
	sndr15		= GTK_WIDGET(gtk_builder_get_object(builder, "sndr15"));
	sndr16		= GTK_WIDGET(gtk_builder_get_object(builder, "sndr16"));
	sndr17		= GTK_WIDGET(gtk_builder_get_object(builder, "sndr17"));
	sndr18		= GTK_WIDGET(gtk_builder_get_object(builder, "sndr18"));
	sndr19		= GTK_WIDGET(gtk_builder_get_object(builder, "sndr19"));
	sndr20		= GTK_WIDGET(gtk_builder_get_object(builder, "sndr20"));
	g_image1	= GTK_WIDGET(gtk_builder_get_object(builder, "image1"));
	first		= GTK_WIDGET(gtk_builder_get_object(builder, "first"));
	g_image9	= GTK_WIDGET(gtk_builder_get_object(builder, "image9"));
	aud1		= GTK_WIDGET(gtk_builder_get_object(builder, "aud1"));
	aud2		= GTK_WIDGET(gtk_builder_get_object(builder, "aud2"));
	g_button1	= GTK_WIDGET(gtk_builder_get_object(builder, "button1"));
	g_button2	= GTK_WIDGET(gtk_builder_get_object(builder, "button2"));
	g_button3	= GTK_WIDGET(gtk_builder_get_object(builder, "button3"));
	g_button4	= GTK_WIDGET(gtk_builder_get_object(builder, "button4"));
	g_button5	= GTK_WIDGET(gtk_builder_get_object(builder, "button5"));
	g_button6	= GTK_WIDGET(gtk_builder_get_object(builder, "button6"));
	g_button7	= GTK_WIDGET(gtk_builder_get_object(builder, "button7"));
	g_button8	= GTK_WIDGET(gtk_builder_get_object(builder, "button8"));
	g_button9	= GTK_WIDGET(gtk_builder_get_object(builder, "button9"));
	g_button10	= GTK_WIDGET(gtk_builder_get_object(builder, "button10"));
	g_button11	= GTK_WIDGET(gtk_builder_get_object(builder, "button11"));
	g_button12	= GTK_WIDGET(gtk_builder_get_object(builder, "button12"));
	g_button13	= GTK_WIDGET(gtk_builder_get_object(builder, "button13"));
	g_button14	= GTK_WIDGET(gtk_builder_get_object(builder, "button14"));
	g_button15	= GTK_WIDGET(gtk_builder_get_object(builder, "button15"));
	g_button18	= GTK_WIDGET(gtk_builder_get_object(builder, "button18"));
	g_button30	= GTK_WIDGET(gtk_builder_get_object(builder, "button30"));
	g_button31	= GTK_WIDGET(gtk_builder_get_object(builder, "button31"));
	g_button32	= GTK_WIDGET(gtk_builder_get_object(builder, "button32"));
	g_button33	= GTK_WIDGET(gtk_builder_get_object(builder, "button33"));
	g_button34	= GTK_WIDGET(gtk_builder_get_object(builder, "button34"));
	g_button35	= GTK_WIDGET(gtk_builder_get_object(builder, "button35"));
	g_button37	= GTK_WIDGET(gtk_builder_get_object(builder, "button37"));
	g_button52	= GTK_WIDGET(gtk_builder_get_object(builder, "button52"));
	g_button57	= GTK_WIDGET(gtk_builder_get_object(builder, "button57"));
	g_button58	= GTK_WIDGET(gtk_builder_get_object(builder, "button58"));
	g_btn_hello	= GTK_WIDGET(gtk_builder_get_object(builder, "btn_hello"));
	g_btn_hello1	= GTK_WIDGET(gtk_builder_get_object(builder, "btn_hello1"));
	g_rnd_btn1	= GTK_WIDGET(gtk_builder_get_object(builder, "rnd_btn1"));
	g_rnd_btn2	= GTK_WIDGET(gtk_builder_get_object(builder, "rnd_btn2"));
	searchEntry1	= GTK_SEARCH_ENTRY(gtk_builder_get_object(builder, "searchentry1"));
	searchEntry2	= GTK_SEARCH_ENTRY(gtk_builder_get_object(builder, "searchentry2"));
	g_lbl_entry1	= GTK_ENTRY(gtk_builder_get_object(builder, "entry1"));
	g_lbl_entry2	= GTK_ENTRY(gtk_builder_get_object(builder, "entry2"));
	g_lbl_entry3	= GTK_ENTRY(gtk_builder_get_object(builder, "entry3"));
	g_lbl_entry4	= GTK_ENTRY(gtk_builder_get_object(builder, "entry4"));
	g_lbl_entry5	= GTK_ENTRY(gtk_builder_get_object(builder, "entry5"));
	g_lbl_entry6	= GTK_ENTRY(gtk_builder_get_object(builder, "entry6"));
	g_lbl_entry7	= GTK_ENTRY(gtk_builder_get_object(builder, "entry7"));
	g_lbl_entry8	= GTK_ENTRY(gtk_builder_get_object(builder, "entry8"));
	g_lbl_entry9	= GTK_ENTRY(gtk_builder_get_object(builder, "entry9"));
	g_lbl_entry10	= GTK_ENTRY(gtk_builder_get_object(builder, "entry10"));
	g_lbl_entry11	= GTK_ENTRY(gtk_builder_get_object(builder, "entry11"));
	g_lbl_entry12	= GTK_ENTRY(gtk_builder_get_object(builder, "entry12"));
	g_lbl_entry13	= GTK_ENTRY(gtk_builder_get_object(builder, "entry13"));
	g_lbl_entry14	= GTK_ENTRY(gtk_builder_get_object(builder, "entry14"));
	g_lbl_entry15	= GTK_ENTRY(gtk_builder_get_object(builder, "entry15"));
	g_btn_play	= GTK_WIDGET(gtk_builder_get_object(builder, "button1"));
	g_btn_play2	= GTK_WIDGET(gtk_builder_get_object(builder, "button4"));

	// make these button pointers available to the timer function

	widgets -> w_lbl_nowplay = g_label1;
	widgets -> w_lbl_timeNext = timeNext;
	widgets -> w_lbl_TimeOfDay = TimeOfDay;
	widgets -> w_lbl_button2 = g_button2;
	widgets -> w_lbl_button3 = g_button3;
	widgets -> w_lbl_button5 = g_button5;
	widgets -> w_lbl_button6 = g_button6;
	widgets -> w_lbl_button8 = g_button8;
	widgets -> w_lbl_button9 = g_button9;
	widgets -> w_lbl_button10 = g_button10;
	widgets -> w_lbl_button11 = g_button11;
	widgets -> w_lbl_button12 = g_button12;
	widgets -> w_lbl_button13 = g_button13;
	widgets -> w_lbl_button14 = g_button14;
	widgets -> w_lbl_button15 = g_button15;
	widgets -> w_lbl_button18 = g_button18;
	widgets -> w_lbl_button31 = g_button31;
	widgets -> w_label4 = g_label4;
	widgets -> w_label2 = g_label2;

//---------------------------------------------------------------------
//	css providers
//---------------------------------------------------------------------

	cssProvider1  = gtk_css_provider_new();
	gtk_css_provider_load_from_data(cssProvider1, css0, -1, NULL);

	cssProvider2  = gtk_css_provider_new();
	gtk_css_provider_load_from_data(cssProvider2, css1, -1, NULL);

	setCssAttributes(cssProvider1); // set initial css attributes of widgets

	PangoAttributes(); // attach Pango related attributes to selected widgets

//---------------------------------------------------------------------
// set width of search boxes based on distribution
//---------------------------------------------------------------------

	int entryWidth;

	s1=popen("cat /etc/issue* | grep Ubuntu" , "r");

	if (s1 != NULL) {
		tmp[0]=0;
		fgets(tmp,1024,s1);
		if (strlen(tmp) != 0) ubuntu = 1;
		fclose(s1);
		}

	if ( ubuntu ) entryWidth = 18; else entryWidth = 22;

	gtk_entry_set_width_chars (GTK_ENTRY(searchEntry1), entryWidth);
	gtk_entry_set_width_chars (GTK_ENTRY(searchEntry2), entryWidth);

	gtk_widget_set_events(g_button18, GDK_SCROLL_MASK); // page up/down/first/last
	gtk_widget_set_events(g_button35, GDK_SCROLL_MASK);
	gtk_widget_set_events(g_button58, GDK_SCROLL_MASK);
	gtk_widget_set_events(g_button33, GDK_SCROLL_MASK);
	gtk_widget_set_events(g_button3,  GDK_SCROLL_MASK);
	gtk_widget_set_events(g_button8,  GDK_SCROLL_MASK);
	gtk_widget_set_events(g_button9,  GDK_SCROLL_MASK);
	gtk_widget_set_events(g_button2,  GDK_SCROLL_MASK);
	gtk_widget_set_events(g_button5,  GDK_SCROLL_MASK);
	gtk_widget_set_events(g_button6,  GDK_SCROLL_MASK);
	gtk_widget_set_events(g_button10, GDK_SCROLL_MASK);
	gtk_widget_set_events(g_button11, GDK_SCROLL_MASK);
	gtk_widget_set_events(g_button13, GDK_SCROLL_MASK);
	gtk_widget_set_events(g_button12, GDK_SCROLL_MASK);
	gtk_widget_set_events(g_button14, GDK_SCROLL_MASK);
	gtk_widget_set_events(g_button15, GDK_SCROLL_MASK);

//---------------------------------------------------------------------
// load the first 24 program buttons with content
//---------------------------------------------------------------------

	sounders();	// first 24 buttons loaded
	*begin = 0;	// next window begins here

	g_object_unref(builder);

	for (int i=0; i<5; i++) strcpy(directories[i],Music_Home_Directory);

	g_timeout_add_seconds(1, (GSourceFunc) timer_handler, widgets);

	gtk_widget_show(window);                

	chdir(Music_Home_Directory);

	gtk_widget_show(onetimeImage);                

	system("wmctrl -r \"Broadcast Player.jpg\" -b add,above");

	if (*audsys == JACK) {
		gtk_button_set_label (GTK_BUTTON(Audsys), (const gchar* ) "Jack");
		}
	else {
		gtk_button_set_label (GTK_BUTTON(Audsys), (const gchar* ) "Pulse");
		}

	GdkColor color; // default background color
	color.red = 0x3333;
	color.green = 0x3333;
	color.blue = 0x6666;
	gtk_widget_modify_bg(window, GTK_STATE_NORMAL, &color);

	inFlg=0;

	gtk_window_set_transient_for (GTK_WINDOW(g_dbox1), GTK_WINDOW(window));
//	gtk_widget_hide(g_dbox1);                

	*Pause = 1; // will be reset in following function to 0
	*PauseKill = 0; // will be reset in following function to 0
	on_button34_clicked (GTK_BUTTON(g_button34)); // show pause off icon

	*playonce = 0;
	on_oneTime_clicked (GTK_BUTTON(oneTime));

	ClearTimeOfNext();

	gtk_range_set_range (GTK_RANGE(scroll), 0, *scount);
	gtk_widget_modify_bg(scroll, GTK_STATE_NORMAL, &color); // same as BG color

//	load up to 8 sweeper buttons
//	may be able to squeeze a 9th

	i=0;
	while (1) {
		if (strlen(sndr[i]) == 0 ) break;
		for (j=strlen(sndr[i]); sndr[i][j] != '/'; j--);
		strcpy(tmp,&sndr[i][j+1]);
		for (j=0; tmp[j] != '.' && tmp[j] != 0; j++);
		tmp[j]=0; // file extension
		tmp[9]=0; // limit
		if (i == 0 ) gtk_button_set_label (GTK_BUTTON(sndr1), tmp);
		else if (i == 1 ) gtk_button_set_label (GTK_BUTTON(sndr2), tmp);
		else if (i == 2 ) gtk_button_set_label (GTK_BUTTON(sndr3), tmp);
		else if (i == 3 ) gtk_button_set_label (GTK_BUTTON(sndr4), tmp);
		else if (i == 4 ) gtk_button_set_label (GTK_BUTTON(sndr5), tmp);
		else if (i == 5 ) gtk_button_set_label (GTK_BUTTON(sndr6), tmp);
		else if (i == 6 ) gtk_button_set_label (GTK_BUTTON(sndr7), tmp);
		else if (i == 7 ) gtk_button_set_label (GTK_BUTTON(sndr8), tmp);
		i++;
		}

	if (WhatAudio() != *audsys ) {
		printf("****************************\n");
		printf("****************************\n");
		printf("****************************\n");
		printf("   audio system mismatch\n");
		printf("****************************\n");
		printf("****************************\n");
		printf("****************************\n");
		return EXIT_FAILURE;
		}

//---------------------------------------------------------------------
//	From here on, it's all callbacks
//---------------------------------------------------------------------

	gtk_main();

	return EXIT_SUCCESS;
	}

// ----------------------------------------------------------------
// The Timer
// Handler for the 1 second timer tick
// ----------------------------------------------------------------

gboolean timer_handler(app_widgets *widgets) {

	gint x,y;
	char cmd[1024];
	char cmd1[1024];
	int i,j;
	double lev;
	char tmp[1024],dmy[16];
	static int stopBlink = 0;

	stopBlink = stopBlink ? 0 : 1; // alternate

	if (*PauseKill == 1) {
		if (stopBlink) gtk_widget_show(g_button31);                
		else gtk_widget_hide(g_button31);                
		}

	if (*PauseKill == 2 ) {
		on_button31_clicked(GTK_BUTTON(g_button31));
		*PauseKill = 0;
		gtk_widget_show(g_button31);                
		}

	if (*vlc_hor == -1 ) {
		gtk_window_get_position(GTK_WINDOW(window), &x, &y);
		*vlc_hor = x +200;
		*vlc_ver = y + 300;
		}

	time_t t = time(0);

	sscanf(ctime(&t),"%s %s %s %s ",dmy,dmy,dmy,dmy);
	strcpy(tmp,"Now: ");
	strcat(tmp,dmy);
	gtk_label_set_text(GTK_LABEL(widgets->w_lbl_TimeOfDay), tmp);    // update time of day

	//flg18a indicates that an advance has taken place - supress the 'clicked' signal

	if (flg18) { AdvanceSounderPage(); flg18a = 1; }

	//flg35a indicates that an advance has taken place - supress the 'clicked' signal

	if (flg35) {
		flg35a = 1;
		*begin = *begin - SOUNDER_BUTTON_COUNT;
		if (*begin <= 0) { *begin = 0; }	// *begin is on last page
		*NowPlaying = -1;
		sounders();
		}

//--------------------------------------------
//	show and decrement clock as needed
//--------------------------------------------

	gtk_label_set_text(GTK_LABEL(widgets->w_lbl_timeNext), TimeNext);    // update label

	if (*player == VLC_PLAYER) {  // vlc

		if (*Clock > 0) { 
			sprintf(tmp,"%d", *Clock);
			tmp[30] = 0; // mix allowed length
			gtk_label_set_text(GTK_LABEL(widgets->w_lbl_nowplay), tmp);    // update label
			*Clock = *Clock-1;
			}
		else {
			gtk_label_set_text(GTK_LABEL(widgets->w_lbl_nowplay), "Timer");    // update label
			}
		}

	else {  // mpv
		FILE *f1;
		strcpy(cmd, "tail -n 1 ");
		strcat(cmd, Code_Home);
		strcat(cmd, "/mpvTmp 2>/dev/null");
		strcpy(cmd1, "");
		f1 = popen(cmd, "r");
		if (f1 != NULL)	{
			fgets(cmd1, 1024, f1);
			cmd1[strlen(cmd1)-1] = 0;

			if (strncmp(cmd1, "tail:", 5) != 0) {
				// a paren terminates the percentage played field
				for(int i = 0; cmd1[i] != 0 && i< 100; i++) {
					if (cmd1[i] == ')') {
						cmd1[i+1] = 0;
						break;
						}
					}
				cmd1[30] = 0;
				if (strlen(cmd1) == 0 ) strcpy(cmd1,"Timer");
				gtk_label_set_text(GTK_LABEL(widgets->w_lbl_nowplay), cmd1);    // update label
				}
			else gtk_label_set_text(GTK_LABEL(widgets->w_lbl_nowplay), "Timer");    // update label
			fclose(f1);
			}
		else gtk_label_set_text(GTK_LABEL(widgets->w_lbl_nowplay), "Timer");    // update label
		}

	if (*player == VLC_PLAYER) {
		gtk_button_set_label (GTK_BUTTON(gPlayer), (const gchar* ) "VLC");
		}
	else {
		gtk_button_set_label (GTK_BUTTON(gPlayer), (const gchar* ) "MPV");
		}

	if (*audsys == PULSE) {
		gtk_button_set_label (GTK_BUTTON(Audsys), (const gchar* ) "Pulse");
		}
	else {
		gtk_button_set_label (GTK_BUTTON(Audsys), (const gchar* ) "Jack");
		}

//--------------------------------------------
//	display next selection
//--------------------------------------------

	strcpy(tmp,next_random); tmp[85]=0; // short form displayed
	gtk_label_set_text(GTK_LABEL(widgets->w_label4), tmp);    // update label

	strcpy(tmp,currentPlay); tmp[85]=0; // short form displayed
	gtk_label_set_text(GTK_LABEL(widgets->w_label2), tmp);    // update label


	if (*NowPlaying != -1) {

		*CurrentPlay = *NowPlaying;

		sounderPlayed[*NowPlaying]++; //number of times played this session

//		*begin = *NowPlaying-6;	// re-display sounder buttons with current in middle
		*begin = *NowPlaying;	// re-display sounder buttons with current at beginning

		if (*begin<0) *begin=0;

		sounders();  // repaint program display

		*NowPlaying=-1;
		}

	while (gtk_events_pending ()) gtk_main_iteration ();

	return TRUE;
	}
 
// ----------------------------------------------------------------
// Window is closed
// Cleanup dependent windows
// ----------------------------------------------------------------

void on_window_main_destroy() { 

	if (PlaySelected > 0) { 
		kill(PlaySelected,SIGTERM); 
		PlaySelected = -1; 
		*NowPlaying=-1;
		}

	ClearTimeOfNext();
	system("killall --signal SIGTERM vlc 2> /dev/null");
	system(mpvFile);
	system("killall --signal SIGTERM mpv 2> /dev/null");
	system("killall --signal SIGTERM Player-bin");

	gtk_main_quit(); 
	}
 
// ----------------------------------------------------------------
// sounder button content load
// ----------------------------------------------------------------

void	sounders() {

	e7Row  = -1;
	e8Row  = -1;
	e9Row  = -1;
	e10Row = -1;
	e11Row = -1;
	e12Row = -1;
	e13Row = -1;
	e14Row = -1;
	e15Row = -1;
	e16Row = -1;
	e17Row = -1;
	e18Row = -1;

	soundersLoading = 1;
	sounderButton(g_button3,  *begin+0);
	sounderButton(g_button8,  *begin+1);
	sounderButton(g_button9,  *begin+2);
	sounderButton(g_button10, *begin+3);
	sounderButton(g_button11, *begin+4);
	sounderButton(g_button12, *begin+5);
	sounderButton(g_button13, *begin+6);
	sounderButton(g_button14, *begin+7);
	sounderButton(g_button2,  *begin+8);
	sounderButton(g_button5,  *begin+9);
	sounderButton(g_button6,  *begin+10);
	sounderButton(g_button15, *begin+11);

	if (sounderTime[*begin+0]->play) gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON(check1),TRUE);
	else gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON(check1),FALSE);

	if (sounderTime[*begin+1]->play) gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON(check2),TRUE);
	else gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON(check2),FALSE);

	if (sounderTime[*begin+2]->play) gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON(check3),TRUE);
	else gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON(check3),FALSE);

	if (sounderTime[*begin+3]->play) gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON(check4),TRUE);
	else gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON(check4),FALSE);

	if (sounderTime[*begin+4]->play) gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON(check5),TRUE);
	else gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON(check5),FALSE);

	if (sounderTime[*begin+5]->play) gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON(check6),TRUE);
	else gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON(check6),FALSE);

	if (sounderTime[*begin+6]->play) gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON(check7),TRUE);
	else gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON(check7),FALSE);

	if (sounderTime[*begin+7]->play) gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON(check8),TRUE);
	else gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON(check8),FALSE);

	if (sounderTime[*begin+8]->play) gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON(check9),TRUE);
	else gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON(check9),FALSE);

	if (sounderTime[*begin+9]->play) gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON(check10),TRUE);
	else gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON(check10),FALSE);

	if (sounderTime[*begin+10]->play) gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON(check11),TRUE);
	else gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON(check11),FALSE);

	if (sounderTime[*begin+11]->play) gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON(check12),TRUE);
	else gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON(check12),FALSE);

	soundersLoading = 0;
	gtk_range_set_value (GTK_RANGE(scroll),*begin);
	}

// ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// program forward 
// ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

void on_button18_clicked (GtkButton *button, GtkEntry *user_data) {
	AdvanceSounderPage();
	}

// ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// play sequential program
// ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

void	on_button30_clicked (GtkButton *button) {
	int j,i,ii,itype;
	char cmd[1024];
	char cmd1[1024];
	FILE *f1;
	gint x,y;

	*PauseKill = 0;

	setButton(GTK_BUTTON(button)); // set this buuton on

	*audio=0;

	// turn off rand pgm button (if on)
	// turn off org pgm button (if on)

	clearButton(GTK_BUTTON(g_button32));

	if (PlaySelected > 0) {  // process id of player - kill any prior
		kill(PlaySelected,9); 
		PlaySelected=-1; 
		*NowPlaying=-1;
		}

	ClearPlayers();	// kill any existing media players

	PlaySelected = fork();  // split to child process to manage players

	if (PlaySelected) return; // if non-zero this thread is the parent

// ----------------- forked child code follows ----------------------------------

	while (1) {

		i = *begin;
		while (i < *scount ) {

			if ( !sounderTime[i]->play ) { i++; continue;}	// play check box not checked

			*NowPlaying = i;
			*NowPlaying2 = i; // can changed by right click on sounder btns

			if (strlen(sounder[i]) == 0) break;	// empty entry - halt

			if (*playonce == 1) sounderTime[i]->play = 0; // this has been played

			MediaProcess(&i, cmd);

			if (*Pause) { 	// Pause button pressed
				if (*PauseKill) {
					ClearPlayers();
					*Pause = 0;
					*PauseKill = 2;
					*Clock = 0;
					*videoPlaying=0;
					exit(EXIT_SUCCESS);
					}
				while (*Pause) sleep(1); 
				}

        		system(cmd);		// play file - run as separate task (&)
			#include "vlc-place.h"	// common player placement code

			if (!*OrgPlay) *NowPlaying2 = i; // Org button not active
	
			else {	// Org button active
	
				*NowPlaying = -1;
	
				for (int i = 0; i++; i < *scount) { // find a playable entry
					if ( sounderTime[i]->play) {
						*NowPlaying = i;
						break;
						}
					}
				}

			if (strcmp(sounderTime[i]->Time, "0") == 0) { GetTime(i, sounder[i]); }
	
			*Clock = atoi(sounderTime[i]->Time);	// play time
			TimeOfNext();
	
			sleep(1);    // wait delay until player is running
	
			// locate first / file mark from the end of next to be played
	
			for (j = strlen(sounder[i+1]); j >= 0 && sounder[i+1][j] != '/'; j--);
	
			// copy number and name of next to be played
	
			sprintf(next_random,"Next: %d %s", i+1, &sounder[i+1][j+1]);

			while (1) {
				if (*OrgPlay) ii = 0; // Org mode starts from 0

				else { // not OrgPlay
					ii = i;
					if ( ii >= *scount ) ii = 0;
					}
	
				*NowPlaying2 = ii;
	
				while (1) {
					if ( !sounderTime[ii]->play ) { // can this be played?
						*NowPlaying2 = ii;
						if (ii >= *scount) {
							*NowPlaying2 = -1; // can't find one
							ii=0;
							break;
							}
						ii++;
						continue;	// iterate
						}
					break;	// found one
					}
	
				if (*NowPlaying2 < 0 ) exit (EXIT_FAILURE);
	
				if ( strlen(sounder[ii]) ) {
					for (j = strlen(sounder[ii]); j >= 0 && sounder[ii][j] != '/'; j--);
					sprintf(next_random,"Next: %d %s", ii, &sounder[ii][j+1]);
					}

				else strcpy(next_random,"Next: <empty>");

				if (*skip) {
					*skip = 0;
					sounderTime[ii]->play = 0;
					continue;
					}

				if (VideoRunning()) { // wait for player to terminate
					*videoPlaying=1;
					sleep(1);
					continue;
					}
				break;
				}

			ClearPlayers();	// kill any media player

			*videoPlaying=0;

			if (*OrgPlay) { 
				*begin = 0; 
				break; // restart iterative loop
				}

			i ++;
			} // iterative loop

		} // while loop

	*Clock = 0;
	*videoPlaying=0;
	exit(EXIT_SUCCESS);
	}

// ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// clear program
// ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

void	on_button31_clicked (GtkButton *button) {

	*Clock = 0;
	*NowPlaying = -1;
	*begin=0;
	*Pause=0;
	*PauseKill = 0;
	gtk_widget_show(g_button31);                
	*OrgPlay = 0;
	*CurrentPlay = -1;

	PauseOff(); // turn button to green

	strcpy(next_random, "Next Program Selection");
	strcpy(currentPlay, "Curr Program Selection");

	if (PlaySelected > 0) { 
		kill(PlaySelected,9); 
		PlaySelected = -1; 
		*NowPlaying = -1;
		}

	ClearPlayers();

	usleep(500000);

	sounders();

	clearButton(GTK_BUTTON(g_button30));
	clearButton(GTK_BUTTON(g_button32));
	clearButton(GTK_BUTTON(orgPlay));
	clearButton(GTK_BUTTON(g_button3)); 
	clearButton(GTK_BUTTON(g_button8));
	clearButton(GTK_BUTTON(g_button9));
	clearButton(GTK_BUTTON(g_button10));
	clearButton(GTK_BUTTON(g_button11));
	clearButton(GTK_BUTTON(g_button12));
	clearButton(GTK_BUTTON(g_button13));
	clearButton(GTK_BUTTON(g_button14));
	clearButton(GTK_BUTTON(g_button2));
	clearButton(GTK_BUTTON(g_button5));
	clearButton(GTK_BUTTON(g_button6));
	clearButton(GTK_BUTTON(g_button15));
	}

//	clear program at end of current

gboolean on_button31_button_press_event 
	(GtkButton *btn, GdkEventButton *event, gpointer user_data) { 
	if (event->button == 1 ) return FALSE;
	if (*videoPlaying == 0 ) return TRUE;
	if (*PauseKill == -1 ) return TRUE;
	*PauseKill = 1;
	*Pause = 1;
	return TRUE;
	}

// ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// rand program
// ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

void	on_button32_clicked (GtkButton *button) {
	int i,itype,j,k;
	char cmd[2048];
	char cmd1[2048];
	gint x,y;
	static int rgen = 1;

//	randomize the entries

	for (i=0; i<*scount; i++) {        // shift entries up one
		j = rand() % *scount;

		strcpy(cmd, sounder[i]);
		strcpy(sounder[i], sounder[j]);
		strcpy(sounder[j], cmd);

		strcpy(cmd, sounderTime[i]->Time);
		strcpy(sounderTime[i]->Time, sounderTime[j]->Time);
		strcpy(sounderTime[j]->Time, cmd);

		k = sounderTime[i]->play;
		sounderTime[i]->play = sounderTime[j]->play;
		sounderTime[j]->play = k;

		k = sounderPlayed[i];
		sounderPlayed[i] = sounderPlayed[j];
		sounderPlayed[j] = k;
		}
	sounders();
	return;

	*PauseKill = 0;

	// highlight rand button

	setButton(GTK_BUTTON(button));
	*audio=0;

	// turn off rand pgm button (if on)
	// turn off org pgm button (if on)

	clearButton(GTK_BUTTON(g_button30));
	clearButton(GTK_BUTTON(orgPlay));

	*OrgPlay = 0;

	if (PlaySelected > 0) { 
		kill(PlaySelected, SIGTERM); 
		PlaySelected = -1; 
		*NowPlaying =  -1;
		}

	ClearPlayers();

	usleep(500000);

	if (rgen) { // init random number generator
		srand(time(0));	// init random number generator
		rgen = 0; 	// do it only once
		}

	PlaySelected = fork();

//---------- forked code ------------------------------------------

	char * nme;

	if (PlaySelected) { // we are the parent from the fork()
		return;
		}

	int retry = 0; // to prevent infinite loops

	i = 0;	// always begin with the first

	if (strlen(sounder[i]) == 0 || sounderTime[i]->play == 0)  { // 1st not playable

		while(1) { // first selection

			retry ++;
			if (retry > *scount) {
				Message("No unplayed files remain.");    // update label
				return;
				}
	
			i = rand() % *scount;
			if (strlen(sounder[i]) == 0) continue; // no entry here
			if (sounderTime[i]->play == 0 ) continue; // this has been played
			break; // i is playable
			}
		}

	while (1) {

		MediaProcess(&i, cmd);

		if (*Pause) { 
			if (*PauseKill) {
				ClearPlayers();
				*Pause = 0;
				*PauseKill = 2;
				*Clock = 0;
				*videoPlaying=0;
				exit(EXIT_SUCCESS);
				}
			while (*Pause) { sleep(1); } 
			}

		*NowPlaying = i;

		printf("+++ %s\n",sounder[i]);

		strcpy(cmd1, sounder[i]);
		nme = sounderName(cmd1,i);
		for(int k=0; nme[k] != 0; k++) if (nme[k] == '\n') nme[k]=' ';
		sprintf(currentPlay,"Curr: %d %s", i, nme);

		if (*playonce == 1) sounderTime[i]->play = 0; // this has been played

                system(cmd); // PLAY forked

		#include "vlc-place.h"

		if (strcmp(sounderTime[i]->Time, "0") == 0) {
			GetTime(i, sounder[i]);
			}
		*Clock = atoi(sounderTime[i]->Time);
		TimeOfNext();

                sleep(1);    // wait until playing

// skp label is for re-selection reentry

skp:		retry = 0; 
		while(1) { // next selection

			if (sounderTime[0]->play == 1) { // 1st slot has been repopulated
				sounderTime[0]->play = 2; // prevents infinite loop
				i = 0; 
				break; 
				}
			retry ++;

			if (retry > *scount) {
				Message("No unplayed files remain.");    // update label
				return;
				}

			i = rand() % *scount;
			if (strlen(sounder[i]) == 0) continue;
			if (sounderTime[i]->play == 0 ) continue;
			break;
			}

		strcpy(cmd1, sounder[i]);

		nme = sounderName(cmd1,i);

		for(int k=0; nme[k] != 0; k++) if (nme[k] == '\n') nme[k]=' ';
		sprintf(next_random, "Next: %d %s", i, nme);

                while (1) { // wait until done

			if (sounderTime[0]->play == 1) { // 1st slot has been repopulated
				i = 0; 
				goto skp;
				}

			if (*skip) {
				*skip = 0;
				goto skp; // get a new next selection
				}

			if (VideoRunning()) {
				sleep(1);
				*videoPlaying = 1;
				continue;
				}

			ClearPlayers();

			PlaySelected = -1;
			*videoPlaying = 0;
                        break;
			}
		}

	*videoPlaying = 0;
	*Clock = 0;
        exit(EXIT_SUCCESS);
	}

// ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// org program
// ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

void	on_button33_clicked (GtkButton *button) {
	*PauseKill = 0;
	orgPgm();
	}
	
void	orgPgm() {
	*PauseKill = 0;
	*begin = 0;
	*NowPlaying = -1;
	sounders();
	}

// ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// last program
// ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

void	on_button58_clicked (GtkButton *button) {
	
	*begin=*scount - SOUNDER_BUTTON_COUNT;
	if (*begin < 0 ) *begin = 0;
	sounders();
	}

// ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// pause program
// ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

void	on_button34_clicked (GtkButton *b) {

	GtkWidget *pauseOn, *pauseOff;
	char cmd[1024];

	strcpy(cmd,Code_Home);
	strcat(cmd,"/");

	if (*Pause) {	// end pause
		*Pause = 0; 
		strcat(cmd,"backgrounds/pauseOff.png"); // green
		pauseOff = gtk_image_new_from_file (cmd);
		gtk_button_set_image(GTK_BUTTON(b),pauseOff);
		}

	else {	// pausing
		*Pause = 1;
		strcat(cmd,"backgrounds/pauseOn.png"); //red
		pauseOn = gtk_image_new_from_file (cmd);
		gtk_button_set_image(GTK_BUTTON(b),pauseOn);
		}

	}

void	PauseOff () {

	GtkWidget *pauseOff;
	char cmd[1024];

	strcpy(cmd,Code_Home);
	strcat(cmd,"/");

	*Pause = 0; 
	strcat(cmd,"backgrounds/pauseOff.png"); // green
	pauseOff = gtk_image_new_from_file (cmd);
	gtk_button_set_image(GTK_BUTTON(g_button34),pauseOff);
	}

// ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// program back 
// ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

void on_button35_clicked (GtkButton *button, GtkEntry *user_data) {

	int i = 0,j = 0,k = 0;
	
	*begin = *begin - SOUNDER_BUTTON_COUNT;
	if (*begin <= 0) { *begin = 0; }	// *begin is on last page
	*NowPlaying = -1;
	sounders();
	}

// ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// play next
// ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

void on_button37_clicked (GtkButton *button, GtkEntry *user_data) {

	int hx,d1;
	char cmd[1024];

//----------------------------------
// locate the current player window
//----------------------------------

	*NowPlaying = -1;		// indicate action needed

	ClearPlayers();

	}

// ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// row 1 code
// ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

//-----------------------------------------------------------------
// File chooser 
//-----------------------------------------------------------------

void on_btn_hello_clicked (GtkButton *button, GtkEntry *user_data) {
	FileChooser(button, user_data, 0, g_btn_play);
	}

//-----------------------------------------------------------------
// Play row 
//-----------------------------------------------------------------

void on_button1_clicked (GtkButton *button, GtkEntry *user_data) {
	*Clock = 0;
	PlayClip (button, user_data, -10);
	}

//-----------------------------------------------------------------
// Random file chooser
//-----------------------------------------------------------------

void on_rnd_btn1_clicked (GtkButton *button, GtkEntry *user_data) {
	RandomPlay (button, user_data, g_btn_play, 0); 
	}

// ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// row 2 code
// ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

//-----------------------------------------------------------------
// File chooser 
//-----------------------------------------------------------------

void on_btn_hello1_clicked (GtkButton *button, GtkEntry *user_data) {
	FileChooser(button, user_data, 1, g_btn_play2);
	}

//-----------------------------------------------------------------
// Random file chooser 
//-----------------------------------------------------------------

void on_rnd_btn2_clicked (GtkButton *button, GtkEntry *user_data) {
	RandomPlay (button, user_data, g_btn_play2, 1); 
	}

//-----------------------------------------------------------------
// Play row 
//-----------------------------------------------------------------

void on_button4_clicked (GtkButton *button, GtkEntry *user_data) {
	PlayClip (button, user_data, -20);
	}

void on_button7_clicked (GtkButton *button, GtkEntry *user_data) {
	PlayClip (button, user_data, -30);
	}

// ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// row 5 code
// ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

// ----------------------------------------------------------------
// search directories box activate
// ----------------------------------------------------------------

void	on_searchentry1_activate(GtkEntry *entry, gpointer  user_data) {

	const gchar *sentry = gtk_entry_get_text( entry );
	char cmd[1024];
	FILE *p1;

	if (strlen((char *) sentry) == 0) {
		Message("No expression specified.");    // update label
		return;
		}

	sprintf(cmd,"%s/fsearch-bin %s \"!%s\" %d %d", 
		Code_Home, Music_Home_Directory, sentry, 0, 0);

	p1=popen(cmd, "r"); // run search window

	cmd[0] = 0; // empty string in case nothing returned

	fgets(cmd,1024,p1);
	cmd[strlen(cmd)-1] = 0;
	fclose(p1);

	if (strcmp(cmd, "<none found>") == 0 || strlen(cmd) == 0)  {
		gtk_entry_set_text (g_lbl_entry6, "");
		return;
		}

	entry_insert(GTK_EDITABLE(g_lbl_entry6), cmd);
	gtk_entry_set_text (g_lbl_entry6, cmd);

	}

// ----------------------------------------------------------------
// search program entries
// ----------------------------------------------------------------

void	on_searchentry2_activate(GtkEntry *entry, gpointer  user_data) {

	const gchar *sentry = gtk_entry_get_text( entry );
	char cmd[1024];
	char cmd1[1024];
	int i, j, k;
	FILE *p1;

	if (strlen((char *) sentry)==0) {
		Message("No expression specified.");    // update label
		return;
		}

	strcpy(cmd, Code_Home);
	strcat(cmd, "/tmp");
	p1 = fopen(cmd, "w");

	for (int i = 0; i < *scount; i++) fprintf(p1,"%s\n", sounder[i]);
	fclose(p1);

	sprintf(cmd,"%s/fsearch-bin %s \"%s\" %d %d", 
		Code_Home, Code_Home, sentry, 0, 0);

	p1=popen(cmd, "r"); // run search box

	strcpy(cmd, ""); // in case nothing returned

	fgets(cmd, 1024, p1);
	cmd[strlen(cmd)-1] = 0;
	fclose(p1);

	if (strcmp(cmd, "<none found>") == 0 || strlen(cmd) == 0)  {
		gtk_entry_set_text (g_lbl_entry6, "");
		return;
		}

	for (i = 0; i < *scount; i++) {
		if (strcmp(cmd,sounder[i]) == 0) {
			*begin = i;
			sounders();
			return;
			}
		}
	gtk_entry_set_text (g_lbl_entry6, "");
	return;
	}

// ----------------------------------------------------------------
// skip next 
// ----------------------------------------------------------------

void	on_button57_clicked(GtkButton *button) {
	*skip = 1;
	*NowPlaying2 = *NowPlaying2 + 1; // sequential skip
	}

// ----------------------------------------------------------------
// button 52 - Exit
// ----------------------------------------------------------------

void on_button52_clicked (GtkToggleButton *button) {

        int i;

        GtkDialogFlags flags = GTK_DIALOG_DESTROY_WITH_PARENT |
                GTK_MESSAGE_ERROR | GTK_DIALOG_MODAL | GTK_WINDOW_TOPLEVEL |
                GTK_DIALOG_USE_HEADER_BAR ;

        GtkWidget *dialog = gtk_dialog_new_with_buttons  ("Exit Player?",
                GTK_WINDOW(window),
                flags,
                "Yes", 100, "No", 200, NULL);

//	system("wmctrl -r \"Spark Gap Radio Player\" -b add,above");

	gtk_window_set_keep_above(GTK_WINDOW(dialog), TRUE);

//      expect 200 for NO and 100 for YES

//      gtk_window_move(GTK_WINDOW(dialog), 300,730);
        gint r = gtk_dialog_run (GTK_DIALOG (dialog));

        gtk_widget_destroy (dialog);

        if ( r == 100) on_window_main_destroy();

	}

// ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// tooltip code
// ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

gboolean on_btn_hello_query_tooltip(GtkWidget *widget,gint x,gint y, 
		gboolean keyboard, GtkTooltip *tooltip) {
        gtk_tooltip_set_text(tooltip,directories[0]); 
        return 1;
	}

gboolean on_entry1_query_tooltip(GtkWidget *widget,gint x,gint y, 
		gboolean keyboard, GtkTooltip *tooltip) {
	const gchar *tip = gtk_entry_get_text(GTK_ENTRY(widget));
        gtk_tooltip_set_text(tooltip, tip); 
        return 1;
	}

// gboolean on_entry4_query_tooltip(GtkWidget *widget,gint x,gint y, 
// 		gboolean keyboard, GtkTooltip *tooltip) {
// 	const gchar *tip = gtk_entry_get_text(GTK_ENTRY(widget));
//         gtk_tooltip_set_text(tooltip, tip); 
//         return 1;
// 	}

gboolean on_entry5_query_tooltip(GtkWidget *widget,gint x,gint y, 
		gboolean keyboard, GtkTooltip *tooltip) {
	const gchar *tip = gtk_entry_get_text(GTK_ENTRY(widget));
        gtk_tooltip_set_text(tooltip, tip); 
        return 1;
	}

gboolean on_entry6_query_tooltip(GtkWidget *widget, gint x, gint y, 
		gboolean keyboard, GtkTooltip *tooltip) {
	const gchar *tip = gtk_entry_get_text(GTK_ENTRY(widget));
        gtk_tooltip_set_text(tooltip, tip); 
        return 1;
	}

gboolean on_btn_hello1_query_tooltip(GtkWidget *widget,gint x,gint y, 
		gboolean keyboard, GtkTooltip *tooltip) {
        gtk_tooltip_set_text(tooltip, directories[1]); 
        return 1;
	}

//-------

void on_entry1_insert_text(GtkEditable  *entry, gchar *edit) {

	int i = gtk_entry_get_text_length (GTK_ENTRY(entry));

	if (i>STR_MAX) {
		Message("Entry Error");
		return;
		}

	entry_insert(entry,edit);
	}

void on_entry2_insert_text(GtkEditable  *entry, gchar *edit) {
	entry_insert(entry, edit);
	}

void on_entry3_insert_text(GtkEditable  *entry, gchar *edit) {
	entry_insert(entry, edit);
	}

void on_entry4_insert_text(GtkEditable  *entry, gchar *edit) {
	entry_insert(entry, edit);
	}

void on_entry5_insert_text(GtkEditable  *entry, gchar *edit) {

        int i = gtk_entry_get_text_length (GTK_ENTRY(entry));

        if (i>STR_MAX) {
                Message("Entry Error");
                return;
                }

	entry_insert(entry,edit);
	}

void on_entry6_insert_text(GtkEditable  *entry, gchar *edit) {

	int i = gtk_entry_get_text_length (GTK_ENTRY(entry));

	if (i>STR_MAX) {
		Message("Entry Error");
		return;
		}
	entry_insert(entry, edit);
	}

void on_entry7_insert_text(GtkEditable  *entry, gchar *edit) {
	entry_insert(entry, edit);
	}

void on_entry8_insert_text(GtkEditable  *entry, gchar *edit) {
	entry_insert(entry, edit);
	}

void on_entry9_insert_text(GtkEditable  *entry, gchar *edit) {
	entry_insert(entry, edit);
	}

void on_entry10_insert_text(GtkEditable  *entry, gchar *edit) {
	entry_insert(entry, edit);
	}

void on_entry11_insert_text(GtkEditable  *entry, gchar *edit) {
	entry_insert(entry, edit);
	}

void on_entry12_insert_text(GtkEditable  *entry, gchar *edit) {
	entry_insert(entry, edit);
	}

void on_entry13_insert_text(GtkEditable  *entry, gchar *edit) {
	entry_insert(entry, edit);
	}

void on_entry14_insert_text(GtkEditable  *entry, gchar *edit) {
	entry_insert(entry, edit);
	}

void on_entry15_insert_text(GtkEditable  *entry, gchar *edit) {
	entry_insert(entry, edit);
	}

 //---------------

void on_entry1_changed (GtkEntry *entry, gchar *preedit, gpointer  user_data) {

	int i = gtk_entry_get_text_length (entry);

	if (i>STR_MAX) {
		Message("Entry Error");
		gtk_editable_delete_text (GTK_EDITABLE(entry), 0, -1 );
		return;
		}

	entry_changed(entry, preedit, user_data, 0);
	}

void on_entry2_changed (GtkEntry *entry, gchar *preedit, 
		gpointer user_data, GtkButton *b) {
	int i = atoi(gtk_button_get_label(GTK_BUTTON(g_button2)));
	entry_changed1(entry, preedit, user_data, i);
	}

void on_entry3_changed (GtkEntry *entry, gchar *preedit, 
		gpointer  user_data) {
	int i = atoi(gtk_button_get_label(GTK_BUTTON(g_button5)));
	entry_changed1(entry, preedit, user_data, i);
	}

void on_entry4_changed (GtkEntry *entry, gchar *preedit, 
		gpointer  user_data) {
	int i = atoi(gtk_button_get_label(GTK_BUTTON(g_button6)));
	entry_changed1(entry, preedit, user_data, i);
	}

void on_entry5_changed (GtkEntry *entry, gchar *preedit, 
		gpointer  user_data) {

	int i = gtk_entry_get_text_length (entry);

	if (i>STR_MAX) {
		Message("Entry Error");
		gtk_editable_delete_text (GTK_EDITABLE(entry), 0, -1 );
		return;
		}

	entry_changed(entry, preedit, user_data, 1);
	}

void on_entry6_changed (GtkEntry *entry, gchar *preedit, 
		gpointer  user_data) {

	int i = gtk_entry_get_text_length (entry);

	if (i>STR_MAX) {
		Message("Entry Error");
		gtk_editable_delete_text (GTK_EDITABLE(entry), 0, -1 );
		return;
		}

	entry_changed(entry, preedit, user_data, 0);
	}

void on_entry7_changed (GtkEntry *entry, gchar *preedit, 
		gpointer  user_data, GtkButton *b) {
	int i = atoi(gtk_button_get_label(GTK_BUTTON(g_button3)));
	entry_changed1(entry, preedit, user_data, i);
	}

void on_entry8_changed (GtkEntry *entry, gchar *preedit, 
		gpointer  user_data, GtkButton *b) {
	int i = atoi(gtk_button_get_label(GTK_BUTTON(g_button8)));
	entry_changed1(entry, preedit, user_data, i);
	}

void on_entry9_changed (GtkEntry *entry, gchar *preedit, 
		gpointer  user_data, GtkButton *b) {
	int i = atoi(gtk_button_get_label(GTK_BUTTON(g_button9)));
	entry_changed1(entry, preedit, user_data, i);
	}

void on_entry10_changed (GtkEntry *entry, gchar *preedit, 
		gpointer  user_data, GtkButton *b) {
	int i = atoi(gtk_button_get_label(GTK_BUTTON(g_button10)));
	entry_changed1(entry, preedit, user_data, i);
	}

void on_entry11_changed (GtkEntry *entry, gchar *preedit, 
		gpointer  user_data, GtkButton *b) {
	int i = atoi(gtk_button_get_label(GTK_BUTTON(g_button11)));
	entry_changed1(entry, preedit, user_data, i);
	}

void on_entry12_changed (GtkEntry *entry, gchar *preedit, 
		gpointer  user_data, GtkButton *b) {
	int i = atoi(gtk_button_get_label(GTK_BUTTON(g_button12)));
	entry_changed1(entry, preedit, user_data, i);
	}

void on_entry13_changed (GtkEntry *entry, gchar *preedit, 
		gpointer  user_data, GtkButton *b) {
	int i = atoi(gtk_button_get_label(GTK_BUTTON(g_button13)));
	entry_changed1(entry, preedit, user_data, i);
	}

void on_entry14_changed (GtkEntry *entry, gchar *preedit, 
		gpointer  user_data, GtkButton *b) {
	int i = atoi(gtk_button_get_label(GTK_BUTTON(g_button14)));
	entry_changed1(entry, preedit, user_data, i);
	}

void on_entry15_changed (GtkEntry *entry, gchar *preedit, 
		gpointer  user_data, GtkButton *b) {
	int i = atoi(gtk_button_get_label(GTK_BUTTON(g_button15)));
	entry_changed1(entry, preedit, user_data, i);
	}

// ----------------------------------------------------------------
// sounder buttons
// ----------------------------------------------------------------

// ----------------------------------------------------------------
// Sounder macro - notice line continuation backslashes
// Invocation of this macro builds three functions to 
// handle the program buttons.
// ----------------------------------------------------------------

#define SOUNDER(BTN_NBR,FILE_NAME) \
\
void on_button ## BTN_NBR ##_clicked (GtkButton *button, GtkEntry *user_data) { \
	sounderTime[FILE_NAME]->play = 0; *NowPlaying = FILE_NAME; \
	PlayClip(button, (GtkEntry *) sounder[FILE_NAME],-1); } \
\
gboolean on_button ## BTN_NBR ##_button_press_event (GtkButton *btn, GdkEventButton *event, gpointer user_data) { \
	if (event->button == 1 ) return FALSE; \
	*NowPlaying = *begin=FILE_NAME; \
	*NowPlaying2 = *begin?*begin-1:0; \
	return TRUE; \
	} \
\
gboolean on_button ## BTN_NBR ## _query_tooltip(GtkWidget *widget,gint x,gint y, gboolean keyboard, GtkTooltip *tooltip) { \
	char tmp[2048]; strcpy(tmp,"Play: "); strncat(tmp,sounder[FILE_NAME],2000); \
	gtk_tooltip_set_text(tooltip,tmp); \
	/* alt version gtk_tooltip_set_text(tooltip, "click to play");*/  \
	return 1;  \
	}

SOUNDER(3,  *begin+0)
SOUNDER(8,  *begin+1)
SOUNDER(9,  *begin+2)
SOUNDER(10, *begin+3)
SOUNDER(11, *begin+4)
SOUNDER(12, *begin+5)
SOUNDER(13, *begin+6)
SOUNDER(14, *begin+7)
SOUNDER(2,  *begin+8)
SOUNDER(5,  *begin+9)
SOUNDER(6,  *begin+10)
SOUNDER(15, *begin+11)

// ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// is this the lastWord?
// ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

int	lastWord(char * tmp, int off) {
	int i;
	for (i = 0; off + i < SOUNDER_ROW && tmp[i] != 0; i++)
		if (tmp[i] == ' ' ) { return 0;}

	return 1;
	}

// ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// entry insert
// ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

void	entry_insert(GtkEditable  *entry, gchar *edit) {
	unsigned char *file;
	int i=0, j=0;

	gtk_editable_delete_text (entry, 0, -1 );

	i = strlen( (char *) edit);

	file = malloc(i + 1);

	if (file == NULL ) {
		printf("Out of memory in entry_insert\n");
		abort();
		}

	strcpy(file, (char *) edit);

//	for (int j = i; j>0; j--) 
//		if (file[j] < ' ' ) { file[j] = 0;}

//	hexcleaner(file);

	if (strncmp(file, "file://", 7) == 0) strcpy(edit, &file[7]);
	else strcpy(edit, file);
	free (file);

	return;
	}

void	entry_insert1(GtkEntry  *entry, char *edit) {
	char file[1024];
	int i = 0, j = 0;

	strcpy(file, (char *) edit);
	hexcleaner(file);

	if (strncmp(file, "file://", 7) == 0) strcpy(edit, &file[7]);
	else strcpy(edit, file);
	return;
	}

// ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// hexcleaner
// ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

int	hexcleaner(char *t) {

	char *_ftmp;
	int m, n = 0, p,f = 0;

	int i = strlen(t);

	_ftmp = malloc(i + 1);

	if (_ftmp == NULL ) {
		printf("Out of memory in hexcleaner\n");
		abort();
		}

	for(m=0; t[m]; m++) {

		if (t[m] == '\n' ) { f = 1; continue; }
		if (t[m] == '\r' ) { f = 1; continue; }

		if (t[m] == '%') {
	
			f = 1;
			if (t[m+1] >= 'a' && t[m+1] <= 'z') t[m+1] = t[m+1]-32;
			if (t[m+1] >= '0' && t[m+1] <= '9') p=( t[m+1]-'0' ) * 16;
				else p=(t[m+1]-'A'+10)*16;

			if (t[m+2] >= 'a' && t[m+2] <= 'z') t[m+2] = t[m+2]-32;
                	if (t[m+2] >= '0' && t[m+2] <= '9') p = p+(t[m+2]-'0');
	                	else p = p + (t[m+2] - 'A' + 10);

                	_ftmp[n] = p;  /* insert value into output string */
			n++; m = m + 2;
                	}

		else {
			_ftmp[n++] = t[m];
			}
		}

        _ftmp[n]=0; 
	strcpy(t,_ftmp);

	free(_ftmp);

	return f;
	}

// ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// entry changed
// ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

void	entry_changed (GtkEntry *entry, gchar *preedit, gpointer  user_data, int Row) {

	char file[1024],file1[1024];
	int i;

// Note: changes in the entry result in a loopback to this code

        strcpy(file, gtk_entry_get_text(entry));

	if (hexcleaner(file)) { 
		gtk_entry_set_text(entry, (gchar *) file); 
		return;
		}

	if (strncmp(file, "https://",8) == 0 ||
	    strncmp(file, "http://",7) == 0 ) {
		gtk_entry_set_text (GTK_ENTRY(entry), (gchar *) file); 
		if (entry == g_lbl_entry1) strcpy(top1,file);
		else if (entry == g_lbl_entry5) strcpy(top2,file);
		else if (entry == g_lbl_entry6) strcpy(top3,file);
		return;
		}

// basename

	if (file[0] == '/' && entry == g_lbl_entry1) strcpy(top1,file);
	else if (file[0] == '/' && entry == g_lbl_entry5) strcpy(top2, file);
	else if (file[0] == '/' && entry == g_lbl_entry6) strcpy(top3, file);

	i=strlen(file);
	for (int j=0; j<i; j++) if (file[j] == ':') { file[j]=0; break;}

	i=strlen(file);
	for (i = strlen(file); i > 0; i--) if (file[i] == '/') break;

	if (i > 0) strcpy(file1, &file[i+1]);
		else return; // file name had no directory info

	file[i] = 0;
	strcpy(directories[Row], file);

	if (filetype(file1)) { 
		gtk_entry_set_text (GTK_ENTRY(entry), (gchar *) file1); 
		i = strlen(file1) - 4; 
		}

	}

// ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// entry changed1
// ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

void    entry_changed1 (GtkEntry *entry, gchar *preedit, gpointer  user_data, int Row) {

        char *file, *file1;
        int i;
	int k;
	int m;
	int flg1 = 0;

	i = gtk_entry_get_text_length (entry);

	file = malloc (i + 9);

	if (file == NULL) {
		printf("Out of memory in entry_changed1\n");
		abort();
		}

	file1 = malloc (i + 9);

	if (file1 == NULL) {
		printf("Out of memory in entry_changed1\n");
		abort();
		}

        strcpy(file, gtk_entry_get_text(entry));

again:
	flg1 = 0;
	for(k = 0; file[k] != 0; k++) if (file[k] == '\n') { flg1 = 1; break;}
	file[k] = 0;

	if (hexcleaner(file)) {
//                gtk_entry_set_text(entry, (gchar *) file);
//                return;
		}

        if (strncmp(file, "https://",8) == 0 ||
            strncmp(file, "http://",7) == 0 ) {
		gtk_entry_set_text (GTK_ENTRY(entry), (gchar *) file); 
                goto end; // exit and deallocate
		}

// basename

        i = strlen(file);

	int flg = 0;

	if (file[0] == '/') flg=1;

        for (i = strlen(file); i > 0; i--) if (file[i] == '/') break;

	if (Row >= *scount) *scount = Row + 1;

        if (i > 0) strcpy(file1, &file[i + 1]);
//                else return;
	else strcpy(file1, file);

	if (flg) { 
		char tmp[1024], *p1;
		FILE *f1;

		strcpy(sounder[Row], file);

		strcpy(tmp,"ffprobe -v quiet -of csv=p=0 -show_entries format=duration \"");
		strcat(tmp, file);
		strcat(tmp, "\"");
		f1=popen(tmp, "r"); strcpy(tmp, ""); fgets(tmp, 256, f1); fclose(f1);
		p1=strstr(tmp, ".");
		if (p1 != NULL) *p1=0;
		else { strcpy(tmp, "");}

		strcpy(sounderTime[Row]->Time, tmp);
		sounderTime[Row]->play = 1; // play it?
		sounderPlayed[i] = 0;

	        file[i]=0;

		i=filetype(file1);

		if (i == FALSE) { strcpy(file,"Bad File Type"); }

		else {
			if (i == 7 || i == 8) file1[strlen(file1) - 5] = 0; // filetype ending removal
			else file1[strlen(file1) - 4] = 0;
			strcat(file1, " (");
			strcat(file1, tmp);
			strcat(file1, "/0)");
                	gtk_entry_set_text (GTK_ENTRY(entry), (gchar *) file1);
			while (gtk_events_pending ()) gtk_main_iteration ();
                	}
		}

	if ( !flg1 ) goto end;

	if (! strlen(file)) goto end;

	strcpy(file1, &file[k + 1]);

	strcpy(file, file1);
	for (m = 0; file[m] != 0; m++) if (file[m] == '\n') break;
	if (file[m] == 0) goto end; // last row
	paste1(&Row);

	flg1=0;

	if (strncmp(file, "file://", 7) == 0) { 
		strcpy(file1, &file[7]); 
		strcpy(file, file1); 
		}

	if (strlen(file)) { goto again; }

end:	free (file);
	free (file1);
        }


// ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// strip blanks
// ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

void	strip(char* str) {
	int i, x;
	for(i = x = 0; str[i]; ++i)
		if( !isspace(str[i] ) || (i > 0 && !isspace(str[i - 1])))
			str[x++] = str[i];
	str[x] = '\0';
	}

// ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// load sounder file name into button
// ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

char *	sounderName(char * name, int nbr) {

	int i, j, k, m;
	static char tmp[1024];
	char played[64];
	FILE * f1;

	for (i = strlen(name); i > 0 && name[i] != '.'; i--);	// find file extension start

	if (i == 0) ;				// no file extension
		else name[i]=0;			// truncate file extension

	for (; i>0 && name[i] != '/'; i--);	// find last slash character

	if (i) i++;				// i will be zero if no slash found

	for (m = i; name[m] != 0; m++) {
		if (name[m] == '-') name[m] = ' ';
		if (name[m] == '_') name[m] = ' ';
		if (name[m] == '[') name[m] = ' ';
		if (name[m] == ']') name[m] = ' ';
		else name[m] = tolower(name[m]);
		}

	strip(name);

	for (m = i; name[m] != 0; m++) {
		if ( name[m] == ' ' && isalpha(name[m + 1]) ) 
			name[m + 1] = toupper(name[m + 1]);
		}

	name[i] = toupper(name[i]);

//.........................................................................................

	name[i+SOUNDER_SIZE-7] = 0;	// truncate

	sprintf(played,"/%d)", sounderPlayed[nbr]);

	if (strlen(sounder[nbr])) {
		if ( strlen(sounderTime[nbr]->Time) == 0 ) {	// play time not known
			strcpy(tmp,"ffprobe -v quiet -of csv=p=0 -show_entries format=duration \"");
			strcat(tmp, sounder[nbr]);
			strcat(tmp, "\"");

			f1 = popen(tmp, "r"); strcpy(tmp, ""); fgets(tmp, 256, f1); fclose(f1);
	
			if (strlen(tmp) != 0) 
				for(int n = 0; tmp[n]; n++) if (tmp[n] == '.') { tmp[n] = 0; break; }
			else strcpy(tmp, "0");

			strcat(name, ": (");
			strcat(name, tmp);
			strcat(name, played);
			strcpy(sounderTime[nbr]->Time, tmp);
			sounderTime[nbr]->play = 1; // play it?
			}
		else { 
			strcat(name, ": ("); 
			strcat(name, sounderTime[nbr]->Time);
			strcat(name, played);
			}	// already calculated
		}

	else strcpy(name, "<empty>");

	// name is shorter than limit

	strcpy(tmp, &name[i]);
	return tmp;

	j=strlen(&name[i]);

	if (j > SOUNDER_SIZE) ;
	else {
		for (k = i + j; k < SOUNDER_SIZE + i; k++) name[k] = ' ';	// pad with blanks
		name[k] = 0;
		}

	for (k=0; k<SOUNDER_ROW; k++) {
		tmp[k] = name[i+k];
		if (tmp[k] == ' ' && lastWord(&name[k + i + 1], k)) { k--; break; } }

	tmp[k] = name[i + k];
	k++;

	for (; name[i + k] != 0; k++) {
		tmp[k] = name[i + k - 1];
		}

	tmp[k] = name[i + k - 1];
	tmp[k + 1] = 0;

	return tmp;
	}

//---------------------------------
// load content into sounder button
//---------------------------------

void	sounderButton(GtkWidget *button, int nbr) {
	char name[1024], tmp[1024];

	strncpy(name,sounder[nbr], 1024);

	if (sounderPlayed[nbr]) {
		clearButton(GTK_BUTTON(button));
		}

	if (strcmp(name, "") == 0) {
		gtk_button_set_label (GTK_BUTTON(button), (const gchar* ) " ");
//		tmp = "";
		strcpy(tmp, "");
		}

//	tmp=sounderName(name,nbr);
	strncpy(tmp, sounderName(name, nbr), 1023);

	hexcleaner(tmp);

	if (button == g_button3) {
		gtk_entry_set_text (g_lbl_entry7, (gchar *) tmp); 
		e7Row = nbr;
		}
	if (button == g_button8) {
		e8Row = nbr;
		gtk_entry_set_text (g_lbl_entry8, (gchar *) tmp); 
		}
	if (button == g_button9) {
		gtk_entry_set_text (g_lbl_entry9, (gchar *) tmp); 
		e9Row=nbr;
		}
	if (button == g_button10) {
		gtk_entry_set_text (g_lbl_entry10, (gchar *) tmp); 
		e10Row=nbr;
		}
	if (button == g_button11) {
		gtk_entry_set_text (g_lbl_entry11, (gchar *) tmp); 
		e11Row = nbr;
		}
	if (button == g_button12) {
		gtk_entry_set_text (g_lbl_entry12, (gchar *) tmp); 
		e12Row=nbr;
		}
	if (button == g_button13) {
		gtk_entry_set_text (g_lbl_entry13, (gchar *) tmp); 
		e13Row=nbr;
		}
	if (button == g_button14) {
		gtk_entry_set_text (g_lbl_entry14, (gchar *) tmp); 
		e14Row = nbr;
		}
	if (button == g_button2) { // s9
		gtk_entry_set_text (g_lbl_entry2, (gchar *) tmp); 
		e15Row=nbr;
		}
	if (button == g_button5) { // s10
		gtk_entry_set_text (g_lbl_entry3, (gchar *) tmp); 
		e16Row=nbr;
		}
	if (button == g_button6) { // s11
		gtk_entry_set_text (g_lbl_entry4, (gchar *) tmp); 
		e17Row=nbr;
		}
	if (button == g_button15) { // s12
		gtk_entry_set_text (g_lbl_entry15, (gchar *) tmp); 
		e18Row = nbr;
		}

	sprintf(tmp,"%d", nbr);
	gtk_button_set_label (GTK_BUTTON(button), (const gchar* ) tmp);
	return;
	}

// ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//	make a button blue
// ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

void	clearButton(GtkButton *button) {
	css_set(cssProvider1, GTK_WIDGET(button));
	}

// ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//	make a button red
// ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

void	setButton(GtkButton *button) {
	css_set(cssProvider2, GTK_WIDGET(button));
	}

// ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// paint css on a widget
// ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

void	css_set(GtkCssProvider * cssProvider, GtkWidget *g_widget) {

	GtkStyleContext *context;

	context = gtk_widget_get_style_context(g_widget);    

	gtk_style_context_add_provider (context,
		GTK_STYLE_PROVIDER(cssProvider),
		500);

	gtk_style_context_save (context);
	}

//--------------------------------------
// rearrange the page of sounder buttons
//--------------------------------------

void	AdvanceSounderPage() {
	*begin = *begin + SOUNDER_BUTTON_COUNT;
	if (*begin + SOUNDER_BUTTON_COUNT >= *scount) { *begin = *scount - SOUNDER_BUTTON_COUNT; }	
	if (*begin < 0 ) *begin = 0;
	*NowPlaying = -1;
	sounders();
	}

// ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// filetype
// ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

int	filetype(char *name) {

//---------------------------------------------
//	File extensions acceptable to be played.
//	Add more as needed.
//---------------------------------------------

	int j, i = strlen(name);

	for (j = i; j > 0 && name[j] != '.'; j--);
	if (j==0) return FALSE;
	if (strcmp(&name[j], ".mp4") == 0) return MP4;
	if (strcmp(&name[j], ".MP4") == 0) return MP4;
	if (strcmp(&name[j], ".mp3") == 0) return MP3;
	if (strcmp(&name[j], ".MP3") == 0) return MP3;
	if (strcmp(&name[j], ".mpg") == 0) return MPG;
	if (strcmp(&name[j], ".dvd") == 0) return DVD;
	if (strcmp(&name[j], ".ogg") == 0) return OGG;
	if (strcmp(&name[j], ".wav") == 0) return WAV;
	if (strcmp(&name[j], ".webm") == 0) return WEB;
	if (strcmp(&name[j], ".flac") == 0) return FLAC;
	if (strcmp(&name[j], ".m3u") == 0) return PLAY;
	if (strcmp(&name[j], ".pls") == 0) return PLAY;
	if (strcmp(&name[j], ".png") == 0) return PNG;
	if (strcmp(&name[j], ".jpg") == 0) return JPG;
	if (strcmp(&name[j], ".wmv") == 0) return WMV;
	if (strcmp(&name[j], ".avi") == 0) return AVI;
	if (strcmp(&name[j], ".mpv") == 0) return MPV;
	return FALSE;
	}

int	bigFilename(GtkEntry *user_data, char *file) {

	int i;
	char file1[1024];

	strcpy(file1,file);

	if (filetype(file)) { 
		gtk_entry_set_text (user_data, (gchar *) file1); 
		i = strlen(file1) - 4; 
		}
	else { 
		gtk_entry_set_text (user_data, (gchar *) "<bad file name>"); 
		i = -1; 
		}

	gtk_editable_set_position(GTK_EDITABLE(user_data), i);
	return i;
	}

int	smallFilename(GtkEntry *user_data, char *file) {

	int i;
	char file1[1024];

	i = strlen(file);
	for (i = strlen(file); i > 0; i--)
		if (file[i] == '/') break;

	if (i > 0) strcpy(file1, &file[i + 1]);
	else strcpy(file1, file);

	if (filetype(file)) { 
		gtk_entry_set_text (user_data, (gchar *) file1); 
		i=strlen(file1) - 4; 
		}
	else { 
		gtk_entry_set_text (user_data, (gchar *) "<bad file name>"); 
		i = -1; 
		}

	gtk_editable_set_position(GTK_EDITABLE(user_data), i);
	return i;
	}

int	smallDirname(GtkEntry *user_data, char *file) {

	int i;
	char file1[1024];

	gtk_entry_set_text (user_data, (gchar *) file); 

	gtk_editable_set_position(GTK_EDITABLE(user_data), i);
	return i;
	}

// ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// shuffle playlist
// ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

char *	shuffle(int directory, char *filename) {

//	Find a random playable file in current directory.

	char mesg[512], mesg1[512];
	FILE * p1;
	char tmp1[1024];

	strcpy(tmp1, "ls \"");
	strcat(tmp1, directories[directory]);
	strcat(tmp1, "\" | shuf -n 10");

	p1=popen(tmp1, "r");

	for (int i = 0; i < 10; i++) {
		if (fgets(filename, 256, p1) == NULL) {
			break;
			}
		filename[strlen(filename) - 1] = 0;
		if (filetype(filename)) {
			fclose(p1);
			return filename;
			}
		}

	fclose(p1);

	strcpy(filename, "<bad file name>");
	return filename;
	}

// ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// file chooser common code function
// ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

void	FileChooser (GtkButton *button, GtkEntry *user_data, int Row, GtkWidget *g_btn) {

	GtkWidget *dialog;
	GtkFileChooserAction action = GTK_FILE_CHOOSER_ACTION_OPEN;
	char mesg[512], mesg1[512], file[1024], file1[1024];
	FILE *p1;
	gint x = 0, y = 0, res;
	int i, j;
	char cmd[512];

	chdir(directories[Row]);

	dialog = gtk_file_chooser_dialog_new ("Open File",
                                      GTK_WINDOW(window),
                                      action,
                                      ("_Cancel"),
                                      GTK_RESPONSE_CANCEL,
                                      ("_Open"),
                                      GTK_RESPONSE_ACCEPT,
                                      NULL);


	gtk_window_set_keep_above(GTK_WINDOW(dialog), TRUE);

	res = gtk_dialog_run (GTK_DIALOG (dialog));

	if (res == GTK_RESPONSE_ACCEPT) {
		char *filename;
		char *directory;
		GtkFileChooser *chooser = GTK_FILE_CHOOSER (dialog);

		filename = gtk_file_chooser_get_filename (chooser);
		directory = gtk_file_chooser_get_current_folder (chooser);

		if (directory != NULL) {
			strcpy(directories[Row], directory);
			lastDir = Row;
			chdir(directory);
			}
		else {
			char tmp[2048];
			strcpy(tmp, filename);
			strcpy(directories[Row], dirname(tmp));	
			lastDir = Row;
			chdir(directory);
			}

		strcpy(file, filename);

		// user_data points to the entry box to to contain the result
		// user_data is passed with the click to the file select button
		// and passed here. smallFilename will populate the entry box.

//		i = smallFilename(user_data, file);
		i = bigFilename(user_data, file);

		g_free (filename);
                g_free (directory);
		}

	gtk_widget_destroy (dialog);

	if (res == GTK_RESPONSE_CANCEL) return;

	}

// ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// directory chooser common code function
// ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

void	DirChooser (GtkButton *button, GtkEntry *user_data, int Row, GtkWidget *g_btn) {

	GtkWidget *dialog;
	GtkFileChooserAction action = GTK_FILE_CHOOSER_ACTION_SELECT_FOLDER;
	char mesg[512], mesg1[512], file[1024], file1[1024];
	FILE *p1;
	gint x = 0, y = 0, res;
	int i, j;
	char cmd[512];

	chdir(directories[Row]);

	dialog = gtk_file_chooser_dialog_new ("Open File",
                                      GTK_WINDOW(window),
                                      action,
                                      ("_Cancel"),
                                      GTK_RESPONSE_CANCEL,
                                      ("_Open"),
                                      GTK_RESPONSE_ACCEPT,
                                      NULL);

	gtk_window_set_keep_above(GTK_WINDOW(dialog), TRUE);

	res = gtk_dialog_run (GTK_DIALOG (dialog));

	if (res == GTK_RESPONSE_ACCEPT) {
		char *filename;
		char *directory;
		GtkFileChooser *chooser = GTK_FILE_CHOOSER (dialog);
		filename = gtk_file_chooser_get_filename (chooser);
		directory = gtk_file_chooser_get_current_folder (chooser);

		if (directory != NULL) {
			strcpy(directories[Row],directory);
			lastDir = Row;
			chdir(directory);
			}
		else {
			char tmp[2048];
			strcpy(tmp,filename);
			strcpy(directories[Row],dirname(tmp));	
			lastDir = Row;
			chdir(directory);
			}

		strcpy(file, filename);

		i=smallDirname(user_data, file);

		g_free (filename);
                g_free (directory);
		}

	gtk_widget_destroy (dialog);

	if (res == GTK_RESPONSE_CANCEL) return;

	}

// ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// Play common code function
// ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

void	PlayClip (GtkButton *button, GtkEntry *user_data, int Row) {

	gchar * text ;
	char mesg[512];
	char cmd[1024];
	char cmd1[1024];
	char lcl[1024];
	FILE *f1;
	int i, j, k = -1;

	*Pause = 0;
	*PauseKill = -1;

	// Row is -1 if call is from a sounder

	if (Row >= 0 ) {
		if (chdir(directories[Row]))
			printf("*** Directory error Row %d\n",Row); // move to correct directory
		text = (gchar *) gtk_entry_get_text (user_data);
		*CurrentPlay = Row;
		}
	if (Row == -10) { // top row
		text = lcl; // make pointer point at real data
		strcpy(text, top1);
		}
	else if (Row == -20) { // top row
		text = lcl; // make pointer point at real data
		strcpy(text, top2);
		}
	else if (Row == -30) { // top row
		text = lcl; // make pointer point at real data
		strcpy(text, top3);
		}
	else if (Row == -1) {
		*CurrentPlay = k = *NowPlaying;
		text = (gchar *) user_data;
		}

	else text = (gchar *) gtk_entry_get_text (user_data); // was for sounder buttons

	if (strlen((char *) text) == 0) {
		Message("No file specified.");    // update label
		return;
		}

	for (j = strlen(text); j >= 0 && text[j] != '/'; j--);
	sprintf(currentPlay,"Curr: %d %s", k, text+(j+1));

	*audio=0;

	if (PlaySelected > 0) { 
		kill(PlaySelected, 9); 
		PlaySelected = -1; 
		}

	ClearPlayers();

	sleep(1); // wait for kill to be effective

	strcpy(mesg, "Playing* "); 
	strcat(mesg, text); 
	strcat(mesg, "\n"); 
	printf(mesg);

	strcpy(mesg, "\""); 
	strcat(mesg, text); 
	strcat(mesg, "\"");

	clearButton(GTK_BUTTON(g_button32));
	clearButton(GTK_BUTTON(g_button30));

	int itype = filetype(text);

	*audio=0;

       	if (itype == MP3 || itype == WAV || itype == OGG ||
			itype == FLAC || itype == WEB) {
		if (*player == 0 ) MPV_PLAY_AUDIO
		else VLC_PLAY
		*audio=1;
               	}
	else {
		if (*player == 0 ) MPV_PLAY
		else VLC_PLAY
		}

		if (itype == PNG || itype == JPG) strcat(cmd,"--input-repeat 10 ");

	AudioSubSys(cmd); // adds parameters for Pulse and JACK

	strcat(cmd, mesg); 
 		if (*player ==0 ) {
			strcat(cmd, " 2>&1 > /dev/null | cat > "); 
			strcat(cmd, Code_Home);
			strcat(cmd, "/mpvTmp &");
			}
	else strcat(cmd," &");


	*Clock = 0;

//-----------------------------------------------------------------------------------
	PlaySelected = fork(); // the child will monitor the player
//-----------------------------------------------------------------------------------

	if (PlaySelected != 0) {		// parent returns after setting button to gray
		return; 
		}

	if (Row != -1) chdir(directories[Row]);	// move to correct directory

	i=filetype(text);

	system(cmd); // forked

#include "vlc-place.h"

	char tmp[2048];

	strcpy(tmp,"ffprobe -v quiet -of csv=p=0 -show_entries format=duration ");
	strcat(tmp,mesg);

	f1=popen(tmp,"r"); strcpy(tmp,""); fgets(tmp,256,f1); fclose(f1);

	*Clock=atoi(tmp);
	TimeOfNext();

	*videoPlaying = 1;

        while (1) { // wait until done

		if ( VideoRunning() ) {
			sleep(1);
			continue;
			}

		if (PlaySelected > 0) { kill(PlaySelected,9); PlaySelected=-1; *NowPlaying=-1;}

		ClearPlayers();

                PlaySelected = -1;
		*videoPlaying = 0;
                break;
                }

	*Clock = 0;
	*videoPlaying = 0;
	*PauseKill = 0;
	*Pause = 0;
	strcpy(next_random,"Next Program Selection");
	strcpy(currentPlay,"Curr Program Selection");
        exit(EXIT_SUCCESS);
        }

// ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// Random selection common code function
// ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

void RandomPlay (GtkButton *button, GtkEntry *user_data, GtkWidget *g_btn, int Row) {

	char mesg[512],mesg1[512],tmp1[1024],file[1024],file1[1024];
	FILE * p1;
	int i,j;
	shuffle(Row,file);
	chdir(directories[Row]);
	strcpy(file1,directories[Row]);
	strcat(file1,"/");
	strcat(file1,file);
	strcpy(file,file1);
	if (Row == -10) strcpy(top1,file);
	else if (Row == -20) strcpy(top2,file);
	else if (Row == -30) strcpy(top3,file);


//	i=smallFilename(user_data, file);
	i=bigFilename(user_data, file);

	}

// ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//	pop up message window
// ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

void	Message(char * msg) {

	GdkColor color1;
	color1.red = 0x5555;
	color1.green = 0x5555;
	color1.blue = 0x5555;
	
	GdkColor color2;
	color2.red = 0xFFFF;
	color2.green = 0xFFFF;
	color2.blue = 0xFFFF;

	GtkDialogFlags flags = GTK_DIALOG_DESTROY_WITH_PARENT ;
	GtkWidget *dialog = gtk_message_dialog_new (GTK_WINDOW(window),
		flags,
		GTK_MESSAGE_WARNING,
		GTK_BUTTONS_CLOSE,
		msg);

	gtk_widget_modify_bg(dialog, GTK_STATE_NORMAL, &color1);
	gtk_widget_modify_fg(dialog, GTK_STATE_NORMAL, &color2);

	gtk_window_set_keep_above(GTK_WINDOW(dialog), TRUE);

//	gtk_window_move (GTK_WINDOW(dialog), 215, 582);
	gtk_dialog_run (GTK_DIALOG (dialog));
	gtk_widget_destroy (dialog);
	}

// ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// is video player running?
// ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

int	VideoRunning() {
	FILE * f1;
	char cmd[512];

	cmd[0]=0;
	f1 = popen("ps -ale | grep -i \'mpv\\|vlc\'", "r");

//	if (*player == MPV_PLAYER) f1 = popen("ps -ale | grep -i \'mpv \\| vlc\' ", "r");
//	else f1 = popen("ps -ale | grep -i vlc ", "r");

	if (fgets(cmd,256,f1) == NULL) { // not running
		fclose(f1);
		return 0;
		}

	fclose(f1);
	return 1; // running
	}

// ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// does file exist?
// ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

int	fileExists(const char *fname) {
	FILE *file;
	if ((file = fopen(fname, "r"))) {
		fclose(file);
		return 1;
		}
	return 0;
	}

//------------------------

void	sndr_clicked (GtkButton *button, int bnbr) {
	char cmd[2048];
	if (strlen(sndr[bnbr])==0) return;
	setButton(button);
	while (gtk_events_pending ()) gtk_main_iteration ();
	strcpy(cmd,"mpv --volume=120 "); 
	AudioSubSys(cmd); // add parameters for the correct sound system
	strcat(cmd,"\"");
	strcat(cmd,sndr[bnbr]);
	strcat(cmd,"\"");
	system(cmd);
	*Pause=1;  // will be reset in following function
	on_button34_clicked (GTK_BUTTON(g_button34)); 
	clearButton(button);
	} 

void on_sndr1_clicked (GtkButton *button, GtkEntry *user_data) { 
	sndr_clicked( button, 0 );
	return;
	}

char *	fileName(char * sndr, char * cmd) {
	int i;
	for (i=strlen(sndr); i> 0; i--) 
		if (sndr[i] == '/' ) break;
		else if (sndr[i] == '\n') sndr[i] = 0;
	if ( i == 0 || i == strlen(sndr) ) strcpy(cmd,sndr); // forward slash missing or last
	else strcpy(cmd,&sndr[i+1]);
	return cmd;
	}

gboolean on_sndr1_query_tooltip(GtkWidget *widget, gint x, gint y, 
		gboolean keyboard, GtkTooltip *tooltip) { 
	char cmd[1024];
	gtk_tooltip_set_text(tooltip,fileName(sndr[0],cmd)); 
	return 1;
	}

//------------------------

void on_sndr2_clicked (GtkButton *button, GtkEntry *user_data) { 
	sndr_clicked( button, 1 );
	return;
	}

gboolean on_sndr2_query_tooltip(GtkWidget *widget, gint x, gint y, 
		gboolean keyboard, GtkTooltip *tooltip) { 
	char cmd[1024];
	gtk_tooltip_set_text(tooltip,fileName(sndr[1],cmd)); 
	return 1;
	}

//------------------------

void on_sndr3_clicked (GtkButton *button, GtkEntry *user_data) { 
	sndr_clicked( button, 2 );
	return;
	}

gboolean on_sndr3_query_tooltip(GtkWidget *widget, gint x, gint y, 
		gboolean keyboard, GtkTooltip *tooltip) { 
	char cmd[1024];
	gtk_tooltip_set_text(tooltip,fileName(sndr[2],cmd)); 
	return 1;
	}

//------------------------

void on_sndr4_clicked (GtkButton *button, GtkEntry *user_data) { 
	sndr_clicked( button, 3 );
	return;
	}

gboolean on_sndr4_query_tooltip(GtkWidget *widget, gint x, gint y, 
		gboolean keyboard, GtkTooltip *tooltip) { 
	char cmd[1024];
	gtk_tooltip_set_text(tooltip,fileName(sndr[3],cmd)); 
	return 1;
	}

//------------------------

void on_sndr5_clicked (GtkButton *button, GtkEntry *user_data) { 
	sndr_clicked( button, 4 );
	return;
	}

gboolean on_sndr5_query_tooltip(GtkWidget *widget, gint x, gint y, 
		gboolean keyboard, GtkTooltip *tooltip) { 
	char cmd[1024];
	gtk_tooltip_set_text(tooltip,fileName(sndr[4],cmd)); 
	return 1;
	}

//------------------------

void on_sndr6_clicked (GtkButton *button, GtkEntry *user_data) { 
	sndr_clicked( button, 5 );
	return;
	}

gboolean on_sndr6_query_tooltip(GtkWidget *widget, gint x, gint y, 
		gboolean keyboard, GtkTooltip *tooltip) { 
	char cmd[1024];
	gtk_tooltip_set_text(tooltip,fileName(sndr[5],cmd)); 
	return 1;
	}

//------------------------

void on_sndr7_clicked (GtkButton *button, GtkEntry *user_data) { 
	sndr_clicked( button, 6 );
	return;
	}

gboolean on_sndr7_query_tooltip(GtkWidget *widget, gint x, gint y, 
		gboolean keyboard, GtkTooltip *tooltip) { 
	char cmd[1024];
	gtk_tooltip_set_text(tooltip,fileName(sndr[6],cmd)); 
	return 1;
	}

//------------------------

void on_sndr8_clicked (GtkButton *button, GtkEntry *user_data) { 
	sndr_clicked( button, 7 );
	return;
	}

gboolean on_sndr8_query_tooltip(GtkWidget *widget, gint x, gint y, 
		gboolean keyboard, GtkTooltip *tooltip) { 
	char cmd[1024];
	gtk_tooltip_set_text(tooltip,fileName(sndr[7],cmd)); 
	return 1;
	}

//------------------------

void on_sndr9_clicked (GtkButton *button, GtkEntry *user_data) {
	sndr_clicked( button, 8 );
	return;
	}

gboolean on_sndr9_query_tooltip(GtkWidget *widget, gint x, gint y, 
		gboolean keyboard, GtkTooltip *tooltip) {
	char cmd[1024];
	gtk_tooltip_set_text(tooltip,fileName(sndr[8],cmd)); 
        return 1;
        }

//------------------------

void on_sndr10_clicked (GtkButton *button, GtkEntry *user_data) { 
	sndr_clicked( button, 9 );
	return;
	}

gboolean on_sndr10_query_tooltip(GtkWidget *widget, gint x, gint y, 
		gboolean keyboard, GtkTooltip *tooltip) { 
	char cmd[1024];
	gtk_tooltip_set_text(tooltip,fileName(sndr[9],cmd)); 
	return 1;
	}

//------------------------

void on_sndr11_clicked (GtkButton *button, GtkEntry *user_data) { 
	sndr_clicked( button, 10 );
	return;
	}

gboolean on_sndr11_query_tooltip(GtkWidget *widget, gint x, gint y, 
		gboolean keyboard, GtkTooltip *tooltip) { 
	char cmd[1024];
	gtk_tooltip_set_text(tooltip,fileName(sndr[10],cmd)); 
	return 1;
	}

//------------------------

void on_sndr12_clicked (GtkButton *button, GtkEntry *user_data) { 
	sndr_clicked( button, 11 );
	return;
	}

gboolean on_sndr12_query_tooltip(GtkWidget *widget, gint x, gint y, 
		gboolean keyboard, GtkTooltip *tooltip) { 
	char cmd[1024];
	gtk_tooltip_set_text(tooltip,fileName(sndr[11],cmd)); 
	return 1;
	}

//------------------------

void on_sndr13_clicked (GtkButton *button, GtkEntry *user_data) { 
	sndr_clicked( button, 12 );
	return;
	}

gboolean on_sndr13_query_tooltip(GtkWidget *widget, gint x, gint y, 
		gboolean keyboard, GtkTooltip *tooltip) { 
	char cmd[1024];
	gtk_tooltip_set_text(tooltip,fileName(sndr[12],cmd)); 
	return 1;
	}

//------------------------

void on_sndr14_clicked (GtkButton *button, GtkEntry *user_data) { 
	sndr_clicked( button, 13 );
	return;
	}

gboolean on_sndr14_query_tooltip(GtkWidget *widget, gint x, gint y, 
		gboolean keyboard, GtkTooltip *tooltip) { 
	char cmd[1024];
	gtk_tooltip_set_text(tooltip,fileName(sndr[13],cmd)); 
	return 1;
	}

//------------------------

void on_sndr15_clicked (GtkButton *button, GtkEntry *user_data) { 
	sndr_clicked( button, 14 );
	return;
	}

gboolean on_sndr15_query_tooltip(GtkWidget *widget, gint x, gint y, 
		gboolean keyboard, GtkTooltip *tooltip) { 
	char cmd[1024];
	gtk_tooltip_set_text(tooltip,fileName(sndr[14],cmd)); 
	return 1;
	}

//------------------------

void on_sndr16_clicked (GtkButton *button, GtkEntry *user_data) { 
	sndr_clicked( button, 15 );
	return;
	}

gboolean on_sndr16_query_tooltip(GtkWidget *widget, gint x, gint y, 
		gboolean keyboard, GtkTooltip *tooltip) { 
	char cmd[1024];
	gtk_tooltip_set_text(tooltip,fileName(sndr[15],cmd)); 
	return 1;
	}

//------------------------

void on_sndr17_clicked (GtkButton *button, GtkEntry *user_data) { 
	sndr_clicked( button, 16 );
	return;
	}

gboolean on_sndr17_query_tooltip(GtkWidget *widget, gint x, gint y, 
		gboolean keyboard, GtkTooltip *tooltip) { 
	char cmd[1024];
	gtk_tooltip_set_text(tooltip,fileName(sndr[16],cmd)); 
	return 1;
	}

//------------------------

void on_sndr18_clicked (GtkButton *button, GtkEntry *user_data) { 
	sndr_clicked( button, 17 );
	return;
	}

gboolean on_sndr18_query_tooltip(GtkWidget *widget, gint x, gint y, 
		gboolean keyboard, GtkTooltip *tooltip) { 
	char cmd[1024];
	gtk_tooltip_set_text(tooltip,fileName(sndr[17],cmd)); 
	return 1;
	}

//------------------------

void on_sndr19_clicked (GtkButton *button, GtkEntry *user_data) { 
	sndr_clicked( button, 18 );
	return;
	}

gboolean on_sndr19_query_tooltip(GtkWidget *widget, gint x, gint y, 
		gboolean keyboard, GtkTooltip *tooltip) { 
	char cmd[1024];
	gtk_tooltip_set_text(tooltip,fileName(sndr[18],cmd)); 
	return 1;
	}

//------------------------

void on_sndr20_clicked (GtkButton *button, GtkEntry *user_data) { 
	sndr_clicked( button, 19 );
	return;
	}

gboolean on_sndr20_query_tooltip(GtkWidget *widget, gint x, gint y, 
		gboolean keyboard, GtkTooltip *tooltip) { 
	char cmd[1024];
	gtk_tooltip_set_text(tooltip,fileName(sndr[19],cmd)); 
	return 1;
	}

//------------------------

gboolean on_label4_query_tooltip(GtkWidget *widget, gint x, gint y, 
		gboolean keyboard, GtkTooltip *tooltip) { 
	gtk_tooltip_set_text(tooltip,next_random); 
	return 1;
	}

void	on_entry7_copy_clipboard(GtkEntry *e) {
	copyClipRow=e7Row;
	}

void	on_entry8_copy_clipboard(GtkEntry *e) {
	copyClipRow=e8Row;
	}

void	on_entry9_copy_clipboard(GtkEntry *e) {
	copyClipRow=e9Row;
	}

void	on_entry10_copy_clipboard(GtkEntry *e) {
	copyClipRow=e10Row;
	}

void	on_entry11_copy_clipboard(GtkEntry *e) {
	copyClipRow=e11Row;
	}

void	on_entry12_copy_clipboard(GtkEntry *e) {
	copyClipRow=e12Row;
	}

void	on_entry13_copy_clipboard(GtkEntry *e) {
	copyClipRow=e13Row;
	}

void	on_entry14_copy_clipboard(GtkEntry *e) {
	copyClipRow=e14Row;
	}

void	on_entry2_copy_clipboard(GtkEntry *e) {
	copyClipRow=e15Row;
	}

void	on_entry3_copy_clipboard(GtkEntry *e) {
	copyClipRow=e16Row;
	}

void	on_entry4_copy_clipboard(GtkEntry *e) {
	copyClipRow=e17Row;
	}

void	on_entry15_copy_clipboard(GtkEntry *e) {
	copyClipRow=e18Row;
	}

// -------- top rows

void	on_entry1_copy_clipboard(GtkEntry *e) {
	copyClipRow = -10;
	}

void	on_entry1_cut_clipboard(GtkEntry *e) {
	copyClipRow = -10;
	}

void	on_entry5_copy_clipboard(GtkEntry *e) {
	copyClipRow = -20;
	}

void	on_entry5_cut_clipboard(GtkEntry *e) {
	copyClipRow = -20;
	}

void	on_entry6_copy_clipboard(GtkEntry *e) {
	copyClipRow = -30;
	}

void	on_entry6_cut_clipboard(GtkEntry *e) {
	copyClipRow = -30;
	}

void	on_entry1_paste_clipboard(GtkEntry *e) {

	if (copyClipRow>=0) {
		char file[1024],file1[1024];
		int i=0, j=0;
		gtk_editable_delete_text (GTK_EDITABLE(e), 0, -1 );
		strcpy(file, (char *) sounder[copyClipRow]);
		hexcleaner(file);
		if (strncmp(file,"file://",7)==0) { 
			strcpy(file1,&file[7]); 
			strcpy(file,file1); 
			}
		gtk_entry_set_text(e, (gchar *) file); 
		return;
		}
	}

void	on_entry5_paste_clipboard(GtkEntry *e) {

	if (copyClipRow>=0) {
		char file[1024],file1[1024];
		int i=0, j=0;
		gtk_editable_delete_text (GTK_EDITABLE(e), 0, -1 );
		strcpy(file, (char *) sounder[copyClipRow]);
		hexcleaner(file);
		if (strncmp(file,"file://",7)==0) { 
			strcpy(file1,&file[7]); 
			strcpy(file,file1); 
			}
		gtk_entry_set_text(e, (gchar *) file); 
		return;
		}
	}

void	on_entry6_paste_clipboard(GtkEntry *e) {

	if (copyClipRow>=0) {
		char file[1024],file1[1024];
		int i=0, j=0;
		gtk_editable_delete_text (GTK_EDITABLE(e), 0, -1 );
		strcpy(file, (char *) sounder[copyClipRow]);
		hexcleaner(file);
		if (strncmp(file,"file://",7)==0) { 
			strcpy(file1,&file[7]); 
			strcpy(file,file1); 
			}
		gtk_entry_set_text(e, (gchar *) file); 
		return;
		}
	}

// -------- bottom 12 rows

void	on_entry7_paste_clipboard (GtkEntry *b) { 
		paste(&e7Row);
		doSounders = 1;
		}

void	on_entry8_paste_clipboard (GtkEntry *b) { 
		paste(&e8Row);
		doSounders = 1;
		}

void    on_entry9_paste_clipboard (GtkEntry *b) {
		paste(&e9Row);
		doSounders = 1;
                }

void    on_entry10_paste_clipboard (GtkEntry *b) {
		paste(&e10Row);
		doSounders = 1;
                }

void    on_entry11_paste_clipboard (GtkEntry *b) {
		paste(&e11Row);
		doSounders = 1;
		}

void    on_entry12_paste_clipboard (GtkEntry *b) {
		paste(&e12Row);
		doSounders = 1;
		}

void    on_entry13_paste_clipboard (GtkEntry *b) {
		paste(&e13Row);
		doSounders = 1;
		}

void    on_entry14_paste_clipboard (GtkEntry *b) {
		paste(&e14Row);
		doSounders = 1;
		}

void    on_entry2_paste_clipboard (GtkEntry *b) {
		paste(&e15Row);
		doSounders = 1;
		}

void    on_entry3_paste_clipboard (GtkEntry *b) {
		paste(&e16Row);
		doSounders = 1;
		}

void    on_entry4_paste_clipboard (GtkEntry *b) {
		paste(&e17Row);
		doSounders = 1;
		}

void    on_entry15_paste_clipboard (GtkEntry *b) {
		paste(&e18Row);
		doSounders = 1;
		}

void    on_entry7_cut_clipboard (GtkEntry *b) {
		strcpy(cutRow, sounder[e7Row]);
                strcpy(cutTime, sounderTime[e7Row]->Time);
		cutPlay = sounderTime[e7Row]->play;
                cutPlayed = sounderPlayed[e7Row];
		delRow(e7Row);
                sounders();
                }

void    on_entry8_cut_clipboard (GtkEntry *b) {
		strcpy(cutRow,sounder[e8Row]);
                strcpy(cutTime,sounderTime[e8Row]->Time);
		cutPlay = sounderTime[e8Row]->play;
                cutPlayed=sounderPlayed[e8Row];
		delRow(e8Row);
                sounders();
                }

void    on_entry9_cut_clipboard (GtkEntry *b) {
		strcpy(cutRow,sounder[e9Row]);
                strcpy(cutTime,sounderTime[e9Row]->Time);
                cutPlayed=sounderPlayed[e9Row];
		cutPlay = sounderTime[e9Row]->play;
		delRow(e9Row);
                sounders();
                }

void    on_entry10_cut_clipboard (GtkEntry *b) {
		strcpy(cutRow,sounder[e10Row]);
                strcpy(cutTime,sounderTime[e10Row]->Time);
		cutPlay = sounderTime[e10Row]->play;
                cutPlayed=sounderPlayed[e10Row];
		delRow(e10Row);
                sounders();
                }

void    on_entry11_cut_clipboard (GtkEntry *b) {
		strcpy(cutRow,sounder[e11Row]);
                strcpy(cutTime,sounderTime[e11Row]->Time);
		cutPlay = sounderTime[e11Row]->play;
                cutPlayed=sounderPlayed[e11Row];
		delRow(e11Row);
                sounders();
                }

void    on_entry12_cut_clipboard (GtkEntry *b) {
		strcpy(cutRow,sounder[e12Row]);
                strcpy(cutTime,sounderTime[e12Row]->Time);
		cutPlay = sounderTime[e12Row]->play;
                cutPlayed=sounderPlayed[e12Row];
		delRow(e12Row);
                sounders();
                }

void    on_entry13_cut_clipboard (GtkEntry *b) {
		strcpy(cutRow,sounder[e13Row]);
                strcpy(cutTime,sounderTime[e13Row]->Time);
		cutPlay = sounderTime[e13Row]->play;
                cutPlayed=sounderPlayed[e13Row];
		delRow(e13Row);
                sounders();
                }

void    on_entry14_cut_clipboard (GtkEntry *b) {
		strcpy(cutRow,sounder[e14Row]);
                strcpy(cutTime,sounderTime[e14Row]->Time);
		cutPlay = sounderTime[e14Row]->play;
                cutPlayed=sounderPlayed[e14Row];
		delRow(e14Row);
                sounders();
                }

void    on_entry2_cut_clipboard (GtkEntry *b) {
		strcpy(cutRow,sounder[e15Row]);
                strcpy(cutTime,sounderTime[e15Row]->Time);
		cutPlay = sounderTime[e15Row]->play;
                cutPlayed=sounderPlayed[e15Row];
		delRow(e15Row);
                sounders();
                }

void    on_entry3_cut_clipboard (GtkEntry *b) {
		strcpy(cutRow,sounder[e16Row]);
                strcpy(cutTime,sounderTime[e16Row]->Time);
		cutPlay = sounderTime[e16Row]->play;
                cutPlayed=sounderPlayed[e16Row];
		delRow(e16Row);
                sounders();
                }

void    on_entry4_cut_clipboard (GtkEntry *b) {
		strcpy(cutRow,sounder[e17Row]);
                strcpy(cutTime,sounderTime[e17Row]->Time);
		cutPlay = sounderTime[e17Row]->play;
                cutPlayed=sounderPlayed[e17Row];
		delRow(e17Row);
                sounders();
                }

void    on_entry15_cut_clipboard (GtkEntry *b) {
		strcpy(cutRow,sounder[e18Row]);
                strcpy(cutTime,sounderTime[e18Row]->Time);
		cutPlay = sounderTime[e18Row]->play;
                cutPlayed=sounderPlayed[e18Row];
		delRow(e18Row);
                sounders();
                }

void	paste(int *Row) {

	int i,j,played,play;
	char time[32],file[512];

	if (strlen(cutRow)) {
		paste1(Row);
		strcpy(sounder[*Row], cutRow);
		strcpy(sounderTime[*Row]->Time, cutTime);
		sounderTime[*Row]->play = cutPlay;
		sounderPlayed[*Row] = cutPlayed;
		strcpy(cutRow,"");
		sounders();
		return;
		}

                if (*Row == copyClipRow) {
                        Message("Copy to same row");
                        return;
                        }

                if (copyClipRow < 0 ) {
			paste1(Row);
			return;
                        Message("No source row selected");
                        return;
                        }

                if (*scount+1 >= SOUNDER_MAX) {
                        Message("Maximum number of program slots exceeded");
                        return;
                        }

                strcpy(file,sounder[copyClipRow]);
                strcpy(time,sounderTime[copyClipRow]->Time);
		play = sounderTime[copyClipRow]->play;
                played=sounderPlayed[copyClipRow];

                for (i=*scount+1; i>*Row; i--) {        // shift entries up one
                        strcpy(sounder[i], sounder[i-1]);
                        strcpy(sounderTime[i]->Time, sounderTime[i-1]->Time);
			sounderTime[i]->play = sounderTime[i-1]->play;
                        sounderPlayed[i] = sounderPlayed[i-1];
                        }

                strcpy(sounder[*Row], file);   // insert new entry
                strcpy(sounderTime[*Row]->Time, time);
		sounderTime[*Row]->play = play;
                sounderPlayed[*Row] = played;
                (*scount)++;
		inFlg=1;
                sounders();
                copyClipRow = -1;
		inFlg=0;

	if (*Row <= *CurrentPlay ) {
		*CurrentPlay = *CurrentPlay +1;
		for (j = strlen(sounder[*CurrentPlay]); j >= 0 && sounder[*CurrentPlay][j] != '/'; j--);
		sprintf(currentPlay,"Curr: %d %s", *CurrentPlay, &sounder[*CurrentPlay][j+1]);
		}

                }

void	paste1(int *Row) {

                int i,j,played;

                if (*scount+1 >= SOUNDER_MAX) {
                        Message("Maximum number of program slots exceeded");
                        return;
                        }

                printf("copy to row %d copyClipRow=%d\n",*Row,copyClipRow);

                for (i=*scount+1; i>*Row; i--) {        // shift entries up one
                        strcpy(sounder[i], sounder[i-1]);
                        strcpy(sounderTime[i]->Time, sounderTime[i-1]->Time);
			sounderTime[i]->play = sounderTime[i-1]->play;
                        sounderPlayed[i] = sounderPlayed[i-1];
                        }

		if (copyClipRow == -10) { strcpy(sounder[*Row],top1); }
		if (copyClipRow == -20) { strcpy(sounder[*Row],top2); }
		if (copyClipRow == -30) { strcpy(sounder[*Row],top3); }
		sounderTime[*Row]->play=1;
                (*scount)++;

	if (*Row <= *CurrentPlay ) {
		*CurrentPlay = *CurrentPlay +1;
		for (j = strlen(sounder[*CurrentPlay]); j >= 0 && sounder[*CurrentPlay][j] != '/'; j--);
		sprintf(currentPlay,"Curr: %d %s", *CurrentPlay, &sounder[*CurrentPlay][j+1]);
		}
                sounders();
                }

void	on_entry1_drag_data_received(GtkEntry *e, GdkDragContext   *context) {
	char file[1024],file1[1024];

	gtk_drag_finish (context, TRUE, FALSE, 0);

	if (copyClipRow>=0) { strcpy(top1,sounder[copyClipRow]); }
	else return;

	strcpy(file,top1);
	int i=strlen(file);

	for (int j=0; j<i; j++) if (file[j] == ':') { file[j]=0; break;}

	i=strlen(file);
	for (i=strlen(file); i>0; i--) if (file[i]=='/') break;
	file[i]=0;

	strcpy(directories[0],file);
}

void	on_entry5_drag_data_received(GtkEntry *e, GdkDragContext *context) {
	char file[1024],file1[1024];

	gtk_drag_finish (context, TRUE, FALSE, 0);

	if (copyClipRow>=0) { strcpy(top2,sounder[copyClipRow]); }
	else return;

	strcpy(file,top2);
	int i=strlen(file);

	for (int j=0; j<i; j++) if (file[j] == ':') { file[j]=0; break;}

	i=strlen(file);
	for (i=strlen(file); i>0; i--) if (file[i]=='/') break;
	file[i]=0;

	strcpy(directories[1],file);
	}

void	on_entry6_drag_data_received(GtkEntry *e, GdkDragContext *context) {
	char file[1024],file1[1024];
	gtk_drag_finish (context, TRUE, FALSE, 0);
	if (copyClipRow>=0) { strcpy(top3,sounder[copyClipRow]); }
	else return;
	strcpy(file,top3);
	int i=strlen(file);
	for (int j=0; j<i; j++) if (file[j] == ':') { file[j]=0; break;}

	i=strlen(file);
	for (i=strlen(file); i>0; i--) if (file[i]=='/') break;
	file[i]=0;

	strcpy(directories[3],file);
	}

// The following receive draged file names 

void	on_entry7_drag_data_received(GtkButton *b) { 
		if (copyClipRow >= 0 ) paste(&e7Row);
		else paste1(&e7Row);
		}

void	on_entry8_drag_data_received(GtkButton *b) { 
		if (copyClipRow >= 0 ) paste(&e8Row);
		else paste1(&e8Row);
		}

void	on_entry9_drag_data_received(GtkButton *b) { 
		if (copyClipRow >= 0 ) paste(&e9Row);
		else paste1(&e9Row);
		}

void	on_entry10_drag_data_received(GtkButton *b) { 
		if (copyClipRow >= 0 ) paste(&e10Row);
		else paste1(&e10Row);
		}

void	on_entry11_drag_data_received(GtkButton *b) { 
		if (copyClipRow >= 0 ) paste(&e11Row);
		else paste1(&e11Row);
		}

void	on_entry12_drag_data_received(GtkButton *b) { 
		if (copyClipRow >= 0 ) paste(&e12Row);
		else paste1(&e12Row);
		}

void	on_entry13_drag_data_received(GtkButton *b) { 
		if (copyClipRow >= 0 ) paste(&e13Row);
		else paste1(&e13Row);
		}

void	on_entry14_drag_data_received(GtkButton *b) { 
		if (copyClipRow>=0) paste(&e14Row);
		else paste1(&e14Row);
		}

void	on_entry2_drag_data_received(GtkButton *b) { 
		if (copyClipRow>=0) paste(&e15Row);
		else paste1(&e15Row);
		}

void	on_entry3_drag_data_received(GtkButton *b) { 
		if (copyClipRow>=0) paste(&e16Row);
		else paste1(&e16Row);
		}

void	on_entry4_drag_data_received(GtkButton *b) { 
		if (copyClipRow>=0) paste(&e17Row);
		else paste1(&e17Row);
		}

void	on_entry15_drag_data_received(GtkButton *b) { 
		if (copyClipRow>=0) paste(&e18Row);
		else paste1(&e18Row);
		}

void	delRow(int Row) {
                int i,played;
                char time[32],file[512];

                if (*scount == 0) {
                        Message("Empty List");
                        return;
                        }

                printf("delete row %d\n",Row);

		for (i = Row; i < *scount; i++) {
                        strcpy(sounder[i], sounder[i+1]);
                        strcpy(sounderTime[i]->Time, sounderTime[i+1]->Time);
			sounderTime[i]->play = sounderTime[i+1]->play;
                        sounderPlayed[i] = sounderPlayed[i+1];
                        }

                (*scount)--;
                sounders();
		}

void	on_del7_clicked(GtkButton *b) {
		if (delDialog() == 200) return;
		delRow(e7Row);
		}

void	on_del8_clicked(GtkButton *b) {
		if (delDialog() == 200) return;
		delRow(e8Row);
		}

void	on_del9_clicked(GtkButton *b) {
		if (delDialog() == 200) return;
		delRow(e9Row);
		}

void	on_del10_clicked(GtkButton *b) {
		if (delDialog() == 200) return;
		delRow(e10Row);
		}

void	on_del11_clicked(GtkButton *b) {
		if (delDialog() == 200) return;
		delRow(e11Row);
		}

void	on_del12_clicked(GtkButton *b) {
		if (delDialog() == 200) return;
		delRow(e12Row);
		}

void	on_del13_clicked(GtkButton *b) {
		if (delDialog() == 200) return;
		delRow(e13Row);
		}

void	on_del14_clicked(GtkButton *b) {
		delRow(e14Row);
		if (delDialog() == 200) return;
		}

void	on_del1_clicked(GtkButton *b) {
		if (delDialog() == 200) return;
		if (delDialog() == 200) return;
		delRow(e15Row);
		}

void	on_del2_clicked(GtkButton *b) {
		if (delDialog() == 200) return;
		delRow(e16Row);
		}

void	on_del3_clicked(GtkButton *b) {
		if (delDialog() == 200) return;
		delRow(e17Row);
		}

void	on_del4_clicked(GtkButton *b) {
		if (delDialog() == 200) return;
		delRow(e18Row);
		}

void	on_entry1_drag_data_get(GtkEntry *e)	{ copyClipRow = -10; }
void	on_entry5_drag_data_get(GtkEntry *e)	{ copyClipRow = -20; }
void	on_entry6_drag_data_get(GtkEntry *e)	{ copyClipRow = -30; }

void	on_entry7_drag_data_get(GtkEntry *e)	{ copyClipRow=e7Row; }
void	on_entry8_drag_data_get(GtkEntry *e)	{ copyClipRow=e8Row; }
void	on_entry9_drag_data_get(GtkEntry *e)	{ copyClipRow=e9Row; }
void	on_entry10_drag_data_get(GtkEntry *e)	{ copyClipRow=e10Row; }
void	on_entry11_drag_data_get(GtkEntry *e)	{ copyClipRow=e11Row; }
void	on_entry12_drag_data_get(GtkEntry *e)	{ copyClipRow=e12Row; }
void	on_entry13_drag_data_get(GtkEntry *e)	{ copyClipRow=e13Row; }
void	on_entry14_drag_data_get(GtkEntry *e)	{ copyClipRow=e14Row; }
void	on_entry2_drag_data_get(GtkEntry *e)	{ copyClipRow=e15Row; }
void	on_entry3_drag_data_get(GtkEntry *e)	{ copyClipRow=e16Row; }
void	on_entry4_drag_data_get(GtkEntry *e)	{ copyClipRow=e17Row; }
void	on_entry15_drag_data_get(GtkEntry *e)	{ copyClipRow=e18Row; }

void	on_entry1_button_press_event(GtkEntry *entry, GdkEventButton *event) {
	gtk_editable_select_region (GTK_EDITABLE(entry), 0, -1);
	}

void	on_entry5_button_press_event(GtkEntry *entry, GdkEventButton *event) {
	gtk_editable_select_region (GTK_EDITABLE(entry), 0, -1);
	}

void	on_entry6_button_press_event(GtkEntry *entry, GdkEventButton *event) {
	gtk_editable_select_region (GTK_EDITABLE(entry), 0, -1);
	}

void	on_entry7_button_press_event(GtkEntry *entry, GdkEventButton *event) {
	gtk_editable_select_region (GTK_EDITABLE(entry), 0, -1);
	}

void	on_entry8_button_press_event(GtkEntry *entry, GdkEventButton *event) {
	gtk_editable_select_region (GTK_EDITABLE(entry), 0, -1);
	}

void	on_entry9_button_press_event(GtkEntry *entry, GdkEventButton *event) {
	gtk_editable_select_region (GTK_EDITABLE(entry), 0, -1);
	}

void	on_entry10_button_press_event(GtkEntry *entry, GdkEventButton *event) {
	gtk_editable_select_region (GTK_EDITABLE(entry), 0, -1);
	}

void	on_entry11_button_press_event(GtkEntry *entry, GdkEventButton *event) {
	gtk_editable_select_region (GTK_EDITABLE(entry), 0, -1);
	}

void	on_entry12_button_press_event(GtkEntry *entry, GdkEventButton *event) {
	gtk_editable_select_region (GTK_EDITABLE(entry), 0, -1);
	}

void	on_entry13_button_press_event(GtkEntry *entry, GdkEventButton *event) {
	gtk_editable_select_region (GTK_EDITABLE(entry), 0, -1);
	}

void	on_entry14_button_press_event(GtkEntry *entry, GdkEventButton *event) {
	gtk_editable_select_region (GTK_EDITABLE(entry), 0, -1);
	}

void	on_entry2_button_press_event(GtkEntry *entry, GdkEventButton *event) {
	gtk_editable_select_region (GTK_EDITABLE(entry), 0, -1);
	}

void	on_entry3_button_press_event(GtkEntry *entry, GdkEventButton *event) {
	gtk_editable_select_region (GTK_EDITABLE(entry), 0, -1);
	}

void	on_entry4_button_press_event(GtkEntry *entry, GdkEventButton *event) {
	gtk_editable_select_region (GTK_EDITABLE(entry), 0, -1);
	}

void	on_entry15_button_press_event(GtkEntry *entry, GdkEventButton *event) {
	gtk_editable_select_region (GTK_EDITABLE(entry), 0, -1);
	}

gboolean on_save1_button_press_event (GtkButton *btn, GdkEventButton *event, 
		gpointer user_data) { 
	if (event->button == 1 ) return FALSE;
	SaveFlg = 1;
	gtk_window_set_title(GTK_WINDOW(g_dbox1),"File name to read");
	gtk_widget_show(g_dbox1);  // show dialog
	return TRUE;
	}

void	on_save1_clicked (GtkButton *b) {
	SaveFlg = 0;
	gtk_window_set_title(GTK_WINDOW(g_dbox1),"File name to save");
	gtk_widget_show(g_dbox1);  // show dialog
	return;
	}

void	on_dxb2_clicked(GtkButton *b) { // cancel button
	gtk_widget_hide(g_dbox1);                
	}

void	on_dxb1_clicked(GtkButton *b) { // ok button

	FILE *f1;
	int i;
	char fileName[512];
		
	gtk_widget_hide(g_dbox1);                

	const gchar *sentry = gtk_entry_get_text( g_dbox1Entry );

	if (strlen(sentry) == 0) {
		Message("Bad File Name");
		return;
		}

	strcpy(fileName,Code_Home); // build filename
	strcat(fileName,"/");
	strcat(fileName,sentry);

	if (SaveFlg) {
		printf("Loading program list from %s\n",fileName);
		LoadProgram(fileName);
		sounders( );
		return;
		}

	printf("Saving program list to %s\n",fileName);

	f1 = fopen (fileName,"w");
	if (f1 ==NULL ) {
		Message("File Error Saving Program List");
		return;
		}

	for (i=0; i<*scount; i++) {
		if (strlen(sounder[i]) == 0 ) continue;
		fprintf(f1,"%s#%d#%s\n", sounderTime[i]->Time, sounderTime[i]->play, sounder[i]);
		}

	fclose(f1);
	gtk_widget_hide(g_dbox1);                
	}

void	on_showCurrent_clicked (GtkButton *b) {
	if (*CurrentPlay >=0 ) *NowPlaying = *CurrentPlay;
	}

void	on_check1_toggled(GtkToggleButton *b) {
	if (c1state) {
		gtk_toggle_button_set_active (b, FALSE);
		c1state=0;
		sounderTime[*begin+0]->play = 0;
		}
	else {
		gtk_toggle_button_set_active (b, TRUE);
		sounderTime[*begin+0]->play = 1;
		c1state=1;
		}
	}

gboolean on_up1_button_press_event 
	(GtkButton *btn, GdkEventButton *event, gpointer user_data) { 
	if (event->button == 1 ) return FALSE;
	moveToBottom(*begin+0);
	return TRUE;
	}

gboolean on_up2_button_press_event 
	(GtkButton *btn, GdkEventButton *event, gpointer user_data) { 
	if (event->button == 1 ) return FALSE;
	moveToBottom(*begin+1);
	return TRUE;
	}

gboolean on_up3_button_press_event 
	(GtkButton *btn, GdkEventButton *event, gpointer user_data) { 
	if (event->button == 1 ) return FALSE;
	moveToBottom(*begin+2);
	return TRUE;
	}

gboolean on_up4_button_press_event 
	(GtkButton *btn, GdkEventButton *event, gpointer user_data) { 
	if (event->button == 1 ) return FALSE;
	moveToBottom(*begin+3);
	return TRUE;
	}

gboolean on_up5_button_press_event 
	(GtkButton *btn, GdkEventButton *event, gpointer user_data) { 
	if (event->button == 1 ) return FALSE;
	moveToBottom(*begin+4);
	return TRUE;
	}

gboolean on_up6_button_press_event 
	(GtkButton *btn, GdkEventButton *event, gpointer user_data) { 
	if (event->button == 1 ) return FALSE;
	moveToBottom(*begin+5);
	return TRUE;
	}

gboolean on_up7_button_press_event 
	(GtkButton *btn, GdkEventButton *event, gpointer user_data) { 
	if (event->button == 1 ) return FALSE;
	moveToBottom(*begin+6);
	return TRUE;
	}

gboolean on_up8_button_press_event 
	(GtkButton *btn, GdkEventButton *event, gpointer user_data) { 
	if (event->button == 1 ) return FALSE;
	moveToBottom(*begin+7);
	return TRUE;
	}

gboolean on_up9_button_press_event 
	(GtkButton *btn, GdkEventButton *event, gpointer user_data) { 
	if (event->button == 1 ) return FALSE;
	moveToBottom(*begin+8);
	return TRUE;
	}

gboolean on_up10_button_press_event 
	(GtkButton *btn, GdkEventButton *event, gpointer user_data) { 
	if (event->button == 1 ) return FALSE;
	moveToBottom(*begin+9);
	return TRUE;
	}

gboolean on_up11_button_press_event 
	(GtkButton *btn, GdkEventButton *event, gpointer user_data) { 
	if (event->button == 1 ) return FALSE;
	moveToBottom(*begin+10);
	return TRUE;
	}

gboolean on_up12_button_press_event 
	(GtkButton *btn, GdkEventButton *event, gpointer user_data) { 
	if (event->button == 1 ) return FALSE;
	moveToBottom(*begin+11);
	return TRUE;
	}

void	on_up1_clicked (GtkButton *btn) { \
	moveToTop(*begin+0);
	}

void on_up2_clicked (GtkButton *btn) { \
	moveToTop(*begin+1);
	}

void on_up3_clicked (GtkButton *btn) { \
	moveToTop(*begin+2);
	}

void on_up4_clicked (GtkButton *btn) { \
	moveToTop(*begin+3);
	}

void on_up5_clicked (GtkButton *btn) { \
	moveToTop(*begin+4);
	}

void on_up6_clicked (GtkButton *btn) { \
	moveToTop(*begin+5);
	}

void on_up7_clicked (GtkButton *btn) { \
	moveToTop(*begin+6);
	}

void on_up8_clicked (GtkButton *btn) { \
	moveToTop(*begin+7);
	}

void on_up9_clicked (GtkButton *btn) { \
	moveToTop(*begin+8);
	}

void on_up10_clicked (GtkButton *btn) { \
	moveToTop(*begin+9);
	}

void on_up11_clicked (GtkButton *btn) { \
	moveToTop(*begin+10);
	}

void on_up12_clicked (GtkButton *btn) { \
	moveToTop(*begin+11);
	}

void	moveToTop (int row) {
	int i;
	char row_entry[PGM_MAX];
	struct ST sndr;
	int played;

	if ( row == 0 ) return;

	// caputure current row

	strcpy(row_entry, sounder[row]);
	strcpy(sndr.Time, sounderTime[row]->Time);
	sndr.play = sounderTime[row]->play;
	played = sounderPlayed[row];

	delRow ( row );

	for (i = *scount; i>0; i--) {        // shift entries up one
		strcpy(sounder[i], sounder[i-1]);
		strcpy(sounderTime[i]->Time, sounderTime[i-1]->Time);
		sounderTime[i]->play = sounderTime[i-1]->play;
		sounderPlayed[i] = sounderPlayed[i-1];
		}

	strcpy(sounder[0], row_entry);   // insert new entry
	strcpy(sounderTime[0]->Time, sndr.Time);
//	sounderTime[0]->play = sndr.play;
	sounderTime[0]->play = 1; // checked
	sounderPlayed[0] = played;
	if ( *CurrentPlay == row) *CurrentPlay = 0;
	else if (*CurrentPlay > -1 ) *CurrentPlay = *CurrentPlay + 1;
	(*scount)++;
	sounders();
	}

void    moveToBottom (int row) {
        int i;
        char row_entry[PGM_MAX];
        struct ST sndr;
        int played;

        // caputure current row

        strcpy(row_entry, sounder[row]);
        strcpy(sndr.Time, sounderTime[row]->Time);
        sndr.play = sounderTime[row]->play;
        played = sounderPlayed[row];

        delRow ( row );

        strcpy(sounder[*scount], row_entry);   // insert new entry
        strcpy(sounderTime[*scount]->Time, sndr.Time);
        sounderTime[*scount]->play = sndr.play;
        sounderPlayed[*scount] = played;
        (*scount)++;
        sounders();
        }

void	on_check2_toggled(GtkToggleButton *b) {
	if (c2state) {
		gtk_toggle_button_set_active (b, FALSE);
		c2state=0;
		sounderTime[*begin+1]->play = 0;
		}
	else {
		gtk_toggle_button_set_active (b, TRUE);
		c2state=1;
		sounderTime[*begin+1]->play = 1;
		}
	}

void	on_check3_toggled(GtkToggleButton *b) {
	if (c3state) {
		gtk_toggle_button_set_active (b, FALSE);
		c3state=0;
		sounderTime[*begin+2]->play = 0;
		}
	else {
		gtk_toggle_button_set_active (b, TRUE);
		c3state=1;
		sounderTime[*begin+2]->play = 1;
		}
	}

void	on_check4_toggled(GtkToggleButton *b) {
	if (c4state) {
		gtk_toggle_button_set_active (b, FALSE);
		c4state=0;
		sounderTime[*begin+3]->play = 0;
		}
	else {
		gtk_toggle_button_set_active (b, TRUE);
		c4state=1;
		sounderTime[*begin+3]->play = 1;
		}
	}

void	on_check5_toggled(GtkToggleButton *b) {
	if (c5state) {
		gtk_toggle_button_set_active (b, FALSE);
		c5state=0;
		sounderTime[*begin+4]->play = 0;
		}
	else {
		gtk_toggle_button_set_active (b, TRUE);
		c5state=1;
		sounderTime[*begin+4]->play = 1;
		}
	}

void	on_check6_toggled(GtkToggleButton *b) {
	if (c6state) {
		gtk_toggle_button_set_active (b, FALSE);
		c6state=0;
		sounderTime[*begin+5]->play = 0;
		}
	else {
		gtk_toggle_button_set_active (b, TRUE);
		c6state=1;
		sounderTime[*begin+5]->play = 1;
		}
	}

void	on_check7_toggled(GtkToggleButton *b) {
	if (c7state) {
		gtk_toggle_button_set_active (b, FALSE);
		c7state=0;
		sounderTime[*begin+6]->play = 0;
		}
	else {
		gtk_toggle_button_set_active (b, TRUE);
		c7state=1;
		sounderTime[*begin+6]->play = 1;
		}
	}

void	on_check8_toggled(GtkToggleButton *b) {
	if (c8state) {
		gtk_toggle_button_set_active (b, FALSE);
		c8state=0;
		sounderTime[*begin+7]->play = 0;
		}
	else {
		gtk_toggle_button_set_active (b, TRUE);
		c8state=1;
		sounderTime[*begin+7]->play = 1;
		}
	}

void	on_check9_toggled(GtkToggleButton *b) {
	if (c9state) {
		gtk_toggle_button_set_active (b, FALSE);
		c9state=0;
		sounderTime[*begin+8]->play = 0;
		}
	else {
		gtk_toggle_button_set_active (b, TRUE);
		c9state=1;
		sounderTime[*begin+8]->play = 1;
		}
	}

void	on_check10_toggled(GtkToggleButton *b) {
	if (c10state) {
		gtk_toggle_button_set_active (b, FALSE);
		c10state=0;
		sounderTime[*begin+9]->play = 0;
		}
	else {
		gtk_toggle_button_set_active (b, TRUE);
		c10state=1;
		sounderTime[*begin+9]->play = 1;
		}
	}

void	on_check11_toggled(GtkToggleButton *b) {
	if (c11state) {
		gtk_toggle_button_set_active (b, FALSE);
		c11state=0;
		sounderTime[*begin+10]->play = 0;
		}
	else {
		gtk_toggle_button_set_active (b, TRUE);
		c11state=1;
		sounderTime[*begin+10]->play = 1;
		}
	}

void	on_check12_toggled(GtkToggleButton *b) {
	if (c12state) {
		gtk_toggle_button_set_active (b, FALSE);
		c12state=0;
		sounderTime[*begin+11]->play = 0;
		}
	else {
		gtk_toggle_button_set_active (b, TRUE);
		c12state=1;
		sounderTime[*begin+11]->play = 1;
		}
	}

void	probe(GtkTooltip *tooltip, int row) {

	char cmd[2048],rslt[2048],*p2;
	FILE *p1;

	strcpy(cmd,"ffprobe \"");
	strcat(cmd, (char *) sounder[row] );
	strcat(cmd, "\" 2>&1 | grep -A20 'Metadata:'");

	p1 = popen(cmd,"r");

	if (p1 == NULL) {
        	gtk_tooltip_set_text(tooltip," "); 
		return;
		}

	strcpy(cmd,"");
	strcpy(rslt,"");

	while (fgets(cmd,2048,p1)) {

		cmd[strlen(cmd)-1] = 0;

		if (p2=strstr(cmd,"Duration:")) {
			strcat(rslt,p2);
			}

		else if (p2=strstr(cmd,"Stream:")) {
			strcat(rslt,"\n");
			strcat(rslt,p2);
			}

		else if (p2=strstr(cmd,"TITLE:")) {
			strcat(rslt,"\n");
			strcat(rslt,p2);
			}

		else if (p2=strstr(cmd,"ALBUM:")) {
			strcat(rslt,"\n");
			strcat(rslt,p2);
			}

		else if (p2=strstr(cmd,"ARTIST:")) {
			strcat(rslt,"\n");
			strcat(rslt,p2);
			}
		}
		
	fclose(p1);

	if (strlen(rslt)==0) strcpy(rslt,"no information available");

        gtk_tooltip_set_text(tooltip,rslt); 
	}

gboolean on_entry7_query_tooltip(GtkWidget *widget,gint x,gint y, 
		gboolean keyboard, GtkTooltip *tooltip) {
	return FALSE;
	probe(tooltip,e7Row);
	return TRUE;
	}

gboolean on_entry8_query_tooltip(GtkWidget *widget,gint x,gint y, 
		gboolean keyboard, GtkTooltip *tooltip) {
	return FALSE;
	probe(tooltip,e8Row);
	return TRUE;
	}

gboolean on_entry9_query_tooltip(GtkWidget *widget,gint x,gint y, 
		gboolean keyboard, GtkTooltip *tooltip) {
	return FALSE;
	probe(tooltip,e9Row);
	return TRUE;
	}

gboolean on_entry10_query_tooltip(GtkWidget *widget,gint x,gint y, 
		gboolean keyboard, GtkTooltip *tooltip) {
	return FALSE;
	probe(tooltip,e10Row);
	return TRUE;
	}

gboolean on_entry11_query_tooltip(GtkWidget *widget,gint x,gint y, 
		gboolean keyboard, GtkTooltip *tooltip) {
	return FALSE;
	probe(tooltip,e11Row);
	return TRUE;
	}

gboolean on_entry12_query_tooltip(GtkWidget *widget,gint x,gint y, 
		gboolean keyboard, GtkTooltip *tooltip) {
	return FALSE;
	probe(tooltip,e12Row);
	return TRUE;
	}

gboolean on_entry13_query_tooltip(GtkWidget *widget,gint x,gint y, 
		gboolean keyboard, GtkTooltip *tooltip) {
	return FALSE;
	probe(tooltip,e13Row);
	return TRUE;
	}

gboolean on_entry14_query_tooltip(GtkWidget *widget,gint x,gint y, 
		gboolean keyboard, GtkTooltip *tooltip) {
	return FALSE;
	probe(tooltip,e14Row);
	return TRUE;
	}

gboolean on_entry2_query_tooltip(GtkWidget *widget,gint x,gint y, 
		gboolean keyboard, GtkTooltip *tooltip) {
	return FALSE;
	probe(tooltip,e15Row);
	return TRUE;
	}

gboolean on_entry3_query_tooltip(GtkWidget *widget,gint x,gint y, 
		gboolean keyboard, GtkTooltip *tooltip) {
	return FALSE;
	probe(tooltip,e16Row);
	return TRUE;
	}

gboolean on_entry4_query_tooltip(GtkWidget *widget,gint x,gint y, 
		gboolean keyboard, GtkTooltip *tooltip) {
	return FALSE;
	probe(tooltip,e17Row);
	return TRUE;
	}

gboolean on_entry15_query_tooltip(GtkWidget *widget,gint x,gint y, 
		gboolean keyboard, GtkTooltip *tooltip) {
	return FALSE;
	probe(tooltip,e18Row);
	return TRUE;
	}

void    on_clrPgm_clicked (GtkButton *b) {

	int i;

	GtkDialogFlags flags = GTK_DIALOG_DESTROY_WITH_PARENT;

	GtkWidget *dialog = gtk_message_dialog_new (GTK_WINDOW(window),
		flags,
		GTK_MESSAGE_ERROR,
		GTK_BUTTONS_YES_NO,
		"%s\nAre You Sure?",
		"Erase Program");

//	expect -9 for NO and -8 for YES

	gtk_window_set_keep_above(GTK_WINDOW(dialog), TRUE);

	gint r = gtk_dialog_run (GTK_DIALOG (dialog));

	gtk_widget_destroy (dialog);

	if ( r == -8) {

		for (i=0; i< SOUNDER_MAX; i++) {

			strcpy(sounder[i],"");
			sounderTime[i]->play = 0;
			sounderPlayed[i] = 0;
			strcpy(sounderTime[i]->Time, "");	// record time stamp
			}

		*scount = 0; // sounder list is empty
		sounders();
		*begin=0;
		}

        }

void	on_player_clicked(GtkButton *b) {

	// switch players

		if (*player == MPV_PLAYER) {
			*player = VLC_PLAYER;
			gtk_button_set_label (GTK_BUTTON(b), (const gchar* ) "VLC");
			}
		else {
			*player = MPV_PLAYER;
			gtk_button_set_label (GTK_BUTTON(b), (const gchar* ) "MPV");
			}
		}

void	on_Audsys_clicked(GtkButton *b) {

	// switch audio systems

	return; // should not let people change this themselves.

		if (*audsys == PULSE) {

        		GtkDialogFlags flags = GTK_DIALOG_DESTROY_WITH_PARENT |
                		GTK_MESSAGE_ERROR | GTK_DIALOG_MODAL | GTK_WINDOW_TOPLEVEL |
                		GTK_DIALOG_USE_HEADER_BAR ;

        		GtkWidget *dialog = gtk_dialog_new_with_buttons  
						("Are You Sure Jack is Running?",
                	GTK_WINDOW(window),
                		flags,
                		"Yes", 100, "No", 200, NULL);

        		gtk_window_set_keep_above(GTK_WINDOW(dialog), TRUE);

        		gint r = gtk_dialog_run (GTK_DIALOG (dialog));

        		gtk_widget_destroy (dialog);

        		if ( r == 200) return; //nope
			*audsys = JACK;
			gtk_button_set_label (GTK_BUTTON(b), (const gchar* ) "Jack");
			}
		else {
			*audsys = PULSE;
			gtk_button_set_label (GTK_BUTTON(b), (const gchar* ) "Pulse");
			}
		}

//-------------------------------------------------------
//	locate and lock media player location
//-------------------------------------------------------

void	on_findVlc_clicked (GtkButton *b) {

	int desktop,hor,ver,wid,hgt;
	unsigned int hexId;
	char cmd[1024];
	FILE *f1;

	if (*player == MPV_PLAYER) {
		f1=popen("wmctrl -lG | grep \"mpv player\"", "r");
		}
	else {
		f1=popen("wmctrl -lG | grep \"VLC media player\"", "r");
		}

	strcpy(cmd,"");

	if (fgets(cmd,1024,f1) != NULL) { // where are they?

		sscanf(cmd,"%x %d %d %d %d %d", &hexId, &desktop,
			vlc_hor, vlc_ver, vlc_wid, vlc_hgt);

		printf("* co-ordinates hor=%d ver=%d wide=%d high=%d\n", 
			*vlc_hor, *vlc_ver, *vlc_wid, *vlc_hgt);
		}

	if (*player == MPV_PLAYER) { 
		*vlc_ver = *vlc_ver - 39;
		}

	else if (*player == VLC_PLAYER) { 
		*vlc_ver = *vlc_ver - 28;
		}

	fclose(f1);
	}

void	on_Playerx_clicked (GtkButton *b) { // reset play buttons to on
	for (int i=0; i< SOUNDER_MAX; i++) sounderTime[i]->play = 1;
	sounders();
	}

void	on_oneTime_clicked (GtkButton *b) {

	GtkWidget *Once, *Many;
	char cmd[1024];

	strcpy(cmd,Code_Home);
	strcat(cmd,"/");

	if (*playonce) {	// many
		*playonce = 0; 
		strcat(cmd,"backgrounds/infinity.png"); // green
		Many = gtk_image_new_from_file (cmd);
		gtk_button_set_image(GTK_BUTTON(b),Many);
		}

	else {	// once
		*playonce = 1;
		strcat(cmd,"backgrounds/once.png"); //red
		Once = gtk_image_new_from_file (cmd);
		gtk_button_set_image(GTK_BUTTON(b),Once);
		}
	}

void	on_orgPlay_clicked(GtkButton *b) {

	if (*OrgPlay) {
		*OrgPlay = 0;
		clearButton(b);
		}
	else {
		*OrgPlay = 1;
		setButton (b);
		*begin = 0;
		on_button30_clicked (GTK_BUTTON(g_button30));
		}
	}

gboolean	on_entry7_scroll_event(GtkEntry *b, GdkEventScroll *e) {
		return scroll_event(GTK_BUTTON(b),e);
		}

gboolean	on_button35_scroll_event(GtkButton *b, GdkEventScroll *e) {
		return scroll_event(b,e);
		}

gboolean	on_button18_scroll_event(GtkButton *b, GdkEventScroll *e) {
		return scroll_event(b,e);
		}

gboolean	on_button18_button_press_event(GtkEntry *entry, GdkEventButton *event) {
		flg18=1;
		flg18a = 0;
		return FALSE;
		}

gboolean	on_button18_button_release_event(GtkEntry *entry, GdkEventButton *event) {
		flg18=0;
		if (flg18a) return TRUE;
		return FALSE;
		}

gboolean	on_button35_button_press_event(GtkEntry *entry, GdkEventButton *event) {
		flg35=1;
		flg35a = 0;
		return FALSE;
		}

gboolean	on_button35_button_release_event(GtkEntry *entry, GdkEventButton *event) {
		flg35 = 0;
		if (flg35a) return TRUE;
		return FALSE;
		}

gboolean	on_button33_scroll_event(GtkButton *b, GdkEventScroll *e) {
		return scroll_event(b,e);
		}

gboolean	on_button58_scroll_event(GtkButton *b, GdkEventScroll *e) {
		return scroll_event(b,e);
		}

gboolean	scroll_event(GtkButton *b, GdkEventScroll *e) {

		// see GdkEventScroll data structure for valid elements
		// see: gtk_widget_set_events(g_buttonxx,GDK_SCROLL_MASK);

		if (e->direction == 1) { // down
			*begin=*begin + 1;
			if (*begin + SOUNDER_BUTTON_COUNT >= *scount) { *begin = *scount - SOUNDER_BUTTON_COUNT; }	
			if (*begin < 0 ) *begin = 0;
			*NowPlaying = -1;
			sounders();
			}
		else { // up
			*begin=*begin - 1;
			if (*begin + SOUNDER_BUTTON_COUNT >= *scount) { *begin = *scount - SOUNDER_BUTTON_COUNT; }	
			if (*begin < 0 ) *begin = 0;
			*NowPlaying = -1;
			sounders();
			}
		return TRUE;
		}

gboolean	on_button3_scroll_event(GtkButton *b, GdkEventScroll *e) {
		return scroll_event(b,e);
		}

gboolean	on_button8_scroll_event(GtkButton *b, GdkEventScroll *e) {
		return scroll_event(b,e);
		}

gboolean	on_button9_scroll_event(GtkButton *b, GdkEventScroll *e) {
		return scroll_event(b,e);
		}

gboolean	on_button10_scroll_event(GtkButton *b, GdkEventScroll *e) {
		return scroll_event(b,e);
		}

gboolean	on_button11_scroll_event(GtkButton *b, GdkEventScroll *e) {
		return scroll_event(b,e);
		}

gboolean	on_button13_scroll_event(GtkButton *b, GdkEventScroll *e) {
		return scroll_event(b,e);
		}

gboolean	on_button12_scroll_event(GtkButton *b, GdkEventScroll *e) {
		return scroll_event(b,e);
		}

gboolean	on_button14_scroll_event(GtkButton *b, GdkEventScroll *e) {
		return scroll_event(b,e);
		}

gboolean	on_button2_scroll_event(GtkButton *b, GdkEventScroll *e) {
		return scroll_event(b,e);
		}

gboolean	on_button5_scroll_event(GtkButton *b, GdkEventScroll *e) {
		return scroll_event(b,e);
		}

gboolean	on_button6_scroll_event(GtkButton *b, GdkEventScroll *e) {
		return scroll_event(b,e);
		}

gboolean	on_button15_scroll_event(GtkButton *b, GdkEventScroll *e) {
		return scroll_event(b,e);
		}

int	delDialog() {

        GtkDialogFlags flags = GTK_DIALOG_DESTROY_WITH_PARENT |
                GTK_MESSAGE_ERROR | GTK_DIALOG_MODAL |
                GTK_DIALOG_USE_HEADER_BAR ;

        GtkWidget *dialog = gtk_dialog_new_with_buttons  ("Delete entry?",
                GTK_WINDOW(window),
                flags,
                "Yes", 100, "No", 200, NULL);

//      expect 200 for NO and 100 for YES

	gtk_window_set_keep_above(GTK_WINDOW(dialog), TRUE);

//      gtk_window_move(GTK_WINDOW(dialog), 300,730);
        gint r = gtk_dialog_run (GTK_DIALOG (dialog));

        gtk_widget_destroy (dialog);

	return (int) r;
	}

void	PangoAttributes() {

	PangoAttrList *attrlist;
	PangoAttribute *attr;
	PangoFontDescription *df;

//struct PangoColor {
//  guint16 red;
//  guint16 green;
//  guint16 blue;
//} pc;
//
//pc.red=0xffff;
//pc.blue=0xffff;
//pc.green=0xffff;

	attrlist = pango_attr_list_new();

// First, let's set up the base attributes.

	df = pango_font_description_new();
	pango_font_description_set_family(df, "Arial Rounded MT");
	pango_font_description_set_size(df, 9 * PANGO_SCALE);
	pango_font_description_set_weight(df, PANGO_WEIGHT_BOLD);

// You can also use pango_font_description_new_from_string() and pass in a string 
// like "Arial Rounded MT Bold (whatever fontsize is)".
// But here's where things change:

	attr = pango_attr_font_desc_new(df);

// This is not documented, but pango_attr_font_desc_new() makes a copy of df, so let's release ours:

	pango_font_description_free(df);

// Pango and GTK+ use UTF-8, so our string is indexed between 0 and 11.
// Note that the end_index is exclusive!

	attr->start_index = 0;

// attr->end_index = strlen(tmp);

	attr->end_index = 100;

pango_attr_list_insert(attrlist, attr);

// And pango_attr_list_insert() takes ownership of attr, so we don't free it ourselves.
// As an alternative to all that, you can have each component of the PangoFontDescriptor be its own attribute; 
// see the PangoAttribute documentation page.

// And now the attribute for the word "World".
// attr = pango_attr_family_new("Arial");
// "World" starts at 6 and ends at 11.
// attr->start_index = 6;
// attr->end_index = 11;
// pango_attr_list_insert(attrlist, attr);

	gtk_entry_set_attributes (GTK_ENTRY(searchEntry1), attrlist);
	gtk_entry_set_attributes (GTK_ENTRY(searchEntry2), attrlist);
	gtk_entry_set_attributes (g_lbl_entry1, attrlist);
	gtk_entry_set_attributes (g_lbl_entry2, attrlist);
	gtk_entry_set_attributes (g_lbl_entry3, attrlist);
	gtk_entry_set_attributes (g_lbl_entry4, attrlist);
	gtk_entry_set_attributes (g_lbl_entry5, attrlist);
	gtk_entry_set_attributes (g_lbl_entry6, attrlist);
	gtk_entry_set_attributes (g_lbl_entry7, attrlist);
	gtk_entry_set_attributes (g_lbl_entry8, attrlist);
	gtk_entry_set_attributes (g_lbl_entry9, attrlist);
	gtk_entry_set_attributes (g_lbl_entry10, attrlist);
	gtk_entry_set_attributes (g_lbl_entry11, attrlist);
	gtk_entry_set_attributes (g_lbl_entry12, attrlist);
	gtk_entry_set_attributes (g_lbl_entry13, attrlist);
	gtk_entry_set_attributes (g_lbl_entry14, attrlist);
	gtk_entry_set_attributes (g_lbl_entry15, attrlist);

	pango_attr_list_unref(attrlist);

//	GdkColor color; // default background color
//	color.red = 0x3333;
//	color.green = 0x3333;
//	color.blue = 0xffff;
//
//	gtk_widget_modify_fg (GTK_WIDGET(g_lbl_entry7), 
//		GTK_STATE_FLAG_NORMAL, &color);
//
//gtk_entry_set_has_frame(g_lbl_entry7, FALSE);

	}

void	setCssAttributes( GtkCssProvider * cssProvider1) {

//---------------------------------------------------------------------
//	set css styles onto objects
//	this applies the CSS file loaded above to the components named.
//	there is probably a better way to do this.
//---------------------------------------------------------------------

	css_set(cssProvider1, GTK_WIDGET(g_lbl_entry7));
	css_set(cssProvider1, GTK_WIDGET(g_lbl_entry6));
	css_set(cssProvider1, showCurrent);
	css_set(cssProvider1, clrPgm);
	css_set(cssProvider1, oneTime);
	css_set(cssProvider1, findVlc);
	css_set(cssProvider1, Playerx);
	css_set(cssProvider1, gPlayer);
	css_set(cssProvider1, Audsys);
	css_set(cssProvider1, orgPlay);
	css_set(cssProvider1, up1);
	css_set(cssProvider1, up2);
	css_set(cssProvider1, up3);
	css_set(cssProvider1, up4);
	css_set(cssProvider1, up5);
	css_set(cssProvider1, up6);
	css_set(cssProvider1, up7);
	css_set(cssProvider1, up8);
	css_set(cssProvider1, up9);
	css_set(cssProvider1, up10);
	css_set(cssProvider1, up11);
	css_set(cssProvider1, up12);
	css_set(cssProvider1, check1);
	css_set(cssProvider1, check2);
	css_set(cssProvider1, check3);
	css_set(cssProvider1, check4);
	css_set(cssProvider1, check5);
	css_set(cssProvider1, check6);
	css_set(cssProvider1, check7);
	css_set(cssProvider1, check8);
	css_set(cssProvider1, check9);
	css_set(cssProvider1, check10);
	css_set(cssProvider1, check11);
	css_set(cssProvider1, check12);
	css_set(cssProvider1, aud1);
	css_set(cssProvider1, aud2);
	css_set(cssProvider1, del1);
	css_set(cssProvider1, del2);
	css_set(cssProvider1, del3);
	css_set(cssProvider1, del4);
	css_set(cssProvider1, del7);
	css_set(cssProvider1, del8);
	css_set(cssProvider1, del9);
	css_set(cssProvider1, del10);
	css_set(cssProvider1, del11);
	css_set(cssProvider1, del12);
	css_set(cssProvider1, del13);
	css_set(cssProvider1, del14);
	css_set(cssProvider1, del1);
	css_set(cssProvider1, del2);
	css_set(cssProvider1, del3);
	css_set(cssProvider1, del4);
	css_set(cssProvider1, save1);
	css_set(cssProvider1, sndr1);
	css_set(cssProvider1, sndr2);
	css_set(cssProvider1, sndr3);
	css_set(cssProvider1, sndr4);
	css_set(cssProvider1, sndr5);
	css_set(cssProvider1, sndr6);
	css_set(cssProvider1, sndr7);
	css_set(cssProvider1, sndr8);
	css_set(cssProvider1, sndr9);
	css_set(cssProvider1, sndr10);
	css_set(cssProvider1, sndr11);
	css_set(cssProvider1, sndr12);
	css_set(cssProvider1, sndr13);
	css_set(cssProvider1, sndr14);
	css_set(cssProvider1, sndr15);
	css_set(cssProvider1, sndr16);
	css_set(cssProvider1, sndr17);
	css_set(cssProvider1, sndr18);
	css_set(cssProvider1, sndr19);
	css_set(cssProvider1, sndr20);
	css_set(cssProvider1, g_rnd_btn1);
	css_set(cssProvider1, g_rnd_btn2);
	css_set(cssProvider1, g_button1);
	css_set(cssProvider1, g_button2);
	css_set(cssProvider1, g_button3);
	css_set(cssProvider1, g_button4);
	css_set(cssProvider1, g_button5);
	css_set(cssProvider1, g_button6);
	css_set(cssProvider1, g_button7);
	css_set(cssProvider1, g_button8);
	css_set(cssProvider1, g_button9);
	css_set(cssProvider1, g_button10);
	css_set(cssProvider1, g_button11);
	css_set(cssProvider1, g_button12);
	css_set(cssProvider1, g_button13);
	css_set(cssProvider1, g_button14);
	css_set(cssProvider1, g_button15);
	css_set(cssProvider1, button16);
	css_set(cssProvider1, g_button18);
	css_set(cssProvider1, g_button30);
	css_set(cssProvider1, g_button31);
	css_set(cssProvider1, g_button32);
	css_set(cssProvider1, g_button33);
	css_set(cssProvider1, g_button34);
	css_set(cssProvider1, g_button35);
	css_set(cssProvider1, g_button37);
	css_set(cssProvider1, g_button52);
	css_set(cssProvider1, g_button57);
	css_set(cssProvider1, g_button58);
	css_set(cssProvider1, g_btn_hello);
	css_set(cssProvider1, g_btn_hello1);
	css_set(cssProvider1, scroll);
	}

void	on_aud1_toggled(GtkToggleButton *b) {
	*AudioSystem=PULSE;
	}

void	on_aud2_toggled(GtkToggleButton *b) {

	*AudioSystem=JACK;
	}

void	AudioSubSys(char * cmd) {

	int i;

	if ( (i=WhatAudio()) != *audsys) {
		Message("Audio sub-system requested not available\n"
		"You asked for Pulse or Jack and the other was actually running");
		printf("*** Audio system mismatch: actual=%d requested=%d - exiting\n",
			i, *audsys);
		on_window_main_destroy();
		return;
		}

	if (*audsys == PULSE) {

		if (*player == MPV_PLAYER) 
			strcat (cmd, " --ao=pulse ");
		else  // VLC_PLAYER
			strcat(cmd, " --aout=pulse ");
		}

	else { // JACK
		if (*player == MPV_PLAYER) 
			strcat (cmd, " --ao=jack --jack-connect=yes --jack-port=jamin ");
		else  // VLC_PLAYER
			strcat(cmd, " --aout=jack --jack-auto-connect ");
		}
	}

int	WhatAudio() {

	char cmd[1024];
	FILE *f1;

	strcpy(cmd, "ps -e | grep jack ");
	f1 = popen(cmd,"r");

	if (f1 == NULL ) { 
		printf("*** unable to determine audio system\n");
		on_window_main_destroy();
		return 999;
		}

	strcpy(cmd, "");
	fgets(cmd,1024,f1);
	fclose(f1);
	if (strlen(cmd) > 2 ) return JACK;
	else return PULSE;
	}

void	TimeOfNext() {
	time_t t,t1;
	char dmy[16];

	t = time(0);
	t1 = t + *Clock;

	sscanf(ctime(&t1),"%s %s %s %s ",dmy,dmy,dmy,dmy);

	strcpy(TimeNext, "Next: ");
	strcat(TimeNext, dmy);

	return;
	}

void	ClearTimeOfNext() {
	strcpy(TimeNext,"Time of next");
	}

#ifdef WINDOWS

//	callback connections for Windows version.

void	WindowsCallBacks() {

	g_signal_connect(g_btn_hello1,"clicked",G_CALLBACK(on_btn_hello1_clicked),NULL);
	g_signal_connect(g_btn_hello,"clicked",G_CALLBACK(on_btn_hello_clicked),NULL);
	g_signal_connect(g_rnd_btn1,"clicked",G_CALLBACK(on_rnd_btn1_clicked),NULL);
	g_signal_connect(g_rnd_btn2,"clicked",G_CALLBACK(on_rnd_btn2_clicked),NULL);

	g_signal_connect(g_button1,"clicked",G_CALLBACK(on_button1_clicked),NULL);
	g_signal_connect(g_button4,"clicked",G_CALLBACK(on_button4_clicked),NULL);
	g_signal_connect(g_button7,"clicked",G_CALLBACK(on_button7_clicked),NULL);
	g_signal_connect(g_button18,"clicked",G_CALLBACK(on_button18_clicked),NULL);
	g_signal_connect(g_button30,"clicked",G_CALLBACK(on_button30_clicked),NULL);
	g_signal_connect(g_button31,"clicked",G_CALLBACK(on_button31_clicked),NULL);
	g_signal_connect(g_button32,"clicked",G_CALLBACK(on_button32_clicked),NULL);
	g_signal_connect(g_button33,"clicked",G_CALLBACK(on_button33_clicked),NULL);
	g_signal_connect(g_button34,"clicked",G_CALLBACK(on_button34_clicked),NULL);
	g_signal_connect(g_button35,"clicked",G_CALLBACK(on_button35_clicked),NULL);
	g_signal_connect(g_button37,"clicked",G_CALLBACK(on_button37_clicked),NULL);
	g_signal_connect(g_button52,"clicked",G_CALLBACK(on_button52_clicked),NULL);
	g_signal_connect(g_button57,"clicked",G_CALLBACK(on_button57_clicked),NULL);
	g_signal_connect(g_button58,"clicked",G_CALLBACK(on_button58_clicked),NULL);

	g_signal_connect(sndr1,"clicked",G_CALLBACK(on_sndr1_clicked),NULL);
	g_signal_connect(sndr2,"clicked",G_CALLBACK(on_sndr2_clicked),NULL);
	g_signal_connect(sndr3,"clicked",G_CALLBACK(on_sndr3_clicked),NULL);
	g_signal_connect(sndr4,"clicked",G_CALLBACK(on_sndr4_clicked),NULL);
	g_signal_connect(sndr5,"clicked",G_CALLBACK(on_sndr5_clicked),NULL);
	g_signal_connect(sndr6,"clicked",G_CALLBACK(on_sndr6_clicked),NULL);
	g_signal_connect(sndr7,"clicked",G_CALLBACK(on_sndr7_clicked),NULL);
	g_signal_connect(sndr8,"clicked",G_CALLBACK(on_sndr8_clicked),NULL);
	g_signal_connect(sndr9,"clicked",G_CALLBACK(on_sndr9_clicked),NULL);
	g_signal_connect(sndr10,"clicked",G_CALLBACK(on_sndr10_clicked),NULL);

	g_signal_connect(del1,"clicked",G_CALLBACK(on_del1_clicked),NULL);
	g_signal_connect(del2,"clicked",G_CALLBACK(on_del2_clicked),NULL);
	g_signal_connect(del3,"clicked",G_CALLBACK(on_del3_clicked),NULL);
	g_signal_connect(del4,"clicked",G_CALLBACK(on_del4_clicked),NULL);
	g_signal_connect(del7,"clicked",G_CALLBACK(on_del7_clicked),NULL);
	g_signal_connect(del8,"clicked",G_CALLBACK(on_del8_clicked),NULL);
	g_signal_connect(del9,"clicked",G_CALLBACK(on_del9_clicked),NULL);
	g_signal_connect(del10,"clicked",G_CALLBACK(on_del10_clicked),NULL);
	g_signal_connect(del11,"clicked",G_CALLBACK(on_del11_clicked),NULL);
	g_signal_connect(del12,"clicked",G_CALLBACK(on_del12_clicked),NULL);
	g_signal_connect(del13,"clicked",G_CALLBACK(on_del13_clicked),NULL);
	g_signal_connect(del14,"clicked",G_CALLBACK(on_del14_clicked),NULL);

	g_signal_connect(dxb1,"clicked",G_CALLBACK(on_dxb1_clicked),NULL);
	g_signal_connect(dxb2,"clicked",G_CALLBACK(on_dxb2_clicked),NULL);

	g_signal_connect(up1,"clicked",G_CALLBACK(on_up1_clicked),NULL);
	g_signal_connect(up2,"clicked",G_CALLBACK(on_up2_clicked),NULL);
	g_signal_connect(up3,"clicked",G_CALLBACK(on_up3_clicked),NULL);
	g_signal_connect(up4,"clicked",G_CALLBACK(on_up4_clicked),NULL);
	g_signal_connect(up5,"clicked",G_CALLBACK(on_up5_clicked),NULL);
	g_signal_connect(up6,"clicked",G_CALLBACK(on_up6_clicked),NULL);
	g_signal_connect(up7,"clicked",G_CALLBACK(on_up7_clicked),NULL);
	g_signal_connect(up8,"clicked",G_CALLBACK(on_up8_clicked),NULL);
	g_signal_connect(up9,"clicked",G_CALLBACK(on_up9_clicked),NULL);
	g_signal_connect(up10,"clicked",G_CALLBACK(on_up10_clicked),NULL);
	g_signal_connect(up11,"clicked",G_CALLBACK(on_up11_clicked),NULL);
	g_signal_connect(up12,"clicked",G_CALLBACK(on_up12_clicked),NULL);

	g_signal_connect(oneTime,"clicked",G_CALLBACK(on_oneTime_clicked),NULL);
	g_signal_connect(showCurrent,"clicked",G_CALLBACK(on_showCurrent_clicked),NULL);
	g_signal_connect(clrPgm,"clicked",G_CALLBACK(on_clrPgm_clicked),NULL);
	g_signal_connect(player,"clicked",G_CALLBACK(on_player_clicked),NULL);
	g_signal_connect(player,"Audsys",G_CALLBACK(on_Audsys_clicked),NULL);
	g_signal_connect(player,"findVlc",G_CALLBACK(on_findVlc_clicked),NULL);
	g_signal_connect(player,"Playerx",G_CALLBACK(on_Playerx_clicked),NULL);
	g_signal_connect(player,"orgPlay",G_CALLBACK(on_orgPlay_clicked),NULL);

	g_signal_connect(player,"orgPlay",G_CALLBACK(on_orgPlay_clicked),NULL);

//	SOUNDER macro callbacks

	g_signal_connect(g_button2,"clicked",G_CALLBACK(on_button2_clicked),NULL);
	g_signal_connect(g_button3,"clicked",G_CALLBACK(on_button3_clicked),NULL);
	g_signal_connect(g_button5,"clicked",G_CALLBACK(on_button5_clicked),NULL);
	g_signal_connect(g_button6,"clicked",G_CALLBACK(on_button6_clicked),NULL);
	g_signal_connect(g_button8,"clicked",G_CALLBACK(on_button8_clicked),NULL);
	g_signal_connect(g_button9,"clicked",G_CALLBACK(on_button9_clicked),NULL);
	g_signal_connect(g_button10,"clicked",G_CALLBACK(on_button10_clicked),NULL);
	g_signal_connect(g_button11,"clicked",G_CALLBACK(on_button11_clicked),NULL);
	g_signal_connect(g_button12,"clicked",G_CALLBACK(on_button12_clicked),NULL);
	g_signal_connect(g_button13,"clicked",G_CALLBACK(on_button13_clicked),NULL);
	g_signal_connect(g_button14,"clicked",G_CALLBACK(on_button14_clicked),NULL);
	g_signal_connect(g_button15,"clicked",G_CALLBACK(on_button15_clicked),NULL);

	}
#endif

void	LoadProgram(char * file) {

	FILE *s1;
	char cmd[1024];

	for (int i=0; i<SOUNDER_MAX; i++) {
		strcpy(sounder[i], "");
		strcpy(sounderTime[i]->Time, "");
		sounderTime[i]->play = 0;
		}

//----------------------------------------------------------------
//	load program entries
//----------------------------------------------------------------

	s1 = fopen(file,"r");

	if (s1==NULL) {		// no program.lst file
		*scount = 0;	// sounder list is empty
		}

	else {			// program.lst exists
	
		char tmp[2048],tme[32];
		int i;
		*scount=0;

		while (1) {

			tmp[0] = 0;

			if (fgets(tmp,1024,s1) == NULL) break; // read program.lst line

			tmp[strlen(tmp)-1] = 0;

			for (i=0; tmp[i] != 0 && tmp[i] != '#'; i++); // find end of time stamp

			if (tmp[i] == 0 ) {
				strcpy(tme,"0");
				}

			else {
				tmp[i]=0;
				strcpy(tme,tmp);	// extract time stamp
				strcpy(cmd,&tmp[i+1]);	// remove time stamp from file name
				strcpy(tmp,cmd);
				}

			if (tmp[1] != '#' ) {
				sounderTime[*scount]->play = 1;
				strcpy(cmd,tmp);
				}
			else {
				if (tmp[0] == '0' ) sounderTime[*scount]->play = 0;
				else sounderTime[*scount]->play = 1;
				strcpy(cmd,&tmp[2]);
				}
	
			if (filetype(cmd) == FALSE || strstr(cmd,"cover.jpg") ) {
				printf("*** File type not recognized/accepted: %s\n",cmd);
				continue;
				}

			if ( ! (strncmp(cmd,"https",5) == 0 ||
				strncmp(cmd,"http",4) == 0) )
				if ( access(cmd, R_OK) ) {
					printf("*** File not found/unreadable: **%s**\n",cmd);
					continue; // does not exist
					}

			if (strlen(cmd) >= PGM_MAX) {
				printf("*** File reference too long: **%s**\n",cmd);
				continue; // does not exist
				}

			strcpy(sounder[*scount],cmd);	// record file name

			strcpy(sounderTime[*scount]->Time,tme);	// record time stamp

			(*scount)++;
			if (*scount>SOUNDER_MAX-20) { 
				(*scount)--; 
				printf("*** More that SOUNDER_MAX lines in program.lst \n");
				break;
				}
			}

		i = *scount;

		while (i < SOUNDER_MAX) { // finish list

			strcpy(sounderTime[i]->Time,"");
			sounderTime[i]->play = 1; // play it?
			strcpy(sounder[i++],"");
			}

		fclose (s1);
		printf("*** program.lst loader\n");
		}

	// scount is count of actual entries
	// SOUNDER_MAX is size of array

	printf("*scount=%d SOUNDER_MAX=%d\n",*scount,SOUNDER_MAX);
	gtk_range_set_range (GTK_RANGE(scroll),0,*scount);
	printf("\n");

	}

void	GetTime (int Row, char *file) {

	FILE *f1;
	char *p1,tmp[1024];

	strcpy(tmp,"ffprobe -v quiet -of csv=p=0 -show_entries format=duration \"");
	strcat(tmp, file);
	strcat(tmp, "\"");
	f1=popen(tmp, "r"); strcpy(tmp, ""); fgets(tmp, 256, f1); fclose(f1);
	p1=strstr(tmp, ".");
	if (p1 != NULL) *p1=0;
	else { strcpy(tmp, "");}

	strcpy(sounderTime[Row]->Time, tmp);
	}

void	on_scroll_value_changed(GtkRange *r) {
	int k;
	gdouble x = gtk_range_get_value (r);
	k = x; // make integer
	*begin = k;
	sounders();
	}

gboolean	scroll_event1(GtkScrollbar *b, GdkEventScroll *e) {

		// see GdkEventScroll data structure for valid elements
		// see: gtk_widget_set_events(g_buttonxx,GDK_SCROLL_MASK);

		if (e->delta_y == 1) { // down
			*begin=*begin + 1;
			if (*begin + SOUNDER_BUTTON_COUNT >= *scount) { *begin = *scount - SOUNDER_BUTTON_COUNT; }	
			if (*begin < 0 ) *begin = 0;
			*NowPlaying = -1;
			sounders();
			}
		else { // up
			*begin=*begin - 1;
			if (*begin + SOUNDER_BUTTON_COUNT >= *scount) { *begin = *scount - SOUNDER_BUTTON_COUNT; }	
			if (*begin < 0 ) *begin = 0;
			*NowPlaying = -1;
			sounders();
			}
		gtk_range_set_value (GTK_RANGE(b), *begin); // place slider
		return TRUE;
		}

gboolean	on_scroll_scroll_event(GtkScrollbar *b, GdkEventScroll *e) {
		return scroll_event1(b,e);
		}

gboolean on_Playerx_button_press_event (GtkButton *btn, GdkEventButton *event, 
		gpointer user_data) { 

//	uncheck all items in program list

	if (event->button == 1 ) return FALSE;
	for (int i=0; i< SOUNDER_MAX; i++) sounderTime[i]->play = 0;
	sounders();
	return TRUE;
	}

void on_button16_clicked (GtkButton *button, GtkEntry *user_data) {
	}

gboolean	on_button16(GtkButton *b, GdkEventScroll *e) {

		// see GdkEventScroll data structure for valid elements
		// see: gtk_widget_set_events(g_buttonxx,GDK_SCROLL_MASK);

		if (e->direction == 1) { // down
			*begin=*begin + 1;
			if (*begin + SOUNDER_BUTTON_COUNT >= *scount) { *begin = *scount - SOUNDER_BUTTON_COUNT; }	
			if (*begin < 0 ) *begin = 0;
			*NowPlaying = -1;
			sounders();
			}
		else { // up
			*begin=*begin - 1;
			if (*begin + SOUNDER_BUTTON_COUNT >= *scount) { *begin = *scount - SOUNDER_BUTTON_COUNT; }	
			if (*begin < 0 ) *begin = 0;
			*NowPlaying = -1;
			sounders();
			}
		return TRUE;
		}

void    ClearPlayers() {
        ClearTimeOfNext();
        system("killall --signal SIGTERM vlc 2> /dev/null");
        system("killall --signal SIGTERM mpv 2> /dev/null");
        system(mpvFile);
        strcpy(next_random, "Next Program Selection");
        strcpy(currentPlay, "Curr Program Selection");
        }

int     MediaProcess(int *i, char * cmd) {

        int j, itype;

        itype=filetype(sounder[*i]);     // what is the file type?

        *audio = 0;     // audio only file format flag

        if (itype==MP3 || itype==WAV || itype==OGG || itype==FLAC || itype==WEB) {
                if (*player == MPV_PLAYER ) MPV_PLAY_AUDIO
                else VLC_PLAY_AUDIO
                *audio = 1; // this is an audio only file format
                }
        else {  // not an audio only file format
                if (*player == MPV_PLAYER ) MPV_PLAY
                else VLC_PLAY
                }

        if (itype == PNG || itype == JPG)
                strcat(cmd,"--input-repeat 100 ");      // image file

        AudioSubSys(cmd); // adds parameters for Pulse and JACK
        if (*i < 0 ) return EXIT_SUCCESS;

        strcat(cmd,"\"");
        strcat(cmd,sounder[*i]);

        for (j = strlen(sounder[*i]); j >= 0 && sounder[*i][j] != '/'; j--);
        sprintf(currentPlay,"Curr: %d %s", *i, &sounder[*i][j+1]);

        if (*player == MPV_PLAYER ) {
                strcat(cmd, "\" 2>&1 > /dev/null | cat > ");
                strcat(cmd, Code_Home);
                strcat(cmd, "/mpvTmp &");
                }
        else strcat(cmd,"\" &");
        return EXIT_SUCCESS;
        }

